@@ -4,7 +4,7 @@
 #if XtSpecificationRelease <= 4
 
 #  define XrmPermStringToQuark(string)	XrmStringToQuark(string)
-#  define XrmScreenDatabase(screen)	XtDatabase(DisplayOfScreen(screen))
+#  define XtScreenDatabase(screen)	XtDatabase(DisplayOfScreen(screen))
 
 typedef char *XPointer;
 
