@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #include <stdlib.h>
 #include <stdio.h>
@@ -26,45 +26,48 @@
 #include "Knapp.h"
 #include "Layout.h"
 #include "Message.h"
+#include "Scrollable.h"
 #include "ScrBar.h"
 #include "ScrList.h"
+#include "Toggle.h"
 #include "TextField.h"
-#include "FileSelP.h"
 #include "Util.h"
 
+#include "FileSelP.h"
+
 /* a few dirty hacks */
 
-#ifndef MAXPATHLEN
-#define MAXPATHLEN 1024
+#if !defined(MAXPATHLEN) || (MAXPATHLEN < 1024)
+#  undef  MAXPATHLEN
+#  define MAXPATHLEN 1024
 #endif
 
 static XtResource resources[] = {
+    {XtNinput, XtCInput, XtRBool, sizeof(Bool),
+     XtOffsetOf(FileSelRec, wm.wm_hints.input), XtRImmediate, (XtPointer)True},
+    {XtNallowShellResize, XtCAllowShellResize, XtRBoolean, sizeof(Boolean),
+     XtOffsetOf(FileSelRec, shell.allow_shell_resize),
+     XtRImmediate, (XtPointer)True},
 #define offset(field) XtOffsetOf(FileSelRec, filesel.field)
-    {XtNchooseCallback, XtCCallback, XtRCallback, sizeof(XtCallbackList),
-     offset(choose_callback), XtRCallback, (XtPointer)NULL},
-    {XtNcancelCallback, XtCCallback, XtRCallback, sizeof(XtCallbackList),
-     offset(cancel_callback), XtRCallback, (XtPointer)NULL},
+    {XtNcallback, XtCCallback, XtRCallback, sizeof(XtCallbackList),
+     offset(callback), XtRCallback, (XtPointer)NULL},
     {XtNcursor, XtCCursor, XtRCursor, sizeof(Cursor),
      offset(cursor), XtRString, (XtPointer)"top_left_arrow"},
     {XtNbusyCursor, XtCCursor, XtRCursor, sizeof(Cursor),
      offset(busy_cursor), XtRString, (XtPointer)"watch"},
     {XtNdirectory, XtCDirectory, XtRString, sizeof(String),
      offset(directory), XtRCallProc, NULL},
-    {XtNstayUp, XtCStayUp, XtRBoolean, sizeof(Boolean),
-     offset(stay_up), XtRImmediate, (XtPointer)False},
-    {XtNinput, XtCInput, XtRBool, sizeof(Bool),
-     XtOffsetOf(FileSelRec, wm.wm_hints.input), XtRImmediate, (XtPointer)True},
     {XtNpreferredColumns, XtCPreferredColumns, XtRDimension, sizeof(Dimension),
      offset(pref_cols), XtRImmediate, (XtPointer)16},
+    {XtNshowDotFiles, XtCShowDotFiles, XtRBoolean, sizeof(Boolean),
+     offset(show_dot_files), XtRImmediate, (XtPointer)True},
 #undef offset
 };
 
-static void Initialize(Widget, Widget, ArgList, Cardinal*);
-static void Destroy(Widget);
-static void Realize(Widget, XtValueMask*, XSetWindowAttributes*);
-static Boolean AcceptFocus(Widget, Time*);
-static Boolean SetValues(Widget, Widget, Widget, ArgList, Cardinal*);
-static XtGeometryResult QueryGeometry(Widget, XtWidgetGeometry*, XtWidgetGeometry*);
+static void		Initialize(Widget, Widget, ArgList, Cardinal*);
+static void		Destroy(Widget);
+static void		Realize(Widget, XtValueMask*, XSetWindowAttributes*);
+static Boolean		SetValues(Widget, Widget, Widget, ArgList, Cardinal*);
 
 static void close_filesel(Widget, XEvent*, String*, Cardinal*);
 
@@ -102,11 +105,11 @@ FileSelClassRec fileSelClassRec = {
         NULL,                           /* set_values_hook              */
         XtInheritSetValuesAlmost,       /* set_values_almost            */
         NULL,                           /* get_values_hook              */
-        AcceptFocus,                    /* accept_focus                 */
+        NULL,				/* accept_focus                 */
         XtVersion,                      /* version                      */
         NULL,                           /* callback_private             */
         translations,                   /* tm_table                     */
-        QueryGeometry,                  /* query_geometry               */
+        XtInheritQueryGeometry,		/* query_geometry               */
         XtInheritDisplayAccelerator,    /* display_accelerator          */
         NULL                            /* extension                    */
     },
@@ -194,20 +197,20 @@ static void init_pixmaps(FileSelWidget w)
 	    XCreateBitmapFromData(disp, root, (char *)icon_bits[i], 16, 16);
 }
 
-static char layout_string[] =
+static const char *layout_string =
 "vertical {"
 "	height choose "
 "	horizontal { "
-"		height choose "
+"		2 "
 "		vertical { "
 "			0 <+inf> "
-"			dirlabel "
+"			directory "
 "			0 <+inf> "
 "		} "
 "		dirfield <+inf-inf*> "
-"		height choose "
+"		4 + width scrbar "
 "	} "
-"	height choose "
+"	(height choose / 2) "
 "	horizontal { "
 "		2 "
 "		list <+inf-inf*+inf-inf> "
@@ -215,25 +218,27 @@ static char layout_string[] =
 "		scrbar <*+inf> "
 "		2 "
 "	} "
-"	height choose "
+"	(height choose / 2) "
 "	horizontal { "
-"		height choose "
+"		2 "
 "		vertical { "
 "			0 <+inf> "
-"			filelabel "
+"			file "
 "			0 <+inf> "
 "		} "
-"		(width dirlabel) - (width filelabel) "
+"		(width directory - width file) "
 "		filefield <+inf-inf*> "
-"		height choose "
+"		4 + width scrbar "
 "	} "
 "	height choose "
 "	horizontal { "
-"		(2 * height choose) "
+"		2 <+inf-inf>"
 "		choose "
-"		(4 * height choose) <+inf> "
+"		2 <+inf-inf> "
+"		dotfiles "
+"		2 <+inf-inf> "
 "		cancel "
-"		(2 * height choose) "
+"		2 <+inf-inf> "
 "	} "
 "	height choose "
 "} ";
@@ -250,7 +255,7 @@ static char layout_string[] =
 #  if defined(S_IFFIFO) && defined(S_IFMT)
 #    define S_ISFIFO(mode) (((mode) & S_IFMT) == S_IFFIFO)
 #  else
-#    define S_ISFIFO(mode) False
+#    define S_ISFIFO(mode) 0
 #  endif
 #endif
 
@@ -288,10 +293,13 @@ static int change_dir(FileSelWidget w, char *dir)
     DIR			*dirp;
     struct dirent	*dp;
     long		i, n, n_alloc, len;
+    Arg			arg;
 
-    if (dir) {
+    if (dir)
 	if (*dir == '/')
 	    new_dir = XtNewString(dir);
+	else if (*dir == '\0')
+	    new_dir = XtNewString(w->filesel.directory);
 	else {
 	    new_dir =
 		XtMalloc(strlen(w->filesel.directory) + strlen(dir) + 8);
@@ -300,35 +308,35 @@ static int change_dir(FileSelWidget w, char *dir)
 	    else
 		sprintf(new_dir, "%s/%s", w->filesel.directory, dir);
 	}
-    } else {
-	if (strcmp(w->filesel.directory, "/") == 0)
-	    new_dir = XtNewString("/");
-	else {
-	    char	*c;
+    else if (strcmp(w->filesel.directory, "/") == 0)
+	new_dir = XtNewString("/");
+    else {
+	char	*c;
 
-	    new_dir = XtNewString(w->filesel.directory);
-	    len = strlen(new_dir);
-	    if (new_dir[len-1] == '/') {
-		new_dir[len-1] = '\0';
-		len--;
-	    }
-
-	    if ( (c = strrchr(new_dir, '/')) ) {
-		if (c == new_dir)
-		    new_dir[1] = '\0';
-		else
-		    *c = '\0';
-	    }
+	new_dir = XtNewString(w->filesel.directory);
+	len = strlen(new_dir);
+	if (new_dir[len-1] == '/') {
+	    new_dir[len-1] = '\0';
+	    len--;
 	}
+
+	c = strrchr(new_dir, '/');
+	if (c)
+	    if (c == new_dir)
+		new_dir[1] = '\0';
+	    else
+		*c = '\0';
     }
 
     if (stat(new_dir, &stat_buf) < 0) {
+	perror(new_dir);
 	XtFree(new_dir);
 	return -1;
     }
 
     dirp = opendir(new_dir);
     if (!dirp) {
+	perror(new_dir);
 	XtFree(new_dir);
 	return -1;
     }
@@ -347,7 +355,9 @@ static int change_dir(FileSelWidget w, char *dir)
 				      n_alloc * sizeof(char *));
 	}
 
-	if (dp->d_name[0] != '.' || dp->d_name[1] != '\0') {
+	if (dp->d_name[0] != '.' ||
+	    (dp->d_name[1] == '.' && dp->d_name[2] == '\0') ||
+	    (w->filesel.show_dot_files && dp->d_name[1] != '\0')) {
 	    list[n] = XtNewString(dp->d_name);
 	    n++;
 	}
@@ -361,8 +371,8 @@ static int change_dir(FileSelWidget w, char *dir)
     len = strlen(buffer);
     w->filesel.types = (unsigned char *)XtRealloc((char *)w->filesel.types, n);
 
-    ScrBarSetLengthsAndPos(w->filesel.scrbar, 0, 0, 0);
     ScrListClearLines(w->filesel.list);
+    ScrollableSuspend(w->filesel.list);
 
     for (i = 0 ; i < n ; i++) {
 	FileType	type;
@@ -380,10 +390,12 @@ static int change_dir(FileSelWidget w, char *dir)
 	XtFree(list[i]);
     }
 
-    ScrListGetFirstShownSize(w->filesel.list, NULL, &i, &n);
-    ScrBarSetLengthsAndPos(w->filesel.scrbar, n, i, 0);
-    XtVaSetValues(w->filesel.dir_field, XtNbuffer, w->filesel.directory, NULL);
-    XtVaSetValues(w->filesel.file_field, XtNbuffer, "", NULL);
+    ScrollableResume(w->filesel.list);
+
+    XtSetArg(arg, XtNbuffer, w->filesel.directory);
+    XtSetValues(w->filesel.dir_field, &arg, 1);
+    XtSetArg(arg, XtNbuffer, "");
+    XtSetValues(w->filesel.file_field, &arg, 1);
 
     if (XtIsRealized((Widget)w)) {
 	XDefineCursor(XtDisplay(w), XtWindow(w), w->filesel.cursor);
@@ -393,20 +405,26 @@ static int change_dir(FileSelWidget w, char *dir)
     return 0;
 }
 
-static void call_choose_callbacks(FileSelWidget w, long row)
+static void call_callbacks(FileSelWidget w, long row)
 {
-    XtCallbackList	c_list = w->filesel.choose_callback;
-    char		*file;
+    XtCallbackList	c_list = w->filesel.callback;
+    char		*file = "";
     char		*call_data;
     long		len;
+    int			do_free = False;
 
-    if (row < 0)
-	XtVaGetValues(w->filesel.file_field, XtNbuffer, &file, NULL);
-    else
+    if (row >= 0)
 	file = ScrListGetString(w->filesel.list, row);
+    else {
+	file = TextFieldGetBuffer(w->filesel.file_field);
+	do_free = True;
+    }
 
-    if (!c_list || !file || !w->filesel.directory)
+    if (!c_list || !file || !w->filesel.directory) {
+	if (do_free)
+	    XtFree(file);
 	return;
+    }
 
     len = strlen(file) + strlen(w->filesel.directory) + 8;
     call_data = XtMalloc(len);
@@ -417,8 +435,8 @@ static void call_choose_callbacks(FileSelWidget w, long row)
 
     XtCallCallbackList((Widget)w, c_list, (XtPointer)call_data);
     XtFree(call_data);
-
-    if (!w->filesel.stay_up) XtPopdown((Widget)w);
+    if (do_free)
+	XtFree(file);
 }
 
 static void dir_field_callback(Widget gw,
@@ -437,9 +455,8 @@ static void file_field_callback(Widget gw,
 {
     FileSelWidget	w = (FileSelWidget)client_data;
     char		*buffer = (char *)call_data;
-    long		n;
+    long		n = ScrollableGetVSize(w->filesel.list);
 
-    ScrListGetFirstShownSize(w->filesel.list, NULL, NULL, &n);
     if (buffer) {
 	long	i;
 
@@ -458,13 +475,13 @@ static void file_field_callback(Widget gw,
 		    if (change_dir(w, NULL) < 0)
 			XBell(XtDisplay(w), 0);
 		} else
-		    call_choose_callbacks(w, i);
+		    call_callbacks(w, i);
 		return;
 	    }
 	}
     }
 
-    call_choose_callbacks(w, -1);
+    call_callbacks(w, -1);
 }
 
 static void file_field_tab_callback(Widget gw,
@@ -474,10 +491,9 @@ static void file_field_tab_callback(Widget gw,
     FileSelWidget	w = (FileSelWidget)client_data;
     char		*buffer = (char *)call_data;
     char		*c = NULL;
-    long		n;
+    long		n = ScrollableGetVSize(w->filesel.list);
     Arg			arg;
 
-    ScrListGetFirstShownSize(w->filesel.list, NULL, NULL, &n);
     if (n > 0 && buffer) {
 	long	len = strlen(buffer);
 	long	first = 0;
@@ -531,16 +547,6 @@ static void file_field_tab_callback(Widget gw,
     }
 }
 
-static void scrbar_callback(Widget gw,
-			    XtPointer client_data,
-			    XtPointer call_data)
-{
-    FileSelWidget	w = (FileSelWidget)client_data;
-    ScrBarReport	*report = (ScrBarReport *)call_data;
-
-    ScrListSetFirst(w->filesel.list, report->slider_position);
-}
-
 static void list_callback(Widget gw,
 			  XtPointer client_data,
 			  XtPointer call_data)
@@ -557,7 +563,7 @@ static void list_callback(Widget gw,
 	if (change_dir(w, NULL) < 0)
 	    XBell(XtDisplay(w), 0);
     } else
-	call_choose_callbacks(w, row);
+	call_callbacks(w, row);
 }
 
 static void list_select_callback(Widget gw,
@@ -571,20 +577,6 @@ static void list_select_callback(Widget gw,
 
     XtSetArg(arg, XtNbuffer, file);
     XtSetValues(w->filesel.file_field, &arg, 1);
-
-}
-
-static void list_resize_callback(Widget gw,
-				 XtPointer client_data,
-				 XtPointer call_data)
-{
-    FileSelWidget	w = (FileSelWidget)client_data;    
-    ScrListReport	*report = (ScrListReport *)call_data;
-
-    ScrBarSetLengthsAndPos(w->filesel.scrbar,
-			   report->canvas_length,
-			   report->slider_length,
-			   report->slider_position);
 }
 
 static void cancel_callback(Widget gw,
@@ -592,8 +584,10 @@ static void cancel_callback(Widget gw,
 			    XtPointer call_data)
 {
     FileSelWidget	w = (FileSelWidget)client_data;
+    XtCallbackList	c_list = w->filesel.callback;
 
-    XtPopdown((Widget)w);
+    if (c_list)
+	XtCallCallbackList((Widget)w, c_list, NULL);
 }
 
 static void choose_callback(Widget gw,
@@ -602,7 +596,21 @@ static void choose_callback(Widget gw,
 {
     FileSelWidget	w = (FileSelWidget)client_data;
 
-    call_choose_callbacks(w, -1);
+    call_callbacks(w, -1);
+}
+
+static void show_callback(Widget gw,
+			  XtPointer client_data,
+			  XtPointer call_data)
+{
+    FileSelWidget	w = (FileSelWidget)client_data;
+    Boolean		*set = (Boolean *)call_data;
+
+    if (!set)
+	return;
+
+    *set = w->filesel.show_dot_files = !w->filesel.show_dot_files;
+    change_dir(w, "");
 }
 
 static void popup_callback(Widget gw,
@@ -611,31 +619,38 @@ static void popup_callback(Widget gw,
 {
     FileSelWidget	w = (FileSelWidget)gw;
     long		n;
+    Arg			arg;
 
-    XtVaSetValues(w->filesel.dir_field, XtNbuffer, w->filesel.directory, NULL);
-    XtVaSetValues(w->filesel.file_field, XtNbuffer, "", NULL);
-    XtVaSetValues(w->filesel.scrbar, XtNsliderPosition, 0, NULL);
-    ScrListSetFirst(w->filesel.list, 0);
+    XtSetArg(arg, XtNbuffer, w->filesel.directory);
+    XtSetValues(w->filesel.dir_field, &arg, 1);
+    XtSetArg(arg, XtNbuffer, "");
+    XtSetValues(w->filesel.file_field, &arg, 1);
+    ScrollableSetVPos(w->filesel.list, 0);
     n = ScrListGetFirstSelected(w->filesel.list);
     if (n >= 0)
 	ScrListSetSelected(w->filesel.list, n, False);
 }
 
-static void close_filesel(Widget w, XEvent *event,
+static void close_filesel(Widget gw, XEvent *event,
 			  String *params, Cardinal *no_params)
 {
-    if (event->type == ClientMessage) {
-        Display	*disp = XtDisplay(w);
-        Atom	wm_delete_window =
-	    XInternAtom(disp, "WM_DELETE_WINDOW", True);
-        Atom	wm_protocols =
-	    XInternAtom(disp, "WM_PROTOCOLS", True);
+    FileSelWidget	w = (FileSelWidget)gw;
+    Display		*disp = XtDisplay(w);
+    Atom		wm_delete_window, wm_protocols;
+    XtCallbackList	c_list = w->filesel.callback;
 
-        if (event->xclient.message_type == wm_protocols &&
-            event->xclient.data.l[0] == wm_delete_window) {
-	    XtPopdown(w);
-        }
-    }
+    if (event->type != ClientMessage)
+	return;
+
+    wm_delete_window = intern_atom(disp, "WM_DELETE_WINDOW");
+    wm_protocols     = intern_atom(disp, "WM_PROTOCOLS");
+
+    if (event->xclient.message_type != wm_protocols ||
+	event->xclient.data.l[0]    != wm_delete_window)
+	return;
+
+    if (c_list)
+	XtCallCallbackList((Widget)w, c_list, NULL);
 }
 
 /*************************************************************************/
@@ -647,39 +662,42 @@ static void Initialize(Widget grequest, Widget gnew,
     Arg			arg[8];
     Pixel		pix;
     char		*home = getenv("HOME");
+    Widget		layout, temp;
 
     init_pixmaps(new);
-
     new->filesel.types = NULL;
-    new->filesel.layout =
+
+    XtAddCallback((Widget)new, XtNpopupCallback, popup_callback, NULL);
+
+    layout =
 	XtVaCreateManagedWidget("layout", layoutWidgetClass, (Widget)new,
 				XtVaTypedArg, XtNlayout, XtRString,
 				layout_string, sizeof(String),
-				NULL);
+				(void *)0);
 
     XtSetArg(arg[0], XtNbackground, &pix);
-    XtGetValues(new->filesel.layout, arg, 1);
+    XtGetValues(layout, arg, 1);
     XtSetArg(arg[0], XtNborderColor, pix);
     XtSetArg(arg[1], XtNpreferredChars, 20);
     XtSetArg(arg[2], XtNfocusRoot, new);
+    XtSetArg(arg[3], XtNsingleLine, True);
     new->filesel.dir_field =
 	XtCreateManagedWidget("dirfield", textFieldWidgetClass,
-			      new->filesel.layout, arg, 3);
+			      layout, arg, 4);
+    XtAddCallback(new->filesel.dir_field, XtNcallback,
+		  dir_field_callback, (XtPointer)new);
     new->filesel.file_field =
 	XtCreateManagedWidget("filefield", textFieldWidgetClass,
-			      new->filesel.layout, arg, 3);
+			      layout, arg, 4);
+    XtAddCallback(new->filesel.file_field, XtNcallback,
+		  file_field_callback, (XtPointer)new);
+    XtAddCallback(new->filesel.file_field, XtNtabCallback,
+		  file_field_tab_callback, (XtPointer)new);
 
-    XtSetArg(arg[0], XtNbuffer, "Directory:");
-    XtSetArg(arg[1], XtNborderWidth, 0);
-    new->filesel.dir_label =
-	XtCreateManagedWidget("dirlabel", messageWidgetClass,
-			      new->filesel.layout, arg, 2);
+    XtCreateManagedWidget("directory", messageWidgetClass, layout, NULL, 0);
+    XtCreateManagedWidget("file", messageWidgetClass, layout, NULL, 0);
 
-    XtSetArg(arg[0], XtNbuffer, "File:");
-    XtSetArg(arg[1], XtNborderWidth, 0);
-    new->filesel.file_label =
-	XtCreateManagedWidget("filelabel", messageWidgetClass,
-			      new->filesel.layout, arg, 2);
+    temp = XtCreateManagedWidget("scrbar", scrBarWidgetClass, layout, NULL, 0);
 
     XtSetArg(arg[0], XtNatLeastOne, False);
     XtSetArg(arg[1], XtNatMostOne, True);
@@ -688,54 +706,39 @@ static void Initialize(Widget grequest, Widget gnew,
     XtSetArg(arg[4], XtNpixmapHeight, 16);
     XtSetArg(arg[5], XtNusePixmaps, True);
     XtSetArg(arg[6], XtNpreferredColumns, new->filesel.pref_cols);
+    XtSetArg(arg[7], XtNvBar, temp);
     new->filesel.list =
-	XtCreateManagedWidget("list", scrListWidgetClass,
-			      new->filesel.layout, arg, 7);
-
-    new->filesel.scrbar =
-	XtCreateManagedWidget("scrbar", scrBarWidgetClass,
-			      new->filesel.layout, NULL, 0);
-
-    new->filesel.knapp_cancel =
-	XtCreateManagedWidget("cancel", knappWidgetClass,
-			      new->filesel.layout, NULL, 0);
-
-    new->filesel.knapp_choose =
-	XtCreateManagedWidget("choose", knappWidgetClass,
-			      new->filesel.layout, NULL, 0);
-
-    XtAddCallback(new->filesel.dir_field, XtNcallback,
-		  dir_field_callback, (XtPointer)new);
-    XtAddCallback(new->filesel.file_field, XtNcallback,
-		  file_field_callback, (XtPointer)new);
-    XtAddCallback(new->filesel.file_field, XtNtabCallback,
-		  file_field_tab_callback, (XtPointer)new);
+	XtCreateManagedWidget("list", scrListWidgetClass, layout, arg, 8);
     XtAddCallback(new->filesel.list, XtNcallback,
 		  list_callback, (XtPointer)new);
-    XtAddCallback(new->filesel.list, XtNresizeCallback,
-		  list_resize_callback, (XtPointer)new);
     XtAddCallback(new->filesel.list, XtNselectCallback,
 		  list_select_callback, (XtPointer)new);
-    XtAddCallback(new->filesel.scrbar, XtNscrollCallback,
-		  scrbar_callback, (XtPointer)new);
-    XtAddCallback(new->filesel.knapp_cancel, XtNcallback,
-		  cancel_callback, (XtPointer)new);
-    XtAddCallback(new->filesel.knapp_choose, XtNcallback,
-		  choose_callback, (XtPointer)new);
-    XtAddCallback((Widget)new, XtNpopupCallback, popup_callback, NULL);
 
-    if (!new->filesel.directory) {
+    temp = XtCreateManagedWidget("cancel", knappWidgetClass, layout, NULL, 0);
+    XtAddCallback(temp, XtNcallback, cancel_callback, (XtPointer)new);
+
+    temp = XtCreateManagedWidget("choose", knappWidgetClass, layout, NULL, 0);
+    XtAddCallback(temp, XtNcallback, choose_callback, (XtPointer)new);
+
+    XtSetArg(args[0], XtNset, new->filesel.show_dot_files);
+    temp = XtCreateManagedWidget("dotfiles", toggleWidgetClass,
+				 layout, args, 1);
+    XtAddCallback(temp, XtNcallback, show_callback, (XtPointer)new);
+
+    if (!new->filesel.directory)
 	if (home)
 	    new->filesel.directory = XtNewString(home);
 	else
 	    new->filesel.directory = XtNewString("/");
-    } else if (new->filesel.directory[0] == '/')
+    else if (new->filesel.directory[0] == '/')
 	new->filesel.directory = XtNewString(new->filesel.directory);
     else {
-	char	buffer[MAXPATHLEN];
+	char	*dir = new->filesel.directory;
 
-	sprintf(buffer, "%s/%s", home ? home : "", new->filesel.directory);
-	new->filesel.directory = XtNewString(buffer);
+	if (!home)
+	    home = "";
+	new->filesel.directory = XtMalloc(strlen(home) + strlen(dir) + 4);
+	sprintf(new->filesel.directory, "%s/%s", home, dir);
     }
 
     XtSetKeyboardFocus((Widget)new, new->filesel.file_field);
@@ -764,15 +767,10 @@ static void Realize(Widget w, XtValueMask *mask,
 
     topLevelShellWidgetClass->core_class.realize(w, mask, attributes);
 
-    wm_delete_window = XInternAtom(disp, "WM_DELETE_WINDOW", False);
+    wm_delete_window = intern_atom(disp, "WM_DELETE_WINDOW");
     XSetWMProtocols(disp, XtWindow(w), &wm_delete_window, 1);
 }
 
-static Boolean AcceptFocus(Widget gw, Time *time)
-{
-    return False;
-}
-
 static Boolean SetValues(Widget gcurrent,
 			 Widget grequest,
 			 Widget gnew,
@@ -783,10 +781,3 @@ static Boolean SetValues(Widget gcurrent,
 
     return redisplay;
 }
-
-static XtGeometryResult QueryGeometry(Widget gw,
-				      XtWidgetGeometry *intended,
-				      XtWidgetGeometry *preferred)
-{
-    return XtGeometryNo;
-}
