@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #ifndef FileSel_h
 #define FileSel_h
@@ -17,27 +17,21 @@
 #ifndef XtCDirectory
 #define XtCDirectory "Directory"
 #endif
-#ifndef XtCStayUp
-#define XtCStayUp "StayUp"
+#ifndef XtCShowDotFiles
+#define XtCShowDotFiles "ShowDotFiles"
 #endif
 
 #ifndef XtNbusyCursor
 #define XtNbusyCursor "busyCursor"
 #endif
-#ifndef XtNcancelCallback
-#define XtNcancelCallback "cancelCallback"
-#endif
 #ifndef XtNcursor
 #define XtNcursor "cursor"
 #endif
-#ifndef XtNchooseCallback
-#define XtNchooseCallback "chooseCallback"
-#endif
 #ifndef XtNdirectory
 #define XtNdirectory "directory"
 #endif
-#ifndef XtNstayUp
-#define XtNstayUp "stayUp"
+#ifndef XtNshowDotFiles
+#define XtNshowDotFiles "showDotFiles"
 #endif
 
 typedef struct FileSelClassRec*		FileSelWidgetClass;
