@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #ifndef Knapp_h
 #define Knapp_h
@@ -42,9 +42,10 @@ typedef struct KnappRec*	KnappWidget;
 
 extern WidgetClass knappWidgetClass;
 
-extern void KnappSetLabel(Widget, char*);
-extern void KnappSetActive(Widget, int);
-extern void KnappSetLabelNo(Widget, int, int);
-extern void KnappSetSensitive(Widget, int);
+extern void	KnappSetLabel(Widget, char*);
+extern void	KnappSetActive(Widget, int);
+extern void	KnappSetLabelNo(Widget, int, int);
+extern int	KnappGetLabelNo(Widget);
+extern void	KnappSetSensitive(Widget, int);
 
 #endif /* Knapp_h */
