@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #ifndef Manager_h
 #define Manager_h
@@ -17,9 +17,6 @@
 #ifndef XtCContain
 #define XtCContain "Contain"
 #endif
-#ifndef XtCCover
-#define XtCCover "Cover"
-#endif
 #ifndef XtCPreferredHeight
 #define XtCPreferredHeight "PreferredHeight"
 #endif
@@ -33,12 +30,6 @@
 #ifndef XtNcontainVert
 #define XtNcontainVert "containVert"
 #endif
-#ifndef XtNcoverHoriz
-#define XtNcoverHoriz "coverHoriz"
-#endif
-#ifndef XtNcoverVert
-#define XtNcoverVert "coverVert"
-#endif
 #ifndef XtNpreferredHeight
 #define XtNpreferredHeight "preferredHeight"
 #endif
@@ -54,4 +45,6 @@ typedef struct ManagerClassRec		*ManagerWidgetClass;
 
 extern WidgetClass managerWidgetClass;
 
+extern void	Remanage(Widget);
+
 #endif /* Manager_h */
