@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #ifndef MenuKnappP_h
 #define MenuKnappP_h
@@ -30,16 +30,24 @@ typedef struct MenuKnappClassRec {
 
 extern MenuKnappClassRec menuKnappClassRec;
 
+enum {
+    MenuStateDown,
+    MenuStateUp,
+    MenuStateWaiting
+};
+
 typedef struct {
-    String	menu_name;
-    int		multi_click_time;
-    Dimension	arrow_size;
-    Dimension	arrow_offset;
-    Dimension	arrow_shadow_width;
+    String		menu_name;
+    int			multi_click_time;
+    int			popdown_time;
+    Dimension		arrow_size;
+    Dimension		arrow_offset;
+    Dimension		arrow_shadow_width;
     /* private data */
-    Boolean	active;
-    Widget	menu;
-    Time	start_time;
+    Widget		menu;
+    Time		start_time;
+    int			menu_state;
+    XtIntervalId	timer;
 } MenuKnappPart;
 
 typedef struct MenuKnappRec {
