@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #ifndef MessageP_h
 #define MessageP_h
@@ -32,13 +32,13 @@ typedef struct {
     Pixel		foreground_pixel;
     XFontStruct		*font;
     String		buffer;
-    Cardinal		buffer_len;
     Dimension		internal_height;
     Dimension		internal_width;
     Dimension		pref_chars;
     Boolean		center;
     /* private data */
     Cardinal		rows;
+    Cardinal		n_alloc;
     GC			default_gc;
 } MessagePart;
 
