@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
@@ -20,6 +20,10 @@
 #include "ScrListP.h"
 #include "Util.h"
 
+#define FIRST(w)  ((w)->scrollable.pos_y)
+#define SHOWN(w)  ((w)->scrollable.shown_y)
+#define LINES(w)  ((w)->scrollable.height)
+
 static void grey90_default_proc(Widget, int, XrmValue*);
 
 static XtResource resources[] = {
@@ -66,8 +70,6 @@ static XtResource resources[] = {
      offset(allow_dnd), XtRImmediate, (XtPointer)False},
     {XtNusePixmaps, XtCUsePixmaps, XtRBoolean, sizeof(Boolean),
      offset(use_pixmaps), XtRImmediate, (XtPointer)False},
-    {XtNresizeCallback, XtCCallback, XtRCallback, sizeof(XtCallbackList),
-     offset(resize_callback), XtRImmediate, (XtPointer)NULL},
     {XtNselectCallback, XtCCallback, XtRCallback, sizeof(XtCallbackList),
      offset(select_callback), XtRCallback, (XtPointer)NULL},
     {XtNcallback, XtCCallback, XtRCallback, sizeof(XtCallbackList),
@@ -78,25 +80,34 @@ static XtResource resources[] = {
      offset(dnd_callback), XtRCallback, (XtPointer)NULL},
     {XtNdndCursor, XtCCursor, XtRCursor, sizeof(Cursor),
      offset(dnd_cursor), XtRString, (XtPointer)"hand2"},
+    {XtNmarginUp, XtCMargin, XtRDimension, sizeof(Dimension),
+     offset(margin_up), XtRImmediate, (XtPointer)1},
+    {XtNmarginDown, XtCMargin, XtRDimension, sizeof(Dimension),
+     offset(margin_down), XtRImmediate, (XtPointer)1},
+    {XtNpageUp, XtCPage, XtRBoolean, sizeof(Boolean),
+     offset(page_up), XtRImmediate, (XtPointer)True},
+    {XtNpageDown, XtCPage, XtRBoolean, sizeof(Boolean),
+     offset(page_down), XtRImmediate, (XtPointer)True},
 #undef offset
 };
 
-static void Initialize(Widget, Widget, ArgList, Cardinal*);
-static void Destroy(Widget);
-static void Redisplay(Widget, XEvent*, Region);
-static void Resize(Widget);
-static void Realize(Widget, XtValueMask*, XSetWindowAttributes*);
-static Boolean SetValues(Widget, Widget, Widget, ArgList, Cardinal*);
+static void	Initialize(Widget, Widget, ArgList, Cardinal*);
+static void	Destroy(Widget);
+static void	Redisplay(Widget, XEvent*, Region);
+static void	Resize(Widget);
+static void	Realize(Widget, XtValueMask*, XSetWindowAttributes*);
+static Boolean	SetValues(Widget, Widget, Widget, ArgList, Cardinal*);
+static void	SetVPos(ScrollableWidget, long);
 static XtGeometryResult QueryGeometry(Widget, XtWidgetGeometry*,
 				      XtWidgetGeometry*);
 
-static void notify(Widget, XEvent*, String*, Cardinal*);
-static void notify_second(Widget, XEvent*, String*, Cardinal*);
-static void select_action(Widget, XEvent*, String*, Cardinal*);
-static void drag_select(Widget, XEvent*, String*, Cardinal*);
-static void dnd_start(Widget, XEvent*, String*, Cardinal*);
-static void dnd_do(Widget, XEvent*, String*, Cardinal*);
-static void dnd_end(Widget, XEvent*, String*, Cardinal*);
+static void	notify(Widget, XEvent*, String*, Cardinal*);
+static void	notify_second(Widget, XEvent*, String*, Cardinal*);
+static void	select_action(Widget, XEvent*, String*, Cardinal*);
+static void	drag_select(Widget, XEvent*, String*, Cardinal*);
+static void	dnd_start(Widget, XEvent*, String*, Cardinal*);
+static void	dnd_do(Widget, XEvent*, String*, Cardinal*);
+static void	dnd_end(Widget, XEvent*, String*, Cardinal*);
 
 static XtActionsRec actions[] = {
     {"notify",		notify},
@@ -114,7 +125,7 @@ static char translations[] =
 
 ScrListClassRec scrListClassRec = {
     { /* core_class fields */
-	(WidgetClass) &shadowClassRec,	/* superclass			*/
+	(WidgetClass) &scrollableClassRec,	/* superclass		*/
 	"ScrList",			/* class_name			*/
 	sizeof(ScrListRec),		/* widget_size			*/
 	NULL,				/* class_initialize		*/
@@ -165,6 +176,12 @@ ScrListClassRec scrListClassRec = {
 	XtInheritAllocGCs,		/* alloc_gcs			*/
 	NULL,				/* extension			*/
     },
+    {					/* scrollable fields		*/
+	XtInheritScrollableSetPos,	/* set_hpos			*/
+	SetVPos,			/* set_vpos			*/
+	NULL,				/* suspend_hook			*/
+	NULL,				/* extension			*/
+    },
     {					/* scrlist fields		*/
 	NULL				/* extension			*/
     }
@@ -192,7 +209,7 @@ static void grey90_default_proc(Widget w, int offset, XrmValue *to)
 		 col.blue  == 0) ||
 		(col.red   == 0xffff &&
 		 col.green == 0xffff &&
-		 col.blue == 0xffff))
+		 col.blue  == 0xffff))
 		XFreeColors(disp, w->core.colormap, &col.pixel, 1, 0);
 	    else
 		grey90 = col.pixel;
@@ -233,8 +250,8 @@ static void alloc_list(ScrListWidget w, long n_alloc)
     long	n = w->scrlist.n_alloc;
     int		use_pixmaps = w->scrlist.use_pixmaps;
 
-    if (n_alloc < w->scrlist.lines)
-	n_alloc = w->scrlist.lines;
+    if (n_alloc < LINES(w))
+	n_alloc = LINES(w);
 
     w->scrlist.strings =
 	(char **)XtRealloc((char *)w->scrlist.strings,
@@ -270,7 +287,7 @@ static void calc_shown(ScrListWidget w)
     dh += w->scrlist.row_spacing +
 	2 * (w->scrlist.internal_item_height + w->shadow.shadow_width);
 
-    w->scrlist.shown = (dh <= 0) ? 0 : h/dh;
+    SHOWN(w) = (dh <= 0) ? 0 : h/dh;
 }
 
 static void draw_list_item(ScrListWidget w, String *string,
@@ -324,14 +341,14 @@ static void redraw_items(ScrListWidget w, long start, long stop, Boolean clear)
     int		y = w->scrlist.internal_height;
     int		height = 0;
 
-    if (width <= 0 || w->scrlist.lines == 0)
+    if (width <= 0 || LINES(w) == 0)
 	return;
-    if (start < w->scrlist.first)
-	start = w->scrlist.first;
-    if (stop >= w->scrlist.first + w->scrlist.shown)
-	stop = w->scrlist.first + w->scrlist.shown - 1;
-    if (stop >= w->scrlist.lines)
-	stop = w->scrlist.lines - 1;
+    if (start < FIRST(w))
+	start = FIRST(w);
+    if (stop >= FIRST(w) + SHOWN(w))
+	stop = FIRST(w) + SHOWN(w) - 1;
+    if (stop >= LINES(w) - 1)
+	stop = LINES(w) - 1;
     if (start > stop)
 	return;
 
@@ -339,8 +356,6 @@ static void redraw_items(ScrListWidget w, long start, long stop, Boolean clear)
 	string += start;
     if (pixmap)
 	pixmap += start;
-    x = w->scrlist.internal_width;
-    width = w->core.width - 2 * w->scrlist.internal_width;
     if (string)
 	height = w->scrlist.font->ascent + w->scrlist.font->descent;
     if (pixmap && (int)w->scrlist.pixmap_height > height)
@@ -348,7 +363,7 @@ static void redraw_items(ScrListWidget w, long start, long stop, Boolean clear)
     height += 2 * (w->scrlist.internal_item_height + w->shadow.shadow_width);
     if (width <= 0 || height == 0)
 	return;
-    y += (start - w->scrlist.first) * (height + w->scrlist.row_spacing);
+    y += (start - FIRST(w)) * (height + w->scrlist.row_spacing);
 
     if (clear) {
 	int	h = (stop - start + 1) * (height + w->scrlist.row_spacing);
@@ -372,16 +387,19 @@ static void redraw_items(ScrListWidget w, long start, long stop, Boolean clear)
 
 static Dimension preferred_width(ScrListWidget w)
 {
-    long	width;
+    int	ret;
 
-    width = w->scrlist.preferred_columns * w->scrlist.font->max_bounds.width;
+    ret = w->scrlist.max_width;
+    if (ret <= 0)
+	ret = w->scrlist.preferred_columns * w->scrlist.font->max_bounds.width;
 
-    if (width > 32767)
-	width = 32767;
-    else if (width <= 0)
-	width = 1;
+    ret += 2 * (w->shadow.shadow_width + w->scrlist.internal_width +
+		w->scrlist.internal_item_width);
 
-    return (Dimension)width;
+    if (w->scrlist.use_pixmaps)
+	ret += w->scrlist.pixmap_width + w->scrlist.pixmap_spacing;
+
+    return ret;
 }
 
 static Dimension preferred_height(ScrListWidget w)
@@ -404,7 +422,41 @@ static Dimension preferred_height(ScrListWidget w)
     else if (height <= 0)
 	height = 1;
 
-    return (Dimension)height;
+    return height;
+}
+
+static void make_visible(ScrListWidget w, long i, int may_page)
+{
+    long	first = FIRST(w);
+
+    if (LINES(w) <= 0)
+	return;
+
+    if (i >= LINES(w))
+	i = LINES(w) - 1;
+    if (i < 0)
+	i = 0;
+
+    if (i < FIRST(w) + w->scrlist.margin_up)
+	if (may_page && w->scrlist.page_up)
+	    first = i - SHOWN(w) + w->scrlist.margin_down + 1;
+	else
+	    first = i - w->scrlist.margin_up;
+    else if (i >= FIRST(w) + SHOWN(w) - w->scrlist.margin_down)
+	if (may_page && w->scrlist.page_down)
+	    first = i - w->scrlist.margin_up;
+	else
+	    first = i - SHOWN(w) + w->scrlist.margin_down + 1;
+
+    if (i < first)
+	i = first;
+    else if (i >= first + SHOWN(w))
+	first = i - SHOWN(w) + 1;
+    if (first < 0)
+	first = 0;
+
+    if (first != FIRST(w))
+	ScrollableSetVPos((Widget)w, first);
 }
 
 /*************************************************************************/
@@ -420,7 +472,7 @@ static long event_to_index(ScrListWidget w, XEvent *event)
 	return -1;
     }
 
-    if (w->scrlist.lines <= 0 || ex < (int)w->scrlist.internal_width ||
+    if (LINES(w) <= 0 || ex < (int)w->scrlist.internal_width ||
 	ex > (int)(w->core.width - w->scrlist.internal_width))
 	return -1;
 
@@ -431,17 +483,13 @@ static long event_to_index(ScrListWidget w, XEvent *event)
     if (w->scrlist.pixmaps && h < (int)w->scrlist.pixmap_height)
 	h = w->scrlist.pixmap_height;
     h += 2 * (w->scrlist.internal_item_height + w->shadow.shadow_width);
-    if (ey < y) {
-	if (w->scrlist.first > 0)
-	    return w->scrlist.first - 1;
-	else
-	    return 0;
-    }
+    if (ey < y)
+	return FIRST(w) > 0 ? FIRST(w) - 1 : 0;
 
-    i = w->scrlist.first;
-    n = w->scrlist.first + w->scrlist.shown;
-    if (n > w->scrlist.lines)
-	n = w->scrlist.lines;
+    i = FIRST(w);
+    n = FIRST(w) + SHOWN(w);
+    if (n > LINES(w))
+	n = LINES(w);
 
     while (i < n) {
 	if (y <= ey && ey < y + h)
@@ -450,28 +498,17 @@ static long event_to_index(ScrListWidget w, XEvent *event)
 	i++;
     }
 
-    if (w->scrlist.first + w->scrlist.shown >= w->scrlist.lines)
-	return w->scrlist.lines - 1;
-    else
-	return w->scrlist.first + w->scrlist.shown;
+    n = FIRST(w) + SHOWN(w);
+    if (n >= LINES(w))
+	n = LINES(w) - 1;
+
+    return n;
 }
 
-static void send_resize_report(ScrListWidget w)
-{
-    XtCallbackList	c_list = w->scrlist.resize_callback;
-
-    if (c_list) {
-	ScrListReport	report;
-
-	report.canvas_length = w->scrlist.lines;
-	report.slider_length = w->scrlist.shown;
-	report.slider_position = w->scrlist.first;
-	XtCallCallbackList((Widget)w, c_list, (XtPointer)&report);
-    }
-}
-
-static void select_action(Widget gw, XEvent *event,
-			  String *params, Cardinal *no_params)
+static void select_action(Widget	 gw,
+			  XEvent	*event,
+			  String	*params,
+			  Cardinal	*no_params)
 {
     ScrListWidget	w = (ScrListWidget)gw;
     XtCallbackList	c_list = w->scrlist.select_callback;
@@ -481,7 +518,7 @@ static void select_action(Widget gw, XEvent *event,
 	return;
 
     i = event_to_index(w, event);
-    if (i < 0 || i >= w->scrlist.lines)
+    if (i < 0 || i >= LINES(w))
 	return;
 
     ScrListSetSelected((Widget)w, i, !w->scrlist.selected[i]);
@@ -490,8 +527,10 @@ static void select_action(Widget gw, XEvent *event,
 	XtCallCallbackList((Widget)w, c_list, (XtPointer)i);
 }
 
-static void notify(Widget gw, XEvent *event,
-		   String *params, Cardinal *no_params)
+static void notify(Widget	 gw,
+		   XEvent	*event,
+		   String	*params,
+		   Cardinal	*no_params)
 {
     ScrListWidget	w = (ScrListWidget)gw;
     XtCallbackList	c_list = w->scrlist.callback;
@@ -501,14 +540,16 @@ static void notify(Widget gw, XEvent *event,
 	return;
 
     i = event_to_index(w, event);
-    if (i < 0 || i >= w->scrlist.lines)
+    if (i < 0 || i >= LINES(w))
 	return;
 
     XtCallCallbackList((Widget)w, c_list, (XtPointer)i);
 }
 
-static void notify_second(Widget gw, XEvent *event,
-			  String *params, Cardinal *no_params)
+static void notify_second(Widget	 gw,
+			  XEvent	*event,
+			  String	*params,
+			  Cardinal	*no_params)
 {
     ScrListWidget	w = (ScrListWidget)gw;
     XtCallbackList	c_list = w->scrlist.second_callback;
@@ -518,14 +559,16 @@ static void notify_second(Widget gw, XEvent *event,
 	return;
 
     i = event_to_index(w, event);
-    if (i < 0 || i >= w->scrlist.lines)
+    if (i < 0 || i >= LINES(w))
 	return;
 
     XtCallCallbackList((Widget)w, c_list, (XtPointer)i);
 }
 
-static void drag_select(Widget gw, XEvent *event,
-			String *params, Cardinal *no_params)
+static void drag_select(Widget		 gw,
+			XEvent		*event,
+			String		*params,
+			Cardinal	*no_params)
 {
     ScrListWidget	w = (ScrListWidget)gw;
     XtCallbackList	c_list = w->scrlist.select_callback;
@@ -535,7 +578,7 @@ static void drag_select(Widget gw, XEvent *event,
 	return;
 
     i = event_to_index(w, event);
-    if (i < 0 || i >= w->scrlist.lines)
+    if (i < 0 || i >= LINES(w))
 	return;
 
     if (!w->scrlist.selected[i]) {
@@ -545,8 +588,10 @@ static void drag_select(Widget gw, XEvent *event,
     }
 }
 
-static void dnd_start(Widget gw, XEvent *event, String *params,
-		      Cardinal *no_params)
+static void dnd_start(Widget	 gw,
+		      XEvent	*event,
+		      String	*params,
+		      Cardinal	*no_params)
 {
     ScrListWidget	w = (ScrListWidget)gw;
     long		i = event_to_index(w, event);
@@ -572,28 +617,21 @@ static void dnd_do(Widget gw, XEvent *event,
 	return;
     }
 
-    if (i < 0 || w->scrlist.lines <= 0)
+    if (LINES(w) <= 0)
 	return;
 
-    if (i >= w->scrlist.lines)
-	i = w->scrlist.lines - 1;
+    if (i >= LINES(w))
+	i = LINES(w) - 1;
+    if (i < 0)
+	i = 0;
 
-    if (i < w->scrlist.first) {
-	ScrListSetFirst((Widget)w, i);
-	send_resize_report(w);
-    } else if (i >= w->scrlist.first + w->scrlist.shown) {
-	i -= w->scrlist.shown - 1;
-	if (i > w->scrlist.lines)
-	    i = w->scrlist.lines - 1;
-	if (i < 0)
-	    i = 0;
-	ScrListSetFirst((Widget)w, i);
-	send_resize_report(w);
-    }
+    make_visible(w, i, False);
 }
 
-static void dnd_end(Widget gw, XEvent *event,
-		    String *params, Cardinal *no_params)
+static void dnd_end(Widget	 gw,
+		    XEvent	*event,
+		    String	*params,
+		    Cardinal	*no_params)
 {
     ScrListWidget	w = (ScrListWidget)gw;
     XtCallbackList	c_list = w->scrlist.dnd_callback;
@@ -610,8 +648,8 @@ static void dnd_end(Widget gw, XEvent *event,
     w->scrlist.dnd_start = -1;
     index[1] = event_to_index(w, event);
     if (!w->scrlist.allow_dnd || index[0] == index[1] ||
-	index[0] < 0 || index[0] >= w->scrlist.lines ||
-	index[1] < 0 || index[1] >= w->scrlist.lines) {
+	index[0] < 0 || index[0] >= LINES(w) ||
+	index[1] < 0 || index[1] >= LINES(w)) {
 	XBell(XtDisplay((Widget)w), 0);
 	return;
     }	      
@@ -668,23 +706,18 @@ static void Initialize(Widget grequest, Widget gnew,
     new->scrlist.selected = NULL;
     new->scrlist.n_sel = 0;
     new->scrlist.n_alloc = 0;
-    new->scrlist.first = 0;
-    new->scrlist.shown = 0;
-    new->scrlist.lines = 0;
+    new->scrlist.max_width = 0;
 
     init_gcs(new);
     calc_shown(new);
-    if (n_alloc > 0)
-	alloc_list(new, n_alloc);
-    else
-	alloc_list(new, 1);
+    alloc_list(new, n_alloc > 0 ? n_alloc : 1);
     new->scrlist.dnd_start = -1;
     if (new->scrlist.at_least_one) {
 	new->scrlist.selected[0] = True;
 	new->scrlist.n_sel = 1;
     }
-    if (new->core.width  == 0)
-	new->core.width  = preferred_width(new);
+    if (new->core.width == 0)
+	new->core.width = preferred_width(new);
     if (new->core.height == 0)
 	new->core.height = preferred_height(new);
 }
@@ -703,7 +736,7 @@ static void Redisplay(Widget gw, XEvent *event, Region region)
     long		start, stop;
     int			height = 0;
 
-    if (!XtIsRealized((Widget)w) || w->scrlist.lines == 0)
+    if (!XtIsRealized((Widget)w) || LINES(w) == 0)
 	return;
 
     if (w->scrlist.strings)
@@ -720,18 +753,18 @@ static void Redisplay(Widget gw, XEvent *event, Region region)
 	int	y1 = event->xexpose.y - y;
 	int	y2 = y1 + event->xexpose.height;
 
-	start = w->scrlist.first + y1/height;
-	stop = w->scrlist.first + y2/height;
+	start = FIRST(w) + y1/height;
+	stop = FIRST(w) + y2/height;
     } else if (event && event->type == GraphicsExpose) {
 	int	y = w->scrlist.internal_height;
 	int	y1 = event->xgraphicsexpose.y - y;
 	int	y2 = y1 + event->xgraphicsexpose.height;
 
-	start = w->scrlist.first + y1/height;
-	stop = w->scrlist.first + y2/height;
+	start = FIRST(w) + y1/height;
+	stop = FIRST(w) + y2/height;
     } else {
-	start = w->scrlist.first;
-	stop = start + w->scrlist.shown - 1;
+	start = FIRST(w);
+	stop = start + SHOWN(w) - 1;
     }
 
     redraw_items(w, start, stop, False);
@@ -741,24 +774,28 @@ static void Resize(Widget gw)
 {
     ScrListWidget	w = (ScrListWidget)gw;
 
+    ScrollableHFromGeometry((ScrollableWidget)w);
     calc_shown(w);
-    send_resize_report(w);
+    ScrollableFitHBar((ScrollableWidget)w);
+    ScrollableFitVBar((ScrollableWidget)w);
 }
 
-static void Realize(Widget gw, XtValueMask *mask,
-		    XSetWindowAttributes *attributes)
+static void Realize(Widget		   gw,
+		    XtValueMask		  *mask,
+		    XSetWindowAttributes  *attributes)
 {
     ScrListWidget	w = (ScrListWidget)gw;
 
-    shadowWidgetClass->core_class.realize((Widget)w, mask, attributes);
+    ScrollableHFromGeometry((ScrollableWidget)w);
     calc_shown(w);
+    scrollableWidgetClass->core_class.realize((Widget)w, mask, attributes);
 }
 
-static Boolean SetValues(Widget gcurrent,
-			 Widget grequest,
-			 Widget gnew,
-			 ArgList args,
-			 Cardinal *num_args)
+static Boolean SetValues(Widget		 gcurrent,
+			 Widget		 grequest,
+			 Widget		 gnew,
+			 ArgList	 args,
+			 Cardinal	*num_args)
 {
     ScrListWidget	new = (ScrListWidget)gnew;
     ScrListWidget	current = (ScrListWidget)gcurrent;
@@ -766,10 +803,10 @@ static Boolean SetValues(Widget gcurrent,
 
     new->scrlist.use_pixmaps = current->scrlist.use_pixmaps;
 
-    if (new->core.background_pixel != current->core.background_pixel ||
+    if (new->core.background_pixel    != current->core.background_pixel    ||
 	new->scrlist.foreground_pixel != current->scrlist.foreground_pixel ||
-	new->scrlist.highlight_pixel != current->scrlist.highlight_pixel ||
-	new->scrlist.font != current->scrlist.font) {
+	new->scrlist.highlight_pixel  != current->scrlist.highlight_pixel  ||
+	new->scrlist.font             != current->scrlist.font) {
 	free_gcs(current);
 	init_gcs(new);
 	redisplay = True;
@@ -782,20 +819,24 @@ static Boolean SetValues(Widget gcurrent,
 	alloc_list(new, n_alloc);
     }
 
-    if (new->scrlist.row_spacing != current->scrlist.row_spacing ||
-	new->shadow.shadow_width != current->shadow.shadow_width ||
-	new->scrlist.internal_width != current->scrlist.internal_width ||
+    if (new->scrlist.row_spacing     != current->scrlist.row_spacing     ||
+	new->shadow.shadow_width     != current->shadow.shadow_width     ||
+	new->scrlist.internal_width  != current->scrlist.internal_width  ||
 	new->scrlist.internal_height != current->scrlist.internal_height ||
-	new->scrlist.internal_item_width !=
-	current->scrlist.internal_item_height ||
-	new->scrlist.pixmap_width != current->scrlist.pixmap_width ||
-	new->scrlist.pixmap_height != current->scrlist.pixmap_height ||
-	new->scrlist.pixmap_spacing != current->scrlist.pixmap_spacing ||
-	new->scrlist.depth_one != current->scrlist.depth_one)
+	new->scrlist.pixmap_width    != current->scrlist.pixmap_width    ||
+	new->scrlist.pixmap_height   != current->scrlist.pixmap_height   ||
+	new->scrlist.pixmap_spacing  != current->scrlist.pixmap_spacing  ||
+	new->scrlist.depth_one       != current->scrlist.depth_one)
+	redisplay = True;
+
+    if (new->scrlist.internal_item_width !=
+	current->scrlist.internal_item_width ||
+	new->scrlist.internal_item_height !=
+	current->scrlist.internal_item_height)
 	redisplay = True;
 
     if (new->scrlist.at_least_one != current->scrlist.at_least_one &&
-	new->scrlist.n_sel == 0 && new->scrlist.lines > 0) {
+	new->scrlist.n_sel == 0 && LINES(new) > 0) {
 	new->scrlist.selected[0] = True;
 	redisplay = True;
     }
@@ -803,7 +844,7 @@ static Boolean SetValues(Widget gcurrent,
     if (new->scrlist.at_most_one != current->scrlist.at_most_one &&
 	new->scrlist.n_sel > 1) {
 	Boolean	*loop = new->scrlist.selected;
-	int	n = new->scrlist.lines;
+	int	n = LINES(new);
 	    
 	while (n-- > 0)
 	    if (*loop++)
@@ -814,10 +855,8 @@ static Boolean SetValues(Widget gcurrent,
     }
 
     if (new->scrlist.preferred_lines != current->scrlist.preferred_lines)
-	(void)XtMakeResizeRequest((Widget)new,
-				  preferred_width(new),
-				  preferred_height(new),
-				  NULL, NULL);
+	(void)XtMakeResizeRequest((Widget)new, preferred_width(new),
+				  preferred_height(new), NULL, NULL);
 
     if (redisplay)
 	calc_shown(new);
@@ -825,6 +864,56 @@ static Boolean SetValues(Widget gcurrent,
     return redisplay;
 }
 
+static void SetVPos(ScrollableWidget gw, long pos_y)
+{
+    ScrListWidget	w = (ScrListWidget)gw;
+    Display		*disp = XtDisplay(w);
+    Window		win = XtWindow(w);
+    long		old = FIRST(w);
+    long		diff;
+    long		n = SHOWN(w);
+    int			y = w->scrlist.internal_height;
+    int			dy = 0;
+
+    if (LINES(w) <= 0)
+	return;
+
+    if (pos_y > LINES(w) - SHOWN(w))
+	pos_y = LINES(w) - SHOWN(w);
+    if (pos_y < 0)
+	pos_y = 0;
+
+    diff = pos_y - old;
+    FIRST(w) = pos_y;
+    if (w->scrlist.strings)
+	dy = w->scrlist.font->ascent + w->scrlist.font->descent;
+    if (w->scrlist.pixmaps && (int)w->scrlist.pixmap_height > dy)
+	dy = w->scrlist.pixmap_height;
+    dy += 2 * (w->scrlist.internal_item_height + w->shadow.shadow_width) +
+	w->scrlist.row_spacing;
+    if (dy == 0 || diff == 0 || !XtIsRealized((Widget)w))
+	return;
+
+    if (diff <= -n || diff >= n) {
+	XClearWindow(disp, win);
+	redraw_items(w, pos_y, pos_y + n - 1, False);
+    } else if (diff < 0) {
+	XCopyArea(disp, win, win, w->scrlist.default_gc,
+		  0, y,
+		  w->core.width, (n + diff) * dy,
+		  0, y - diff * dy);
+	XClearArea(disp, win, 0, 0, 0, y - diff * dy, False);
+	redraw_items(w, pos_y, pos_y - diff - 1, False);
+    } else if (diff > 0) {
+	XCopyArea(disp, win, win, w->scrlist.default_gc,
+		  0, y + diff * dy,
+		  w->core.width, (n - diff) * dy,
+		  0, y);
+	XClearArea(disp, win, 0, y + (n - diff) * dy, 0, 0, False);
+	redraw_items(w, pos_y + n - diff, pos_y + n - 1, False);
+    }
+}
+
 static XtGeometryResult QueryGeometry(Widget gw,
 				      XtWidgetGeometry *intended,
 				      XtWidgetGeometry *preferred)
@@ -849,11 +938,10 @@ static XtGeometryResult QueryGeometry(Widget gw,
     if (intended_width == preferred->width &&
 	intended_height == preferred->height)
 	return XtGeometryYes;
-    else if (preferred->width == w->core.width &&
-	     preferred->height == w->core.height)
+    if (preferred->width == w->core.width &&
+	preferred->height == w->core.height)
 	return XtGeometryNo;
-    else
-	return XtGeometryAlmost;
+    return XtGeometryAlmost;
 }
 
 /*************************************************************************/
@@ -863,7 +951,7 @@ void ScrListClearLines(Widget gw)
     ScrListWidget	w = (ScrListWidget)gw;
     long		i;
 
-    for (i = 0 ; i < w->scrlist.lines ; i++) {
+    for (i = 0 ; i < LINES(w) ; i++) {
 	XtFree(w->scrlist.strings[i]);
 	w->scrlist.strings[i] = NULL;
 	w->scrlist.selected[i] = False;
@@ -879,50 +967,65 @@ void ScrListClearLines(Widget gw)
     w->scrlist.selected = NULL;
     w->scrlist.n_alloc = 0;
 
-    w->scrlist.lines = 0;
-    w->scrlist.first = 0;
+    LINES(w) = 0;
+    FIRST(w) = 0;
     w->scrlist.dnd_start = -1;
     w->scrlist.n_sel = 0;
+    w->scrlist.max_width = 0;
 
     if (XtIsRealized((Widget)w))
 	XClearWindow(XtDisplay(w), XtWindow(w));
+
+    ScrollableFitVBar((ScrollableWidget)w);
 }
 
 long ScrListAddLine(Widget gw, char *string, Pixmap pixmap)
 {
     ScrListWidget	w = (ScrListWidget)gw;
-    long		n = w->scrlist.lines++;
+    long		n = LINES(w)++;
+    int			len, width;
 
     if (n + 3 > w->scrlist.n_alloc)
 	alloc_list(w, 2 * (w->scrlist.n_alloc + 1));
-    w->scrlist.strings[n] = XtNewString(string);
+    len = strlen(string);
+    w->scrlist.strings[n] = strcpy(XtMalloc(len + 1), string);
     if (w->scrlist.use_pixmaps)
 	w->scrlist.pixmaps[n] = pixmap;
+    width = XTextWidth(w->scrlist.font, string, len);
+    if (width > w->scrlist.max_width)
+	w->scrlist.max_width = width;
 
     if (n == 0 && w->scrlist.at_least_one) {
 	w->scrlist.selected[0] = True;
 	w->scrlist.n_sel = 1;
     }
-/*    calc_shown(w);*/
 
     if (XtIsRealized((Widget)w) &&
-	n >= w->scrlist.first &&
-	n < w->scrlist.first + w->scrlist.shown)
+	n >= FIRST(w) && n < FIRST(w) + SHOWN(w))
 	redraw_items(w, n, n, False);
 
+    if (!w->scrollable.suspended)
+	ScrollableFitVBar((ScrollableWidget)w);
+
     return n;
 }
 
 void ScrListSetLine(Widget gw, long row, char *string, Pixmap pixmap)
 {
     ScrListWidget	w = (ScrListWidget)gw;
+    int			len, width;
 
-    if (row < 0 || row >= w->scrlist.lines) return;
+    if (row < 0 || row >= LINES(w))
+	return;
 
     XtFree(w->scrlist.strings[row]);
-    w->scrlist.strings[row] = XtNewString(string);
+    len = strlen(string);
+    w->scrlist.strings[row] = strcpy(XtMalloc(len + 1), string);
     if (w->scrlist.use_pixmaps)
 	w->scrlist.pixmaps[row] = pixmap;
+    width = XTextWidth(w->scrlist.font, string, len);
+    if (width > w->scrlist.max_width)
+	w->scrlist.max_width = width;
 
     if (XtIsRealized((Widget)w))
 	redraw_items(w, row, row, True);
@@ -935,139 +1038,86 @@ void ScrListDeleteLine(Widget gw, long row)
     int			was_sel;
     long		i, n;
 
-    if (row < 0 || row >= w->scrlist.lines) return;
+    if (row < 0 || row >= LINES(w))
+	return;
 
     was_sel = w->scrlist.selected[row];
     if (was_sel)
 	w->scrlist.n_sel--;
     XtFree(w->scrlist.strings[row]);
-    n = --w->scrlist.lines;
+    n = --LINES(w);
     for (i = row ; i < n ; i++) {
 	w->scrlist.strings[i] = w->scrlist.strings[i + 1];
 	if (use_pixmaps)
 	    w->scrlist.pixmaps[i] = w->scrlist.pixmaps[i + 1];
 	w->scrlist.selected[i] = w->scrlist.selected[i + 1];
     }
-    w->scrlist.strings[n + 1] = NULL;
+    w->scrlist.strings[n] = NULL;
 
     if (was_sel && w->scrlist.at_least_one &&
-	w->scrlist.n_sel == 0 && w->scrlist.lines > 0) {
-	if (row < w->scrlist.lines - 1)
+	w->scrlist.n_sel == 0 && LINES(w) > 0) {
+	if (row < LINES(w) - 1)
 	    w->scrlist.selected[row + 1] = True;
 	else
-	    w->scrlist.selected[w->scrlist.lines - 1] = True;
+	    w->scrlist.selected[LINES(w) - 1] = True;
 	w->scrlist.n_sel = 1;
     }
 
-    if (w->scrlist.first >= w->scrlist.lines &&
-	w->scrlist.lines > 0)
-	w->scrlist.first--;
+    if (FIRST(w) >= LINES(w) && LINES(w) > 0)
+	FIRST(w)--;
 
     if (XtIsRealized((Widget)w)) {
 	XClearWindow(XtDisplay(w), XtWindow(w));
 	Redisplay((Widget)w, NULL, NULL);
     }
+
+    if (!w->scrollable.suspended)
+	ScrollableFitVBar((ScrollableWidget)w);
 }
 
 void ScrListSetSelected(Widget gw, long i, int select)
 {
-    ScrListWidget w = (ScrListWidget)gw;
+    ScrListWidget	w = (ScrListWidget)gw;
 
-    if (select)
-	select = True;
-    if (i < 0 || i >= w->scrlist.lines ||
-	w->scrlist.selected[i] == select)
+    if (i < 0 || i >= LINES(w))
+	return;
+
+    select = !!select;
+    if (w->scrlist.selected[i] == select)
 	return;
 
     if (select) {
 	if (w->scrlist.at_most_one && w->scrlist.n_sel > 0) {
-	    long	j = 0;
-	    long	n = w->scrlist.lines;
-	    Boolean	*loop = w->scrlist.selected;
-	    
-	    while (j < n) {
-		if (*loop) {
-		    *loop = False;
+	    long	j;
+	    Boolean	*sel = w->scrlist.selected;
+
+	    for (j = 0 ; j < LINES(w) ; j++)
+		if (sel[j]) {
+		    sel[j] = False;
 		    redraw_items(w, j, j, True);
 		}
-		j++;
-		loop++;
-	    }
 	    w->scrlist.n_sel = 0;
 	}
 	w->scrlist.n_sel++;
 	w->scrlist.selected[i] = True;
 	redraw_items(w, i, i, True);
-    } else {
-	if (!w->scrlist.at_least_one || w->scrlist.n_sel > 1) {
-	    w->scrlist.n_sel--;
-	    w->scrlist.selected[i] = False;
-	    redraw_items(w, i, i, True);
-	}
+    } else if (!w->scrlist.at_least_one || w->scrlist.n_sel > 1) {
+	w->scrlist.n_sel--;
+	w->scrlist.selected[i] = False;
+	redraw_items(w, i, i, True);
     }
 }
 
-Boolean ScrListGetSelected(Widget gw, long i)
+void ScrListMakeVisible(Widget gw, long i)
 {
-    ScrListWidget w = (ScrListWidget)gw;
-
-    return i >= 0 && i < w->scrlist.lines && w->scrlist.selected[i];
+    make_visible((ScrListWidget)gw, i, True);
 }
 
-void ScrListSetFirst(Widget gw, long first)
-{
-    ScrListWidget w = (ScrListWidget)gw;
-    Display	*disp = XtDisplay(w);
-    Window	win = XtWindow(w);
-    long	old = w->scrlist.first;
-    long	diff = first - old;
-    long	n = w->scrlist.shown;
-    int		y = w->scrlist.internal_height;
-    int		dy = 0;
-
-    if (first < 0 || first >= w->scrlist.lines)
-	return;
-
-    w->scrlist.first = first;
-    if (w->scrlist.strings)
-	dy = w->scrlist.font->ascent + w->scrlist.font->descent;
-    if (w->scrlist.pixmaps && (int)w->scrlist.pixmap_height > dy)
-	dy = w->scrlist.pixmap_height;
-    dy += 2 * (w->scrlist.internal_item_height + w->shadow.shadow_width) +
-	w->scrlist.row_spacing;
-    if (dy == 0 || diff == 0 || !XtIsRealized((Widget)w))
-	return;
-
-    if (diff <= -n || diff >= n) {
-	XClearWindow(disp, win);
-	redraw_items(w, first, first + n - 1, False);
-    } else if (diff < 0) {
-	XCopyArea(disp, win, win, w->scrlist.default_gc,
-		  0, y,
-		  w->core.width, (n + diff) * dy,
-		  0, y - diff * dy);
-	XClearArea(disp, win, 0, 0, 0, y - diff * dy, False);
-	redraw_items(w, first, first - diff - 1, False);
-    } else {
-	XCopyArea(disp, win, win, w->scrlist.default_gc,
-		  0, y + diff * dy,
-		  w->core.width, (n - diff) * dy,
-		  0, y);
-	XClearArea(disp, win, 0, y + (n - diff) * dy, 0, 0, False);
-	redraw_items(w, first + n - diff, first + n - 1, False);
-    }
-}
-
-void ScrListGetFirstShownSize(Widget gw, long *first, long *shown, long *lines)
+int ScrListGetSelected(Widget gw, long i)
 {
     ScrListWidget w = (ScrListWidget)gw;
 
-    if (first)
-	*first = w->scrlist.first;
-    if (shown)
-	*shown = w->scrlist.shown;
-    if (lines)
-	*lines = w->scrlist.lines;
+    return i >= 0 && i < LINES(w) && w->scrlist.selected[i];
 }
 
 long ScrListGetFirstSelected(Widget gw)
@@ -1079,7 +1129,7 @@ long ScrListGetNextSelected(Widget gw, long i)
 {
     ScrListWidget	w = (ScrListWidget)gw;
     Boolean		*loop = w->scrlist.selected;
-    long		n = w->scrlist.lines;
+    long		n = LINES(w);
 
     i++;
     if (i < 0)
@@ -1099,7 +1149,7 @@ char *ScrListGetString(Widget gw, long row)
 {
     ScrListWidget	w = (ScrListWidget)gw;
 
-    if (row < 0 || row >= w->scrlist.lines)
+    if (row < 0 || row >= LINES(w))
 	return NULL;
 
     return w->scrlist.strings[row];
@@ -1110,7 +1160,7 @@ Pixmap ScrListGetPixmap(Widget gw, long row)
     ScrListWidget	w = (ScrListWidget)gw;
 
     if (!w->scrlist.use_pixmaps ||
-	row < 0 || row >= w->scrlist.lines)
+	row < 0 || row >= LINES(w))
 	return None;
 
     return w->scrlist.pixmaps[row];
@@ -1121,9 +1171,10 @@ void ScrListPurgePixmap(Widget gw, Pixmap pixmap)
     ScrListWidget	w = (ScrListWidget)gw;
     long		i;
 
-    if (!w->scrlist.use_pixmaps) return;
+    if (!w->scrlist.use_pixmaps)
+	return;
 
-    for (i = 0 ; i < w->scrlist.lines ; i++) {
+    for (i = 0 ; i < LINES(w) ; i++) {
 	if (w->scrlist.pixmaps[i] == pixmap) {
 	    w->scrlist.pixmaps[i] = None;
 	    redraw_items(w, i, i, True);
