@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #ifndef StringG_h
 #define StringG_h
@@ -23,6 +23,9 @@
 #ifndef XtCShadowWidth
 #define XtCShadowWidth "ShadowWidth"
 #endif
+#ifndef XtCCommand
+#define XtCCommand "Command"
+#endif
 
 #ifndef XtNleftMargin
 #define XtNleftMargin "leftMargin"
@@ -33,10 +36,15 @@
 #ifndef XtNshadowWidth
 #define XtNshadowWidth "shadowWidth"
 #endif
+#ifndef XtNcommand
+#define XtNcommand "command"
+#endif
 
 typedef struct StringGadgetClassRec*  StringGadgetClass;
 typedef struct StringGadgetRec*       StringGadget;
 
 extern WidgetClass stringGadgetClass;
 
+extern char	*StringGadgetCommand(Widget);
+
 #endif /* StringG_h */
