@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #ifndef ToggleG_h
 #define ToggleG_h
@@ -45,15 +45,7 @@ typedef struct ToggleGadgetRec*       ToggleGadget;
 
 extern WidgetClass toggleGadgetClass;
 
-#ifdef _XFUNCPROTOBEGIN
-_XFUNCPROTOBEGIN
-#endif
-
-extern void    ToggleGadgetSet(Widget, Boolean);
-extern Boolean ToggleGadgetGet(Widget);
-
-#ifdef _XFUNCPROTOBEGIN
-_XFUNCPROTOEND
-#endif
+extern void	ToggleGadgetSet(Widget, int);
+extern int	ToggleGadgetGet(Widget);
 
 #endif /* ToggleG_h */
