@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #include "global.h"
 #include <X11/Shell.h>
@@ -22,11 +22,12 @@
 #include "domain.h"
 #include "file.h"
 #include "font.h"
-#include "kaction.h"
+#include "k_action.h"
 #include "mailcap.h"
 #include "misc.h"
-#include "psetup.h"
+#include "p_menu.h"
 #include "procs.h"
+#include "read.h"
 #include "resource.h"
 #include "save.h"
 #include "server.h"
@@ -38,9 +39,9 @@
 
 #include "sysdeps.h"
 
-struct Global	global;
-int 		disp_conn_num = -1;
+struct Global	global = {0, };
 XtAppContext	app_cont;
+Display		*display = NULL;
 struct SERVER	*main_server = NULL;
 
 /*
@@ -55,13 +56,11 @@ static void set_icon(Widget shell)
 #include "knews_mask.xbm"
 
     icon =
-	XCreateBitmapFromData(XtDisplay(shell),
-			      DefaultRootWindow(XtDisplay(shell)),
+	XCreateBitmapFromData(display, DefaultRootWindow(display),
 			      (char *)knews_icon_bits,
 			      knews_icon_width, knews_icon_height);
     mask =
-	XCreateBitmapFromData(XtDisplay(shell),
-			      DefaultRootWindow(XtDisplay(shell)),
+	XCreateBitmapFromData(display, DefaultRootWindow(display),
 			      (char *)knews_mask_bits,
 			      knews_mask_width, knews_mask_height);
 
@@ -77,8 +76,15 @@ static void set_icon(Widget shell)
 
 static int disp_io_error_handler(Display *disp)
 {
-    perror("knews: fatal I/O error or KillClient");
-    global_cleanup(False, True);
+    char	*msg;
+
+    block_sighup();
+    perror("knews: Fatal I/O error or KillClient");
+    fputs("       Updating newsrc and kill files...  ", stderr);
+    msg = do_update();
+    if (!msg)
+	msg = "OK";
+    fprintf(stderr, "%s\n", msg);
     exit(1);
 }
 
@@ -111,8 +117,8 @@ static int disp_error_handler(Display *disp, XErrorEvent *event)
 	break;
     }
     fprintf(stderr, "0x%lx\n", event->resourceid);
-    fprintf(stderr, "knews: exiting!\n");
-    global_cleanup(False, True);
+
+    fputs("knews: exiting!\n", stderr);
     exit(1);
 }
 
@@ -123,6 +129,8 @@ static void init_timer(XtPointer client_data, XtIntervalId *id)
     unset_busy();
     if (global.nntp_server)
 	connect_server();
+    else
+	res_load(NULL);
 }
 
 int main(int argc, char *argv[])
@@ -141,6 +149,14 @@ int main(int argc, char *argv[])
 	 offset(config_nntp_server), XtRImmediate, (XtPointer)NULL},
 	{"editCommand", "EditCommand", XtRString, sizeof(String),
 	 offset(edit_command), XtRImmediate, (XtPointer)DEFAULT_EDIT_COMMAND},
+	{"urlCommand", "UrlCommand", XtRString, sizeof(String),
+	 offset(url_command), XtRImmediate, (XtPointer)NULL},
+	{"printCommand", "PrintCommand", XtRString, sizeof(String),
+	 offset(print_command), XtRImmediate, (XtPointer)NULL},
+	{"needsTerminal", "NeedsTerminal", XtRString, sizeof(String),
+	 offset(needs_terminal), XtRImmediate, (XtPointer)NULL},
+	{"copiousOutput", "CopiousOutput", XtRString, sizeof(String),
+	 offset(copious_output), XtRImmediate, (XtPointer)NULL},
 	{"mailName", "MailName", XtRString, sizeof(String),
 	 offset(mail_name), XtRImmediate, (XtPointer)NULL},
 	{"configFile", "ConfigFile", XtRString, sizeof(String),
@@ -153,6 +169,16 @@ int main(int argc, char *argv[])
 	 offset(stderr_timeout), XtRImmediate, (XtPointer)10000},
 	{"chunkSize", "ChunkSize", XtRInt, sizeof(int),
 	 offset(chunk_size), XtRImmediate, (XtPointer)16},
+	{"postMiscMenuSize", "PostMiscMenuSize", XtRInt, sizeof(int),
+	 offset(post_misc_menu_size), XtRImmediate, (XtPointer)1},
+	{"extraMenuSize", "ExtraMenuSize", XtRInt, sizeof(int),
+	 offset(extra_menu_size), XtRImmediate, (XtPointer)0},
+	{"typeMenuSize", "TypeMenuSize", XtRInt, sizeof(int),
+	 offset(type_menu_size), XtRImmediate, (XtPointer)0},
+	{"forwardMenuSize", "ForwardMenuSize", XtRInt, sizeof(int),
+	 offset(forward_menu_size), XtRImmediate, (XtPointer)0},
+	{"nCols", "NCols", XtRInt, sizeof(int),
+	 offset(n_cols), XtRImmediate, (XtPointer)(5*5*5 - 5 + 17)},
 	{"knewsVersion", "KnewsVersion", XtRString, sizeof(String),
 	 offset(version), XtRImmediate, (XtPointer)NULL},
 	{"separateWindows", "SeparateWindows", XtRBoolean, sizeof(Boolean),
@@ -163,8 +189,6 @@ int main(int argc, char *argv[])
 	 offset(head_debug), XtRImmediate, (XtPointer)False},
 	{"useIcon", "UseIcon", XtRBoolean, sizeof(Boolean),
 	 offset(use_icon), XtRImmediate, (XtPointer)True},
-	{"urlCommand", "UrlCommand", XtRString, sizeof(String),
-	 offset(url_command), XtRImmediate, (XtPointer)NULL},
 	{"defaultHotColor", "DefaultHotColor", XtRPixel, sizeof(Pixel),
 	 offset(default_hot_pixel), XtRString, XtDefaultForeground},
 	{"retrieveDescriptions", "RetrieveDescriptions",
@@ -180,18 +204,45 @@ int main(int argc, char *argv[])
 	 offset(keep_thread_info), XtRImmediate, (XtPointer)NULL},
 	{"checkForNewGroups", "CheckForNewGroups", XtRString, sizeof(String),
 	 offset(check_for_new_groups), XtRImmediate, (XtPointer)NULL},
-	{"confirmQuit", "ConfirmQuit", XtRBoolean, sizeof(Boolean),
+	{"confirmQuit", "Confirm", XtRBoolean, sizeof(Boolean),
 	 offset(confirm_quit), XtRImmediate, (XtPointer)False},
+	{"confirmCatchup", "Confirm", XtRBoolean, sizeof(Boolean),
+	 offset(confirm_catchup), XtRImmediate, (XtPointer)False},
+	{"confirmQuitGroup", "ConfirmQuit", XtRBoolean, sizeof(Boolean),
+	 offset(confirm_quit_group), XtRImmediate, (XtPointer)NULL},
 	{"icaseRegexps", "IcaseRegexps", XtRBoolean, sizeof(Boolean),
 	 offset(icase_regexps), XtRImmediate, (XtPointer)True},
 	{"showCache", "ShowCache", XtRBoolean, sizeof(Boolean),
 	 offset(show_cache), XtRImmediate, (XtPointer)False},
+	{"bogusFileSystem", "BogusFileSystem", XtRBoolean, sizeof(Boolean),
+	 offset(bogus_file_system), XtRImmediate, (XtPointer)False},
+	{"generatePath", "GeneratePath", XtRBoolean, sizeof(Boolean),
+	 offset(generate_path), XtRImmediate, (XtPointer)False},
+	{"quoteEmpty", "QuoteEmpty", XtRBoolean, sizeof(Boolean),
+	 offset(quote_empty), XtRImmediate, (XtPointer)True},
+	{"sortGroups", "SortGroups", XtRBoolean, sizeof(Boolean),
+	 offset(sort_groups), XtRImmediate, (XtPointer)False},
+	{"inlineImages", "InlineImages", XtRBoolean, sizeof(Boolean),
+	 offset(inline_images), XtRImmediate, (XtPointer)True},
+	{"colorHack", "Hack", XtRBoolean, sizeof(Boolean),
+	 offset(color_hack), XtRImmediate, (XtPointer)False},
+	{"mimeForward", "MimeForward", XtRBoolean, sizeof(Boolean),
+	 offset(mime_forward), XtRImmediate, (XtPointer)True},
 	{"newsrcTemplate", "NewsrcTemplate", XtRString, sizeof(String),
 	 offset(newsrc_templ), XtRImmediate, (XtPointer)NULL},
 	{"oldNewsrcTemplate", "OldNewsrcTemplate", XtRString, sizeof(String),
 	 offset(old_newsrc_templ), XtRImmediate, (XtPointer)NULL},
 	{"killFileTemplate", "KillFileTemplate", XtRString, sizeof(String),
-	 offset(kill_file_templ), XtRImmediate, (XtPointer)NULL},
+	 offset(kill_file_templ), XtRImmediate, (XtPointer)"~/.kill-%s"},
+	{"groupKillFileTemplate", "GroupKillFileTemplate",
+	 XtRString, sizeof(String),
+	 offset(group_kill_file_templ), XtRImmediate,
+	 (XtPointer)"~/.knews/%s/%g/KILL"},
+	{"autoSubscribe", "AutoSubscribe", XtRString, sizeof(String),
+	 offset(auto_subscribe), XtRImmediate,
+	 (XtPointer)"news.answers:\nnews.newusers.questions:\n"},
+	{"mimeTypes", "MimeTypes", XtRString, sizeof(String),
+	 offset(mime_types), XtRImmediate, (XtPointer)NULL},
 	/******/
 	{"foreground", "Foreground", XtRPixel, sizeof(Pixel),
 	 offset(pixel), XtRString, XtDefaultForeground},
@@ -215,6 +266,7 @@ int main(int argc, char *argv[])
 	{"reply",			action_reply},
 	{"followup-and-reply",		action_followup_and_reply},
 	{"post-new",			action_post_new},
+	{"forward-by-mail",		action_forward_by_mail},
 	{"list-up",			action_list_up},
 	{"list-down",			action_list_down},
 	{"tree-or-list-up",		action_tree_or_list_up},
@@ -224,6 +276,8 @@ int main(int argc, char *argv[])
 	{"tree-left-or-exit-mode",	action_tree_left_or_exit_mode},
 	{"tree-right-or-enter-mode",	action_tree_right_or_enter_mode},
 	{"read-article",		action_read_article},
+	{"mime-hack",			action_mime_hack},
+	{"goto-next-hot",		action_goto_next_hot},
 	{"view-thread",			action_view_thread},
 	{"mark-read-article",		action_mark_read_article},
 	{"mark-read-subject",		action_mark_read_subject},
@@ -247,32 +301,36 @@ int main(int argc, char *argv[])
 	{"pipe",			action_pipe},
 	{"tag-thread",			action_tag_thread},
 	{"tag-subject",			action_tag_subject},
+	{"untag-thread",		action_untag_thread},
+	{"untag-subject",		action_untag_subject},
 	{"tag-hot",			action_tag_hot},
 	{"catchup",			action_catchup},
 	{"unsubscribe",			action_unsubscribe},
 	{"subscribe",			action_subscribe},
-	{"realize-thread",		action_realize_thread},
 	{"change-size",			action_change_size},
 	{"schedule-thread-ahead",	action_schedule_thread_ahead},
 	{"popup-find-group",		action_popup_find_group},
 	{"do-the-right-thing",		action_do_the_right_thing},
 	{"kill-append",			action_kill_append},
 	{"kill-prepend",		action_kill_prepend},
+	{"popup-kill",			action_popup_kill},
     };
     static String fallback_resources[] = {
 	"*grouplist*PreferredLines:	14",
 	"*killist*PreferredLines:	14",
 	"*PreferredColumns:		84",
 	"*ArtTree.useLineShadows:	True",
-	"*Knapp.BorderWidth:		1",
-	"*MenuKnapp.BorderWidth:	1",
-	"*TextField.BorderWidth:	2",
-	"*BorderWidth:			0",
-	"*Toggle.shadowWidth:		2",
 	"*textscrbar.allowOff:		True",
 	"*Knapp.Justify:		center",
 	"*MenuKnapp.Justify:		center",
 	"*quotetoggle.set:		True",
+	"*stderr*message.center:	False",
+	"*textscrbar.stepSize:		13",
+	"*unreadtoggle.set:		True",
+	"*postpopup*posttext*preferredLines:		8",
+	"*postpopup*posttext*preferredColumns:		64",
+	"*postpopup*attachlist*preferredLines:		3",
+	"*postpopup*attachlist*preferredColumns:	64",
 	/* colors */
 	"*Foreground:		Black",
 	"*Background:		Bisque",
@@ -332,7 +390,9 @@ int main(int argc, char *argv[])
 	"*killeditor*fieldknapp.label:	Message-Id\\nSubject\\nFrom\\nXref",
 	"*killeditor*scopeknapp.label: "
 	"	Article\\nSubject\\nThread\\nSubthread",
-	"*killeditor*actionknapp.label:	Kill\\nHot\\nHotish",
+	"*killeditor*actionknapp.label:	Kill\\nHot",
+	"*postpopup*post.label:		Post\\nMail\\nPost&Mail",
+	"*saveshell*ok.label:		Save\\nPipe",
 	/* "buffers" */
 	"*fieldmessage.buffer:		Header:",
 	"*scopemessage.buffer:		Scope:",
@@ -347,13 +407,26 @@ int main(int argc, char *argv[])
 	"*searchshell*regexpmessage.buffer:	Regexp:",
 	"*searchshell*wildcardmessage.buffer:	Wildcard:",
 	"*searchshell*headermessage.buffer:	Header:",
-	/* misc */
-	"*textscrbar.stepSize:		13",
+	"*postpopup*posttitle.buffer:		Post/Mail Manager",
+	"*postpopup*attachtitle.buffer:		Attachments:",
+	"*postpopup*typetitle.buffer:		Content-Type:",
+	"*postpopup*descrtitle.buffer:		Content-Description:",
+	"*postpopup*disptitle.buffer:		Content-Disposition:",
+	"*postpopup*enctitle.buffer:		Content-Transfer-Encoding:",
+	"*postpopup*nametitle.buffer:		filename=",
+	"*postpopup*inlinetoggle.label:		inline",
+	"*postpopup*attachtoggle.label:		attachment",
+	"*postpopup*nonetoggle.label:		None",
+	"*postpopup*base64toggle.label:		Base 64",
+	"*postpopup*uuetoggle.label:		Uuencode",
+	"*postpopup*qptoggle.label:		Quoted-printable",
 	NULL,
     };
     static XrmOptionDescRec options[] = {
 	{"-nntpServer",		".nntpServer",
 	 XrmoptionSepArg,	(XtPointer)NULL},
+	{"-ncols",		".nCols",
+	 XrmoptionSepArg,	(XtPointer)NULL},
 	{"-bg",			"*Background",
 	 XrmoptionSepArg,	(XtPointer)NULL},
 	{"-fg",			"*Foreground",
@@ -396,25 +469,35 @@ int main(int argc, char *argv[])
 	 XrmoptionNoArg,	(XtPointer)"True"},
 	{"-icase",		".icaseRegexps",
 	 XrmoptionNoArg,	(XtPointer)"False"},
+	{"+images",		".inlineImages",
+	 XrmoptionNoArg,	(XtPointer)"True"},
+	{"-images",		".inlineImages",
+	 XrmoptionNoArg,	(XtPointer)"False"},
     };
     Arg		args[8];
-    Display	*disp;
+    char	*home = getenv("HOME");
 
-    global.home = getenv("HOME");
-    if (!global.home) {
-	fprintf(stderr, "knews: couldn't getenv(\"HOME\")\n");
+    if (!home) {
+	fputs("knews: couldn't getenv(\"HOME\")\n", stderr);
 	exit(1);
     }
-    global.home = XtNewString(global.home);
-    if (chdir(global.home) < 0) {
-	perror("knews: couldn't chdir($HOME)");
+    if (chdir(home) < 0) {
+	int	oerrno = errno;
+
+	fprintf(stderr, "knews: couldn't chdir(%s)", home);
+	errno = oerrno;
+	perror(NULL);
 	exit(1);
     }
 
     freopen("/dev/null", "r", stdin);
 
+    srand(3ul * time(NULL) +  5ul * getuid() +
+	  7ul * getpid()   + 11ul * getppid());
+
     main_thr = create_thread_context();
     main_server = server_create(-1);
+    server_set_quit_func(main_server, nntp_quit);
 
 #if (XtSpecificationRelease > 4)
     XtSetLanguageProc(NULL, NULL, NULL);
@@ -430,15 +513,30 @@ int main(int argc, char *argv[])
     XtToolkitInitialize();
     app_cont = XtCreateApplicationContext();
     XtAppSetFallbackResources(app_cont, fallback_resources);
-    disp = XtOpenDisplay(app_cont, NULL, NULL, "Knews",
-			 options, XtNumber(options), &argc, argv);
-    if (!disp)
+    display = XtOpenDisplay(app_cont, NULL, NULL, "Knews",
+			    options, XtNumber(options), &argc, argv);
+
+    if (argc != 1) {
+	int	n = strlen(argv[1]);
+
+	if (strncmp(argv[1], "-version", n) == 0)
+	    fputs("knews: version " KNEWS_VERSION " compiled "
+		  "on " __DATE__ " " __TIME__ ".\n", stderr);
+	else
+	    fputs("knews: Bad command line arguments, "
+		  "see the man-page for usage.\n", stderr);
 	exit(1);
-    color_init(disp);
+    }
+
+    if (!display)
+	exit(1);
+
+    color_init(display);
     XtSetArg(args[0], XtNinput, True);
     XtSetArg(args[1], XtNcolormap, global.cmap);
     XtSetArg(args[2], XtNvisual, global.visual);
     XtSetArg(args[3], XtNdepth, global.depth);
+    XtSetArg(args[4], XtNmappedWhenManaged, False);
     main_widgets.shell =
 	XtAppCreateShell(NULL, "Knews",
 #if XtSpecificationRelease >= 6
@@ -446,39 +544,25 @@ int main(int argc, char *argv[])
 #else
 			 applicationShellWidgetClass,
 #endif
-			 disp, args, 4);
+			 display, args, 5);
     if (!main_widgets.shell)
 	exit(1);
 
-    if (argc != 1) {
-	int	n = strlen(argv[1]);
-
-	if (strncmp(argv[1], "-v", n) == 0)
-	    fprintf(stderr,
-		    "knews: version " KNEWS_VERSION " compiled "
-		    "on " __DATE__ " " __TIME__ ".\n");
-	else
-	    fprintf(stderr,
-		    "knews: Bad command line arguments, "
-		    "see the man-page for usage.\n");
-	exit(1);
-    }
-
-    mailcap_init();
     init_child_contexts();
     XtAppAddActions(app_cont, actions, XtNumber(actions));
     XtGetApplicationResources(main_widgets.shell, (XtPointer)&global,
 			      resource_spec, XtNumber(resource_spec),
 			      NULL, 0);
     init_fonts(main_widgets.shell);
+    mailcap_init();
 
     if (global.newsrc_templ && !strstr(global.newsrc_templ, "%s")) {
-	fputs("Warning: no %s in newsrcTemplate, ignoring it.\n", stderr);
+	fputs("knews: no %s in newsrcTemplate, ignoring it.\n", stderr);
 	global.newsrc_templ = NULL;
     }
 
     if (global.config_file && !strstr(global.config_file, "%s")) {
-	fputs("Warning: no %s in configFile, ignoring it.\n", stderr);
+	fputs("knews: no %s in configFile, ignoring it.\n", stderr);
 	global.config_file = NULL;
     }
     if (!global.config_file)
@@ -511,24 +595,28 @@ int main(int argc, char *argv[])
 
     if (global.chunk_size <= 0)
 	global.chunk_size = 16;
+    if (global.post_misc_menu_size <= 0)
+	global.post_misc_menu_size = 1;
 
     if (!global.version)
-	fprintf(stderr,
-		"Warning: Application defaults file "
-		"not properly installed.\n");
+	fputs("knews: Application defaults file "
+	      "not properly installed.\n", stderr);
     else if (strncmp(global.version, KNEWS_VERSION, 5) != 0)
-	fprintf(stderr,
-		"Warning: Incompatible version of "
-		"application defaults file.\n");
+	fputs("knews: Incompatible version of "
+	      "application defaults file.\n", stderr);
 
     create_main_widgets();
-    disp_conn_num = ConnectionNumber(XtDisplay(main_widgets.shell));
+    global.gc = DefaultGCOfScreen(XtScreen(main_widgets.shell));
+    alloc_colors();
+    if (fcntl(ConnectionNumber(display), F_SETFD, FD_CLOEXEC) < 0)
+	perror("fcntl");
 
     setNewsModeDisconnected();
     set_standard_message();
-    set_busy(False, False);
+    set_busy(False);
     XtAppAddTimeOut(app_cont, 500, init_timer, NULL);
 
+    XtMapWidget(main_widgets.shell);
     XtAppMainLoop(app_cont);
 
     return 0;
