@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #include "global.h"
 #include <X11/Shell.h>
@@ -17,13 +17,13 @@
 #include "cache.h"
 #include "codes.h"
 #include "connect.h"
-#include "kill.h"
 #include "misc.h"
 #include "newsrc.h"
 #include "partial.h"
 #include "procs.h"
 #include "read.h"
 #include "resource.h"
+#include "save.h"
 #include "search.h"
 #include "server.h"
 #include "tag.h"
@@ -35,18 +35,22 @@
 #include "../Widgets/ArtText.h"
 #include "../Widgets/ArtTree.h"
 #include "../Widgets/Dialogue.h"
-#include "../Widgets/PopdownSh.h"
 #include "../Widgets/Menu.h"
+#include "../Widgets/MenuShell.h"
 #include "../Widgets/Message.h"
 #include "../Widgets/Notice.h"
 #include "../Widgets/PullRight.h"
 #include "../Widgets/ScrBar.h"
 #include "../Widgets/ScrList.h"
+#include "../Widgets/Scrollable.h"
 #include "../Widgets/SeparatorG.h"
 #include "../Widgets/StringG.h"
 #include "../Widgets/ToggleG.h"
 #include "../Widgets/Util.h"
 
+#undef  EXTRA_MENU
+#define EXTRA_MENU 0
+
 typedef enum {
     MarkScopeArticle,
     MarkScopeSubject,
@@ -77,9 +81,10 @@ void update_misc_menu(NewsMode mode)
     switch (mode) {
     case NewsModeDisconnected:
 	break;
-    case NewsModeAllgroups:
-    case NewsModeNewgroups:
     case NewsModeConnected:
+    case NewsModeAllgroups:
+    case NewsModeSomegroups:
+    case NewsModeNewgroups:
 	ToggleGadgetSet(ask_how_many, res_ask_how_many());
 	break;
     case NewsModeGroup:
@@ -110,12 +115,11 @@ static void toggle_subs_callback(Widget w,
 				 XtPointer client_data,
 				 XtPointer call_data)
 {
-    Boolean	temp;
+    int	temp;
 
-    if (global.busy ||
-	(global.mode != NewsModeGroup &&
-	 global.mode != NewsModeThread) ||
-	!global.curr_group) return;
+    if (global.busy || !global.curr_group ||
+	(global.mode != NewsModeGroup && global.mode != NewsModeThread))
+	return;
 
     temp = global.curr_group->subscribed;
     global.curr_group->subscribed = !temp;
@@ -127,58 +131,72 @@ static void toggle_subs_callback(Widget w,
     update_misc_menu(global.mode);
 }
 
+static void print_callback(Widget w,
+			   XtPointer client_data,
+			   XtPointer call_data)
+{
+    String	params[3];
+    Cardinal	no_params;
+
+    if (global.busy || (global.mode != NewsModeGroup &&
+			global.mode != NewsModeThread))
+	return;
+
+    if (!global.curr_art) {
+	set_message("No selected article!", True);
+	return;
+    }
+
+    params[0] = global.print_command;
+    params[1] = "heb";
+    no_params = 2;
+
+    action_pipe(w, NULL, params, &no_params);
+}
+
 static void full_header_callback(Widget w,
 				 XtPointer client_data,
 				 XtPointer call_data)
 {
-    if (global.busy) return;
+    if (global.busy || (global.mode != NewsModeGroup &&
+			global.mode == NewsModeThread))
+	return;
 
-    if (global.mode == NewsModeGroup ||
-	global.mode == NewsModeThread) {
-	if (global.curr_art)
-	    read_article(global.curr_art, True, NULL);
-	else
-	    set_message("No selected article!", True);
-    }
+    if (global.curr_art)
+	read_article(global.curr_art, True, NULL, NULL);
+    else
+	set_message("No selected article!", True);
 }
 
 static void rot13_callback(Widget w,
 			   XtPointer client_data,
 			   XtPointer call_data)
 {
-    if (global.busy) return;
+    if (global.busy || (global.mode != NewsModeGroup &&
+			global.mode != NewsModeThread))
+	return;
 
-    if (global.mode == NewsModeGroup ||
-	global.mode == NewsModeThread) {
-	long		first, shown, size;
-
-	ArtTextRot13(main_widgets.text);
-
-	ArtTextGetFirstShownSize(main_widgets.text, &first, &shown, &size);
-	ScrBarSetLengthsAndPos(main_widgets.text_scrbar, size, shown, first);
-    }
+    ArtTextRot13(main_widgets.text);
 }
 
 static void clear_tags_callback(Widget w,
 				XtPointer client_data,
 				XtPointer call_data)
 {
-    if (global.busy) return;
+    if (global.busy ||(global.mode != NewsModeGroup &&
+		       global.mode != NewsModeThread))
+	return;
 
-    if (global.mode == NewsModeGroup ||
-	global.mode == NewsModeThread) {
-	clear_tagged_articles();
-	set_message("Cleared tagged articles.", False);
-    }
+    clear_tagged_articles();
+    set_message("Cleared tagged articles.", False);
 }
 
 static void tag_hot_callback(Widget w,
 			     XtPointer client_data,
 			     XtPointer call_data)
 {
-    if (global.busy ||
-	(global.mode != NewsModeGroup &&
-	 global.mode != NewsModeThread))
+    if (global.busy || (global.mode != NewsModeGroup &&
+			global.mode != NewsModeThread))
 	return;
 
     tag_hot_articles();
@@ -196,8 +214,8 @@ static void toggle_head_callback(Widget w,
 				 XtPointer client_data,
 				 XtPointer call_data)
 {
-    if (global.busy || !call_data ||
-	(global.mode != NewsModeGroup && global.mode != NewsModeThread))
+    if (global.busy || !call_data || (global.mode != NewsModeGroup &&
+				      global.mode != NewsModeThread))
 	return;
 
     res_set_full_header(*(Boolean *)call_data);
@@ -207,12 +225,11 @@ static void keep_thr_callback(Widget w,
 			      XtPointer client_data,
 			      XtPointer call_data)
 {
-    if (global.busy) return;
+    if (global.busy || !call_data || (global.mode != NewsModeGroup
+				      && global.mode == NewsModeThread))
+	return;
 
-    if ((global.mode == NewsModeGroup ||
-	 global.mode == NewsModeThread) &&
-	call_data)
-	res_set_keep_thread_info(*(Boolean *)call_data);
+    res_set_keep_thread_info(*(Boolean *)call_data);
 }
 
 static void mark_read_callback(Widget w,
@@ -226,16 +243,16 @@ static void mark_read_callback(Widget w,
     long	n;
     int		xref = res_process_xrefs();
 
-    if (global.busy ||
-	(global.mode != NewsModeGroup &&
-	 global.mode != NewsModeThread)) return;
+    if (global.busy || (global.mode != NewsModeGroup &&
+			global.mode != NewsModeThread))
+	return;
 
     if (art)
 	subj = art->subject;
     else
 	subj = global.curr_subj;
 
-    set_busy(False, True);
+    set_busy(False);
 
     switch (scope) {
     case MarkScopeArticle:
@@ -250,7 +267,7 @@ static void mark_read_callback(Widget w,
 		global.curr_group->no_unread--;
 		if (xref)
 		    process_xref(art);
-		if (art->hot > 0) {
+		if (art->pixmap != None) {
 		    global.n_hot--;
 		    update_subj_hot_value(subj);
 		}
@@ -331,7 +348,7 @@ static void mark_read_callback(Widget w,
 		global.curr_group->no_unread--;
 		if (xref)
 		    process_xref(arts[i]);
-		if (arts[i]->hot > 0) {
+		if (arts[i]->pixmap != None) {
 		    global.n_hot--;
 		    update_subj_hot_value(arts[i]->subject);
 		}
@@ -357,7 +374,7 @@ static void mark_read_callback(Widget w,
 
 	for (subj = get_subjects(main_thr) ; subj ; subj = subj->next) {
 	    subj->no_unread = 0;
-	    subj->hot = 0;
+	    subj->pixmap = None;
 	}
 
 	global.curr_group->no_unread = 0;
@@ -417,7 +434,7 @@ static void mark_read_callback(Widget w,
 
 		    if (i == n) {
 			art->read = True;
-			if (art->hot != 0)
+			if (art->pixmap != None)
 			    global.n_hot--;
 			no++;
 		    }
@@ -436,7 +453,7 @@ static void mark_read_callback(Widget w,
     case MarkScopeCold:
 	n = 0;
 	for (art = get_articles(main_thr) ; art ; art = art->next)
-	    if (art->from && !art->read && art->hot == 0) {
+	    if (art->from && !art->read && art->pixmap == None) {
 		art->read = True;
 		art->subject->no_unread--;
 		n++;
@@ -467,14 +484,14 @@ static void mark_unread_callback(Widget w,
     SUBJECT	*subj = global.curr_subj;
     long	n;
 
-    if (global.busy ||
-	(global.mode != NewsModeGroup &&
-	 global.mode != NewsModeThread)) return;
+    if (global.busy || (global.mode != NewsModeGroup &&
+			global.mode != NewsModeThread))
+	return;
 
     if (art)
 	subj = art->subject;
 
-    set_busy(False, True);
+    set_busy(False);
 
     switch (scope) {
     case MarkScopeArticle:
@@ -487,7 +504,7 @@ static void mark_unread_callback(Widget w,
 		art->read = False;
 		subj->no_unread++;
 		global.curr_group->no_unread++;
-		if (art->hot > 0) {
+		if (art->pixmap != None) {
 		    global.n_hot++;
 		    update_subj_hot_value(subj);
 		}
@@ -495,11 +512,10 @@ static void mark_unread_callback(Widget w,
 		if (global.mode == NewsModeThread) {
 		    ArtTreeNodeSetInner(main_widgets.arttree,
 					(ART_TREE_NODE *)art, True);
-		    if (art->hot > 0) {
+		    if (art->pixmap != None)
 			ArtTreeNodeSetPixmap(main_widgets.arttree,
 					     (ART_TREE_NODE *)art,
-					     global.hot_pixmaps[art->hot]);
-		    }
+					     art->pixmap);
 		}
 	    }
 	    set_standard_message();
@@ -569,7 +585,7 @@ static void mark_unread_callback(Widget w,
 		arts[i]->read = False;
 		arts[i]->subject->no_unread++;
 		global.curr_group->no_unread++;
-		if (arts[i]->hot > 0) {
+		if (arts[i]->pixmap != None) {
 		    global.n_hot++;
 		    update_subj_hot_value(arts[i]->subject);
 		}
@@ -577,9 +593,10 @@ static void mark_unread_callback(Widget w,
 		if (global.mode == NewsModeThread) {
 		    ArtTreeNodeSetInner(main_widgets.arttree,
 					(ART_TREE_NODE *)arts[i], True);
-		    ArtTreeNodeSetPixmap(main_widgets.arttree,
-					 (ART_TREE_NODE *)arts[i],
-					 global.hot_pixmaps[arts[i]->hot]);
+		    if (arts[i]->pixmap != None)
+			ArtTreeNodeSetPixmap(main_widgets.arttree,
+					     (ART_TREE_NODE *)arts[i],
+					     arts[i]->pixmap);
 		}
 	    }
 
@@ -590,7 +607,7 @@ static void mark_unread_callback(Widget w,
     case MarkScopeAllarticles:
 	for (subj = get_subjects(main_thr) ; subj ; subj = subj->next) {
 	    subj->no_unread = 0;
-	    subj->hot = 0;
+	    subj->pixmap = 0;
 	}
 	global.n_hot = 0;
 	for (art = get_articles(main_thr) ; art ; art = art->next)
@@ -604,10 +621,10 @@ static void mark_unread_callback(Widget w,
 		    art->read = False;
 		    art->subject->no_unread++;
 		    n++;
-		    if (art->hot != 0) {
+		    if (art->pixmap != None) {
 			global.n_hot++;
-			if (art->subject->hot == 0)
-			    art->subject->hot = art->hot;
+			if (art->subject->pixmap == None)
+			    art->subject->pixmap = art->pixmap;
 		    }
 		}
 	    }
@@ -626,7 +643,7 @@ static void mark_unread_callback(Widget w,
 	for (n = 0, art = get_articles(main_thr) ; art ; art = art->next) {
 	    if (art->read && art->killed && art->from) {
 		art->read = False;
-		if (art->hot != 0)
+		if (art->pixmap != None)
 		    global.n_hot++;  /* notreached? */
 		art->subject->no_unread++;
 		n++;
@@ -669,9 +686,8 @@ static void msgid_lookup_callback(Widget w,
 				  XtPointer client_data,
 				  XtPointer call_data)
 {
-    if (global.busy ||
-	(global.mode != NewsModeGroup &&
-	 global.mode != NewsModeThread))
+    if (global.busy || (global.mode != NewsModeGroup &&
+			global.mode != NewsModeThread))
 	return;
 
     if (!msgid_lookup_dialogue)
@@ -687,9 +703,8 @@ static void add_to_partial_callback(Widget w,
 				    XtPointer client_data,
 				    XtPointer call_data)
 {
-    if (global.busy ||
-	(global.mode != NewsModeGroup &&
-	 global.mode != NewsModeThread))
+    if (global.busy || (global.mode != NewsModeGroup &&
+			global.mode != NewsModeThread))
 	return;
 
     partial_build_cache();
@@ -711,21 +726,28 @@ static void ask_how_many_callback(Widget w,
 {
     Boolean	*set = (Boolean *)call_data;
 
-    if (global.busy ||
-	(global.mode != NewsModeConnected &&
-	 global.mode != NewsModeNewgroups &&
-	 global.mode != NewsModeAllgroups)) return;
+    if (global.busy || (global.mode != NewsModeConnected &&
+			global.mode != NewsModeNewgroups &&
+			global.mode != NewsModeAllgroups))
+	return;
 
     if (set)
 	res_set_ask_how_many(*set);
 }
 
-#if 0
-static void realize_thread_callback(Widget w,
-				    XtPointer client_data,
-				    XtPointer call_data)
+#if EXTRA_MENU
+static void extra_menu_callback(Widget w,
+				XtPointer client_data,
+				XtPointer call_data)
 {
-    action_realize_thread(w, NULL, NULL, NULL);
+    char	*action = StringGadgetCommand(w);
+
+    if (!action) {
+	set_message("No action specified for that menu entry!", True);
+	return;
+    }
+
+    popup_title_notice("extra", "Not yet implemented", 3);
 }
 #endif
 
@@ -739,23 +761,33 @@ void create_misc_menu1(Widget parent)
     XtSetArg(args[1], XtNvisual, global.visual);
     XtSetArg(args[2], XtNdepth, global.depth);
     misc_menu1 =
-	XtCreatePopupShell("miscshell1", popdownShellWidgetClass,
+	XtCreatePopupShell("miscshell1", menuShellWidgetClass,
 			   parent, args, 3);
     menu = XtCreateManagedWidget("miscmenu1", menuWidgetClass,
 				 misc_menu1, NULL, 0);
 
+#if EXTRA_MENU
+    if (global.extra_menu_size > 0) {
+	XtSetArg(args[0], XtNmenuName, "extrashell");
+	temp = MenuCreateGadget("extra", pullRightGadgetClass, menu, args, 1);
+	create_simple_menu(parent, "extra", global.extra_menu_size,
+			   extra_menu_callback, NULL);
+	MenuCreateGadget("separator", separatorGadgetClass, menu, NULL, 0);
+    }
+#endif
+
     temp = MenuCreateGadget("fullheader", stringGadgetClass, menu, NULL, 0);
     XtAddCallback(temp, XtNcallback, full_header_callback, NULL);
-#if 0
-    temp = MenuCreateGadget("realize", stringGadgetClass, menu, NULL, 0);
-    XtAddCallback(temp, XtNcallback, realize_thread_callback, NULL);
-#endif
     temp = MenuCreateGadget("rot13", stringGadgetClass, menu, NULL, 0);
     XtAddCallback(temp, XtNcallback, rot13_callback, NULL);
     temp = MenuCreateGadget("cleartags", stringGadgetClass, menu, NULL, 0);
     XtAddCallback(temp, XtNcallback, clear_tags_callback, NULL);
     temp = MenuCreateGadget("uudecode", stringGadgetClass, menu, NULL, 0);
     XtAddCallback(temp, XtNcallback, uudecode_callback, NULL);
+    if (global.print_command && global.print_command != '\0') {
+	temp = MenuCreateGadget("print", stringGadgetClass, menu, NULL, 0);
+	XtAddCallback(temp, XtNcallback, print_callback, NULL);
+    }
     XtSetArg(args[0], XtNlabel, "Unsubscribe");
     subs_unsubs = MenuCreateGadget("unsubs", stringGadgetClass, menu, args, 1);
     XtAddCallback(subs_unsubs, XtNcallback, toggle_subs_callback, NULL);
@@ -767,12 +799,12 @@ void create_misc_menu1(Widget parent)
     XtAddCallback(temp, XtNcallback, msgid_lookup_callback, NULL);
     temp = MenuCreateGadget("showcache", stringGadgetClass, menu, NULL, 0);
     XtAddCallback(temp, XtNcallback, show_cache_callback, NULL);
-    MenuCreateGadget("separator1", separatorGadgetClass, menu, NULL, 0);
+    MenuCreateGadget("separator", separatorGadgetClass, menu, NULL, 0);
     XtSetArg(args[0], XtNmenuName, "markreadshell");
     MenuCreateGadget("markread", pullRightGadgetClass, menu, args, 1);
     XtSetArg(args[0], XtNmenuName, "markunreadshell");
     MenuCreateGadget("markunread", pullRightGadgetClass, menu, args, 1);
-    MenuCreateGadget("separator1", separatorGadgetClass, menu, NULL, 0);
+    MenuCreateGadget("separator", separatorGadgetClass, menu, NULL, 0);
     head_toggle =
 	MenuCreateGadget("headertoggle", toggleGadgetClass, menu, NULL, 0);
     XtAddCallback(head_toggle, XtNcallback, toggle_head_callback, NULL);
@@ -784,7 +816,7 @@ void create_misc_menu1(Widget parent)
     XtSetArg(args[1], XtNvisual, global.visual);
     XtSetArg(args[2], XtNdepth, global.depth);
     subshell =
-	XtCreatePopupShell("markreadshell", popdownShellWidgetClass,
+	XtCreatePopupShell("markreadshell", menuShellWidgetClass,
 			   parent, args, 3);
     submenu =
 	XtCreateManagedWidget("markreadmenu", menuWidgetClass,
@@ -823,7 +855,7 @@ void create_misc_menu1(Widget parent)
     XtSetArg(args[1], XtNvisual, global.visual);
     XtSetArg(args[2], XtNdepth, global.depth);
     subshell =
-	XtCreatePopupShell("markunreadshell", popdownShellWidgetClass,
+	XtCreatePopupShell("markunreadshell", menuShellWidgetClass,
 			   parent, args, 3);
     submenu =
 	XtCreateManagedWidget("markunreadmenu", menuWidgetClass,
@@ -862,7 +894,7 @@ void create_misc_menu2(Widget parent)
     XtSetArg(args[1], XtNvisual, global.visual);
     XtSetArg(args[2], XtNdepth, global.depth);
     misc_menu2 =
-	XtCreatePopupShell("miscshell2", popdownShellWidgetClass,
+	XtCreatePopupShell("miscshell2", menuShellWidgetClass,
 			   parent, args, 3);
     menu = XtCreateManagedWidget("miscmenu2", menuWidgetClass,
 				 misc_menu2, NULL, 0);
@@ -989,7 +1021,8 @@ void action_catchup(Widget w, XEvent *event,
     GROUP	*g;
     long	n;
 
-    if (global.busy) return;
+    if (global.busy)
+	return;
 
     switch (global.mode) {
     case NewsModeDisconnected:
@@ -1029,10 +1062,9 @@ void action_catchup(Widget w, XEvent *event,
 	    g = global.groups[n];
 	    free_read_arts_list(g);
 	    g->read_arts = NULL;
-	    if (g->last_art >= g->first_art &&
-		g->last_art > 0) {
+	    if (g->last_art >= g->first_art && g->last_art > 0) {
 		g->read_arts =
-		    (ART_LIST_NODE*)XtMalloc(sizeof(ART_LIST_NODE));
+		    (ART_LIST_NODE *)XtMalloc(sizeof g->read_arts[0]);
 		g->read_arts->first = 1;
 		g->read_arts->last = g->last_art;
 		g->read_arts->next = NULL;
@@ -1043,6 +1075,7 @@ void action_catchup(Widget w, XEvent *event,
 	set_message("Catched up.", False);
 	break;
     case NewsModeNewgroups:
+    case NewsModeSomegroups:
 	break;
     }
 }
@@ -1050,7 +1083,8 @@ void action_catchup(Widget w, XEvent *event,
 void action_unsubscribe(Widget w, XEvent *event,
 			String *params, Cardinal *no_params)
 {
-    if (global.busy) return;
+    if (global.busy)
+	return;
 
     switch (global.mode) {
     case NewsModeDisconnected:
@@ -1065,7 +1099,6 @@ void action_unsubscribe(Widget w, XEvent *event,
 	global.curr_group->subscribed = False;
 	sort_groups();
 	setNewsModeConnected();
-
 	break;
     case NewsModeGroup:
     case NewsModeThread:
@@ -1075,6 +1108,7 @@ void action_unsubscribe(Widget w, XEvent *event,
 	break;
     case NewsModeAllgroups:
     case NewsModeNewgroups:
+    case NewsModeSomegroups:
 	knapp2_callback(w, NULL, NULL);
 	break;
     }
@@ -1083,7 +1117,8 @@ void action_unsubscribe(Widget w, XEvent *event,
 void action_subscribe(Widget w, XEvent *event,
 		      String *params, Cardinal *no_params)
 {
-    if (global.busy) return;
+    if (global.busy)
+	return;
 
     switch (global.mode) {
     case NewsModeDisconnected:
@@ -1096,6 +1131,7 @@ void action_subscribe(Widget w, XEvent *event,
 	break;
     case NewsModeAllgroups:
     case NewsModeNewgroups:
+    case NewsModeSomegroups:
 	knapp1_callback(w, NULL, NULL);
 	break;
     }
@@ -1116,7 +1152,7 @@ static void msgid_dialogue_callback(Widget w,
     char		buffer[512];
     DialogueReport	*report = (DialogueReport *)call_data;
     char		*msgid = report->buffer;
-    char		*reply;
+    char		*reply, *c;
     long		n;
     ARTICLE		*art;
 
@@ -1146,7 +1182,7 @@ static void msgid_dialogue_callback(Widget w,
 
 	n = strlen(msgid);
 	if (n > 480) {
-	    XBell(XtDisplay(w), 0);
+	    XBell(display, 0);
 	    break;
 	}
 
@@ -1154,28 +1190,29 @@ static void msgid_dialogue_callback(Widget w,
 	if (msgid[n - 1] == '>')
 	    msgid[--n] = '\0';
 
+	c = strchr(msgid, '@');
+	if (c)
+	    ascii_lower(c);
+
 	art = find_article(msgid, n);
 	set_curr_art(art, True);
 	if (art) {
-            if (read_article(art, False, NULL) &&
+            if (read_article(art, False, NULL, NULL) &&
 		global.mode == NewsModeThread)
 		setNewsModeThread();
 	    XtFree(msgid);
 	    break;
  	}
 
-	set_busy(True, True);
+	set_busy(True);
 	set_message("Asking server for article...", False);
 	sprintf(buffer, "ARTICLE <%s>\r\n", msgid);
 	XtFree(msgid);
 	reply = server_comm(main_server, buffer, True);
-	if (reply) {
-	    ArtTextClearLines(main_widgets.text);
-	    ScrBarSetLengthsAndPos(main_widgets.text_scrbar, 0, 0, 0);
-
+	if (reply)
 	    if (atoi(reply) == NNTP_OK_ARTICLE) {
-		reply = do_mime(NULL, main_server,
-				server_read(main_server), False, NULL, 0);
+		reply = do_mime(NULL, main_server, server_read(main_server),
+				False, NULL, 0, NULL);
 		set_standard_message();
 	    } else {
 		if (strlen(reply) > 200)
@@ -1183,15 +1220,8 @@ static void msgid_dialogue_callback(Widget w,
 		sprintf(buffer, "Error, message from server is: %s", reply);
 		set_message(buffer, True);
 	    }
-	}
 
-	if (reply) {
-	    long	first, shown, size;
-
-	    ArtTextGetFirstShownSize(main_widgets.text, &first, &shown, &size);
-	    ScrBarSetLengthsAndPos(main_widgets.text_scrbar,
-				   shown, size, first);
-	} else {
+	if (!reply) {
 	    reconnect_server(True);
 	    unset_busy();
 	    return;
