@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,15 +9,14 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #define PARSEDATE_ERROR ((time_t)(-1))
 
-extern time_t	 parsedate(char *);
-extern char	*time_t_to_date(time_t, char*);
-extern char	*eat_re(char*);
-extern char	*get_msgid(char*, char*);
-extern char	*parse_author(char *);
+extern time_t		 parsedate(char *);
+extern char		*time_t_to_date(time_t, char*);
+extern char		*eat_re(char*);
+extern const char	*parse_author(const char*, long*);
 
 extern const char	month_names[];
 
@@ -27,25 +26,25 @@ typedef struct MimeArg {
 } MimeArg;
 
 typedef enum {
-    MimeEncUnknown,
     MimeEncNone, /* 7bit, 8bit and binary */
     MimeEncBase64,
     MimeEncQP,
     MimeEncUue
 } MimeEnc;
 
-extern MimeEnc	 parse_content_enc(char*);
+extern int	 parse_content_enc(char**);
 extern int	 parse_content_type(char**, char*, int, char*, int,
-				    MimeArg*, int);
+				    MimeArg*, int, int);
 extern char	*get_charset(MimeArg*);
+extern char	*parse_content_disp(char**);
 
 typedef struct {
     char	*word;
     char	*end;
-    long	len;
-    int		is_qp;
-    XFontStruct	*font;
+    short	len;
+    short	ch_len;
+    short	is_qp;
 } EncWordData;
 
-extern char *next_enc_word(char*, EncWordData*);
-
+extern char	*next_enc_word(char*, EncWordData*);
+extern void	 decode_rfc1522(char*, const char*);
