@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,21 +9,15 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #include "global.h"
-#include "../Widgets/ArtText.h"
-#include "../Widgets/ArtTree.h"
-#include "../Widgets/Notice.h"
-#include "../Widgets/Sash.h"
-#include "../Widgets/ScrBar.h"
-#include "../Widgets/ScrList.h"
 #include "ahead.h"
 #include "cache.h"
 #include "codes.h"
 #include "connect.h"
-#include "kedit.h"
-#include "kill.h"
+#include "ftp.h"
+#include "k_file.h"
 #include "newsrc.h"
 #include "partial.h"
 #include "read.h"
@@ -36,48 +30,122 @@
 #include "util.h"
 #include "widgets.h"
 #include "xutil.h"
+#include "../Widgets/ArtText.h"
+#include "../Widgets/ArtTree.h"
+#include "../Widgets/Notice.h"
+#include "../Widgets/Sash.h"
+#include "../Widgets/ScrList.h"
+
+static int confirm_quit_group(void)
+{
+    char	*c = global.confirm_quit_group;
+
+    if (!c)
+	return False;
+
+    if (case_lstrcmp(c, "true") == 0)
+	return True;
+    if (case_lstrcmp(c, "tagged") == 0)
+	return no_tagged_articles() != 0;
+    if (case_lstrcmp(c, "false") == 0)
+	return True;
+
+    return True;
+}
+
+static void leave_group(int catchup)
+{
+    set_busy(False);
+
+    popdown_save();
+    popdown_search();
+    partial_clear_cache();
+    clear_tagged_articles();
+    clear_history();
+
+    free_read_arts_list(global.curr_group);
+
+    if (!catchup)
+	global.curr_group->read_arts = create_read_arts_list();
+    else {
+	ART_LIST_NODE	*temp;
+	ARTICLE		*art;
+
+	temp = global.curr_group->read_arts =
+	    (ART_LIST_NODE *)XtMalloc(sizeof *temp);
+	temp->next  = NULL;
+	temp->first = 1;
+	temp->last  = global.curr_group->last_art;
+
+	if (res_process_xrefs())
+	    for (art = get_articles(main_thr) ; art ; art = art->next)
+		if (!art->read)
+		    process_xref(art);
+    }
+
+    cache_leave_group();
+    thread_ahead_leave_group(global.curr_group);
+    kill_exit_group(global.curr_group);
+    global.curr_art = NULL;
+    global.curr_subj = NULL;
+    res_enter_group("none");
+
+    if (catchup || global.curr_group->subscribed)
+	setNewsModeConnected();
+    else
+	setNewsModeAllgroups(NULL);
+
+    unset_busy();
+
+    if (global.mode == NewsModeConnected)
+	check_if_rescan_due();
+}
+
+static void confirm_callback(Widget    w,
+			     XtPointer client_data,
+			     XtPointer call_data)
+{
+    NoticeReply	reply = (NoticeReply)call_data;
+    int		catchup = (int)client_data;
+
+    unset_busy();
+    XtPopdown(w);
+    XtDestroyWidget(w);
+
+    if (reply != NoticeReplyLeft)
+	return;
+
+    leave_group(catchup);
+}
+
+static void do_confirm(char *msg, int catchup)
+{
+    set_busy(False);
+    popup_notice("confirm", msg, "Yes", "No", NULL, 0,
+		 confirm_callback, (XtPointer)catchup, XtGrabExclusive);
+}
 
 /* Quit   Done */
 void knapp0_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 {
-    if (global.busy) return;
+    if (global.busy)
+	return;
 
     switch (global.mode) {
     case NewsModeDisconnected: /* Quit */
-	exit(0);
+	disconnect(True);
 	break;
     case NewsModeConnected: /* Quit */
-	quit(True);
+	disconnect(True);
 	break;
     case NewsModeGroup: /* Done */
-	set_busy(False, True);
-
-	popdown_save();
-	popdown_search();
-	partial_clear_cache();
-	clear_tagged_articles();
-	clear_history();
-
-	free_read_arts_list(global.curr_group);
-	global.curr_group->read_arts = create_read_arts_list();
-
-	cache_leave_group();
-	thread_ahead_leave_group(global.curr_group);
-	global.curr_art = NULL;
-	global.curr_subj = NULL;
-
-	if (global.curr_group->subscribed)
-	    setNewsModeConnected();
+	if (confirm_quit_group())
+	    do_confirm("Really exit group?", False);
 	else
-	    setNewsModeAllgroups();
-
-	unset_busy();
-	if (global.mode == NewsModeConnected)
-	    check_if_rescan_due();
-
-	res_enter_group("none");
+	    leave_group(False);
 	break;
     case NewsModeAllgroups: /* Done */
+    case NewsModeSomegroups: /* Done */
 	popdown_search();
 	setNewsModeConnected();
 	check_if_rescan_due();
@@ -104,17 +172,9 @@ void knapp1_callback(Widget gw, XtPointer client_data, XtPointer call_data)
     case NewsModeConnected: /* Disconnect */
 	if (global.busy)
 	    return;
-
-	set_busy(False, False);
-
-	(void)update_newsrc(); /* should check return value */
-	(void)update_kill_file();
-	server_write_raw(main_server, "QUIT\r\n", 6);
-	server_close(main_server);
-	global_cleanup(False, False);
-	set_standard_message();
-
-	unset_busy();
+	if (!ftp_put())
+	    return;
+	disconnect(False);
 	break;
     case NewsModeDisconnected: /* Connect */
 	if (global.busy)
@@ -135,44 +195,67 @@ void knapp1_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 	    return;
 
 	i = ScrListGetFirstSelected(main_widgets.group_list);
-
 	if (i < 0) {
 	    set_message("No selected groups!", True);
 	    return;
 	}
 
-	set_busy(False, True);
-	while (i >= 0) {
+	set_busy(False);
+	do {
 	    global.groups[i]->subscribed = True;
 	    i = ScrListGetNextSelected(main_widgets.group_list, i);
-	}
+	} while (i >= 0);
 	sort_groups();
 	global.curr_group = NULL;
-	setNewsModeAllgroups();
+	setNewsModeAllgroups(NULL);
 	set_standard_message();
 	unset_busy();
 	return;
+    case NewsModeSomegroups: /* Subscribe */
+	if (global.busy)
+	    return;
+
+	i = ScrListGetFirstSelected(main_widgets.group_list);
+	if (i < 0) {
+	    set_message("No selected groups!", True);
+	    return;
+	}
+
+	set_busy(False);
+	j = 0;
+	do {
+	    for ( ; j < global.no_groups ; j++)
+		if (global.groups[j]->disp == i) {
+		    global.groups[j]->subscribed = True;
+		    break;
+		}
+	    i = ScrListGetNextSelected(main_widgets.group_list, i);
+	} while (i >= 0);
+	sort_groups();
+	global.curr_group = NULL;
+	setNewsModeConnected();
+	set_standard_message();
+	unset_busy();
+	break;
     case NewsModeNewgroups: /* Subscribe */
 	if (global.busy)
 	    return;
 
 	i = ScrListGetFirstSelected(main_widgets.group_list);
-
 	if (i < 0) {
 	    set_message("No selected groups!", True);
 	    return;
 	}
 
-	set_busy(False, True);
-	while (i >= 0) {
-	    for (j = 0 ; j < global.no_groups ; j++) {
+	set_busy(False);
+	do {
+	    for (j = 0 ; j < global.no_groups ; j++)
 		if (global.groups[j]->disp == i) {
 		    global.groups[j]->subscribed = True;
 		    break;
 		}
-	    }
 	    i = ScrListGetNextSelected(main_widgets.group_list, i);
-	}
+	} while (i >= 0);
 	setNewsModeNewgroups();
 	unset_busy();
 	break;
@@ -182,16 +265,17 @@ void knapp1_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 /* Unsubscribe   All threads   All groups */
 void knapp2_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 {
-    long	i, j, n;
+    long	i, j;
     SUBJECT	*subj;
 
-    if (global.busy) return;
+    if (global.busy)
+	return;
 
     switch (global.mode) {
     case NewsModeConnected: /* All groups */
-	set_busy(False, True);
+	set_busy(False);
 	global.curr_group = NULL;
-	setNewsModeAllgroups();
+	setNewsModeAllgroups(NULL);
 	unset_busy();
 	break;
     case NewsModeGroup: /* All threads */
@@ -206,25 +290,6 @@ void knapp2_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 	set_standard_message();
 	break;
     case NewsModeAllgroups: /* Unsubscribe */
-	n = ScrListGetFirstSelected(main_widgets.group_list);
-
-	if (n < 0) {
-	    set_message("No selected groups!", True);
-	    return;
-	}
-
-	set_busy(False, True);
-	while (n >= 0) {
-	    global.groups[n]->subscribed = False;
-	    n = ScrListGetNextSelected(main_widgets.group_list, n);
-	}
-	sort_groups();
-	global.curr_group = NULL;
-	setNewsModeAllgroups();
-	set_standard_message();
-	unset_busy();
-	break;
-    case NewsModeNewgroups: /* Unsubscribe */
 	i = ScrListGetFirstSelected(main_widgets.group_list);
 
 	if (i < 0) {
@@ -232,16 +297,59 @@ void knapp2_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 	    return;
 	}
 
-	set_busy(False, True);
-	while (i >= 0) {
-	    for (j = 0 ; j < global.no_groups ; j++) {
+	set_busy(False);
+	do {
+	    global.groups[i]->subscribed = False;
+	    i = ScrListGetNextSelected(main_widgets.group_list, i);
+	} while (i >= 0);
+	sort_groups();
+	global.curr_group = NULL;
+	setNewsModeAllgroups(NULL);
+	set_standard_message();
+	unset_busy();
+	break;
+    case NewsModeSomegroups: /* Unsubscribe */
+	if (global.busy)
+	    return;
+
+	i = ScrListGetFirstSelected(main_widgets.group_list);
+	if (i < 0) {
+	    set_message("No selected groups!", True);
+	    return;
+	}
+
+	set_busy(False);
+	j = 0;
+	do {
+	    for ( ; j < global.no_groups ; j++)
+		if (global.groups[j]->disp == i) {
+		    global.groups[j]->subscribed = True;
+		    break;
+		}
+	    i = ScrListGetNextSelected(main_widgets.group_list, i);
+	} while (i >= 0);
+	sort_groups();
+	global.curr_group = NULL;
+	setNewsModeConnected();
+	set_standard_message();
+	unset_busy();
+	break;
+    case NewsModeNewgroups: /* Unsubscribe */
+	i = ScrListGetFirstSelected(main_widgets.group_list);
+	if (i < 0) {
+	    set_message("No selected groups!", True);
+	    return;
+	}
+
+	set_busy(False);
+	do {
+	    for (j = 0 ; j < global.no_groups ; j++)
 		if (global.groups[j]->disp == i) {
 		    global.groups[j]->subscribed = False;
 		    break;
 		}
-	    }
 	    i = ScrListGetNextSelected(main_widgets.group_list, i);
-	}
+	} while (i >= 0);
 	setNewsModeNewgroups();
 	unset_busy();
 	break;
@@ -254,78 +362,48 @@ void knapp2_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 /* Kill... */
 void knapp5_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 {
-    if (global.mode != NewsModeDisconnected)
-	popup_kill_editor();
+    switch (global.mode) {
+    case NewsModeDisconnected:
+	break;
+    case NewsModeConnected:
+    case NewsModeAllgroups:
+    case NewsModeSomegroups:
+    case NewsModeNewgroups:
+	kill_edit_popup(NULL);
+	break;
+    case NewsModeGroup:
+    case NewsModeThread:
+	kill_edit_popup(global.curr_group);
+	break;
+    }
 }
 
 /* Update   Catchup */
 void knapp6_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 {
-    ART_LIST_NODE	*temp;
-    int			kill_flag;
+    char	*msg;
 
-    if (global.busy) return;
+    if (global.busy)
+	return;
 
     switch (global.mode) {
-    case NewsModeConnected: /* Update */
-    case NewsModeAllgroups:
-	set_busy(False, True);
-
-	kill_flag = update_kill_file();
-	if (update_newsrc()) {
-	    if (kill_flag)
-		set_message("Newsrc and kill files updated.", False);
-	    else
-		set_message("Failed to update kill file! "
-			    "  Newsrc file updated.", True);
-	} else {
-	    if (kill_flag)
-		set_message("Failed to update newsrc! "
-			    "  Kill file updated.", True);
-	    else
-		set_message("Failed to update newsrc and kill files!", True);
-	}
-
-	unset_busy();
-	break;
-    case NewsModeGroup: /* Catchup */
-	set_busy(False, True);
-
-	popdown_save();
-	popdown_search();
-	clear_tagged_articles();
-	partial_clear_cache();
-	clear_history();
-
-	free_read_arts_list(global.curr_group);
-	temp = global.curr_group->read_arts =
-	    (ART_LIST_NODE *)XtMalloc(sizeof(ART_LIST_NODE));
-	temp->next = NULL;
-	temp->first = 1;
-	temp->last = global.curr_group->last_art;
-
-	if (res_process_xrefs()) {
-	    ARTICLE	*art;
-
-	    for (art = get_articles(main_thr) ; art ; art = art->next)
-		if (!art->read)
-		    process_xref(art);
-
-	}
-
-	cache_leave_group();
-	thread_ahead_leave_group(global.curr_group);
-	global.curr_art = NULL;
-	global.curr_subj = NULL;
-	res_enter_group("none");
-
-	setNewsModeConnected();
-
-	unset_busy();
-	break;
     case NewsModeDisconnected:
     case NewsModeThread:
+	break;
+    case NewsModeGroup: /* Catchup */
+	if (global.confirm_catchup && global.curr_group->no_unread > 0)
+	    do_confirm("Really catchup?", True);
+	else
+	    leave_group(True);
+	break;
+    case NewsModeConnected: /* Update */
+    case NewsModeSomegroups:
+    case NewsModeAllgroups:
     case NewsModeNewgroups:
+	set_busy(False);
+	msg = do_update();
+	set_message(msg ? msg : "Newsrc and kill files updated.", !!msg);
+	unset_busy();
 	break;
     }
 }
@@ -334,15 +412,19 @@ void knapp6_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 void knapp7_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 {
     ARTICLE	*art;
-    Boolean	new_thread;
+    int		new_thread;
 
-    if (global.busy) return;
+    if (global.busy)
+	return;
 
     switch (global.mode) {
     case NewsModeConnected: /* Read group */
 	set_curr_group();
-	/* fall through */
+	/*
+	 * FALL THROUGH
+	 */
     case NewsModeAllgroups:
+    case NewsModeSomegroups:
 	if (!global.curr_group) {
 	    set_message("No selected group!", True);
 	    return;
@@ -408,13 +490,9 @@ void knapp7_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 
 	set_curr_art(art, True);
 	if (art) {
-	    if (read_article(art, False, NULL)) {
-		if (global.mode == NewsModeThread) {
-		    if (new_thread)
-			setNewsModeThread();
-		} else
-		    set_curr_art(art, False);
-	    }
+	    if (global.mode == NewsModeThread && new_thread)
+		setNewsModeThread();
+	    read_article(art, False, NULL, NULL);
 	} else {
 	    if (global.mode == NewsModeThread)
 		setNewsModeGroup(False);
@@ -433,10 +511,11 @@ void knapp7_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 void knapp8_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 {
     ARTICLE	*art, *old;
-    Boolean	new_thread;
+    int		new_thread;
     char	*buffer;
 
-    if (global.busy) return;
+    if (global.busy)
+	return;
 
     switch (global.mode) {
     case NewsModeGroup:  /* Previous */
@@ -458,7 +537,7 @@ void knapp8_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 	    old->read = False;
 	    old->subject->no_unread++;
 	    global.curr_group->no_unread++;
-	    if (old->hot > 0) {
+	    if (old->pixmap != None) {
 		global.n_hot++;
 		update_subj_hot_value(old->subject);
 	    }
@@ -466,11 +545,10 @@ void knapp8_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 	    if (global.mode == NewsModeThread) {
 		ArtTreeNodeSetInner(main_widgets.arttree,
 				    (ART_TREE_NODE *)old, True);
-		if (old->hot > 0) {
+		if (old->pixmap != None)
 		    ArtTreeNodeSetPixmap(main_widgets.arttree,
 					 (ART_TREE_NODE *)old,
-					 global.hot_pixmaps[old->hot]);
-		}
+					 old->pixmap);
 	    }
 	}
 
@@ -480,7 +558,7 @@ void knapp8_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 	    new_thread = True;
 
 	set_curr_art(art, True);
-	if (read_article(art, False, NULL)) {
+	if (read_article(art, False, NULL, NULL)) {
 	    if (global.mode == NewsModeThread) {
 		if (new_thread)
 		    setNewsModeThread();
@@ -491,7 +569,7 @@ void knapp8_callback(Widget gw, XtPointer client_data, XtPointer call_data)
     case NewsModeConnected: /* Rescan */
 	remove_rescan_timeout();
 
-	set_busy(True, True);
+	set_busy(True);
 	set_message("Server contacted, waiting for response...", False);
 	buffer = rescan();
 	if (!buffer) {
@@ -514,22 +592,12 @@ void knapp8_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 	break;
     case NewsModeDisconnected:
     case NewsModeAllgroups:
+    case NewsModeSomegroups:
     case NewsModeNewgroups:
 	break;
     }
 }
 
-/* Abort */
-void knapp9_callback(Widget gw, XtPointer client_data, XtPointer call_data)
-{
-    if (global.busy && !global.aborted) {
-	global.aborted = True;
-	if (global.quit_nicely)
-	    server_write_raw(main_server, "\r\nQUIT\r\n", 8);
-	server_close(main_server);
-    }
-}
-
 /* Save... */
 void knapp10_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 {
@@ -541,9 +609,10 @@ void knapp10_callback(Widget gw, XtPointer client_data, XtPointer call_data)
     case NewsModeDisconnected:
     case NewsModeConnected:
     case NewsModeAllgroups:
+    case NewsModeSomegroups:
     case NewsModeNewgroups:
 	if (global.bell)
-	    XBell(XtDisplay(gw), 0);
+	    XBell(display, 0);
 	break;
     }
 }
@@ -559,157 +628,32 @@ void knapp11_callback(Widget gw, XtPointer client_data, XtPointer call_data)
 	break;
     case NewsModeDisconnected:
     case NewsModeConnected:
+    case NewsModeSomegroups:
     case NewsModeNewgroups:
 	if (global.bell)
-	    XBell(XtDisplay(gw), 0);
+	    XBell(display, 0);
 	break;
     }
 }
 
-void top_vbar_callback(Widget gw, XtPointer client_data, XtPointer call_data)
-{
-    ScrBarReport	*report = (ScrBarReport *)call_data;
-    long		shown, size;
-    Arg			arg;
-
-    switch (global.mode) {
-    case NewsModeAllgroups:
-    case NewsModeConnected:
-    case NewsModeNewgroups:
-	ScrListSetFirst(main_widgets.group_list, report->slider_position);
-	ScrListGetFirstShownSize(main_widgets.group_list,
-				 NULL, &shown, &size);
-	if ((report->slider_length != shown ||
-	     report->canvas_length != size) &&
-	    report->slider_position + shown < size)
-	    ScrBarSetLengthsAndPos(main_widgets.top_vbar,
-				   size, shown, report->slider_position);
-	break;
-    case NewsModeGroup:
-	ScrListSetFirst(main_widgets.thread_list, report->slider_position);
-	ScrListGetFirstShownSize(main_widgets.thread_list,
-				 NULL, &shown, &size);
-	if ((report->slider_length != shown ||
-	     report->canvas_length != size) &&
-	    report->slider_position + shown < size)
-	    ScrBarSetLengthsAndPos(main_widgets.top_vbar,
-				   size, shown, report->slider_position);
-	break;
-    case NewsModeThread:
-	XtSetArg(arg, XtNy, -report->slider_position);
-	XtSetValues(main_widgets.arttree, &arg, 1);
-	break;
-    case NewsModeDisconnected:
-	break;
-    }
-}
-
-void top_hbar_callback(Widget gw, XtPointer client_data, XtPointer call_data)
-{
-    ScrBarReport	*report = (ScrBarReport *)call_data;
-
-    if (global.mode == NewsModeThread) {
-	Arg	arg;
-
-	XtSetArg(arg, XtNx, -report->slider_position);
-	XtSetValues(main_widgets.arttree, &arg, 1);
-    }
-}
-
-void text_scrbar_callback(Widget gw, XtPointer client_data,
-			  XtPointer call_data)
-{
-    ScrBarReport	*report = (ScrBarReport *)call_data;
-    long		first, shown, size;
-
-    ArtTextSetFirst(main_widgets.text, report->slider_position);
-    ArtTextGetFirstShownSize(main_widgets.text, &first, &shown, &size);
-    if (report->slider_position != first ||
-	report->slider_length   != shown ||
-	report->canvas_length   != size)
-	ScrBarSetLengthsAndPos(main_widgets.text_scrbar, size, shown, first);
-}
-
-void top_manager_callback(Widget gw, XtPointer client_data,
-			  XtPointer call_data)
-{
-    long	first, shown, size;
-    Position	s_x, s_y;
-    Dimension	s_w, s_h, c_w, c_h;
-    Arg		args[4];
-
-    switch (global.mode) {
-    case NewsModeConnected:
-    case NewsModeAllgroups:
-    case NewsModeNewgroups:
-	ScrListGetFirstShownSize(main_widgets.group_list,
-				 &first, &shown, &size);
-	ScrBarSetLengthsAndPos(main_widgets.top_vbar, size, shown, first);
-	break;
-    case NewsModeThread:
-	XtSetArg(args[0], XtNx, &s_x);
-	XtSetArg(args[1], XtNy, &s_y);
-	XtSetArg(args[2], XtNwidth, &c_w);
-	XtSetArg(args[3], XtNheight, &c_h);
-	XtGetValues(main_widgets.arttree, args, 4);
-	XtSetArg(args[0], XtNwidth, &s_w);
-	XtSetArg(args[1], XtNheight, &s_h);
-	XtGetValues(main_widgets.top_manager, args, 2);
-	ScrBarSetLengthsAndPos(main_widgets.top_vbar, c_h, s_h, - s_y);
-	ScrBarSetLengthsAndPos(main_widgets.top_hbar, c_w, s_w, - s_x);
-	break;
-    case NewsModeGroup:
-	ScrListGetFirstShownSize(main_widgets.thread_list,
-				 &first, &shown, &size);
-	ScrBarSetLengthsAndPos(main_widgets.top_vbar, size, shown, first);
-	break;
-    case NewsModeDisconnected:
-	break;
-    }
-}
-
-void text_resize_callback(Widget gw, XtPointer client_data,
-			  XtPointer call_data)
-{
-    if (global.mode == NewsModeGroup || global.mode == NewsModeThread) {
-	ArtTextReport	*report = (ArtTextReport *)call_data;
-
-	ScrBarSetLengthsAndPos(main_widgets.text_scrbar,
-			       report->canvas_length,
-			       report->slider_length,
-			       report->slider_position);
-    }
-}
-
-void arttree_scroll_callback(Widget gw,
-			     XtPointer client_data,
-			     XtPointer call_data)
-{
-    ArtTreeScrollReport	*report = (ArtTreeScrollReport *)call_data;
-
-    if (global.mode == NewsModeThread) {
-	ScrBarSetSliderPosition(main_widgets.top_hbar, report->slider_x);
-	ScrBarSetSliderPosition(main_widgets.top_vbar, report->slider_y);
-    }
-}
-
 void thread_list_sel_callback(Widget gw,
 			      XtPointer client_data,
 			      XtPointer call_data)
 {
     long	row = (long)call_data;
-    SUBJECT	*loop;
+    SUBJECT	*loop = global.curr_subj;
 
     if (global.busy || global.mode != NewsModeGroup)
 	return;
 
-    global.curr_art = NULL;
-    if (global.curr_subj && global.curr_subj->disp == row)
-	return;
+    if (!loop || loop->disp != row)
+	for (loop = get_subjects(main_thr) ; loop ; loop = loop->next)
+	    if (loop->disp == row)
+		break;
 
-    for (loop = get_subjects(main_thr) ; loop ;	loop = loop->next)
-	if (loop->disp == row)
-	    break;
+    if (!loop || !global.curr_art ||
+	global.curr_art->subject->thread != loop->thread)
+	global.curr_art = NULL;
 
     global.curr_subj = loop;
 }
@@ -720,7 +664,7 @@ void thread_list_callback(Widget gw,
 {
     long	row = (long)call_data;
     SUBJECT	*subj;
-    ARTICLE	*art = NULL;
+    ARTICLE	*art = NULL, *first = NULL;
 
     if (global.busy || global.mode != NewsModeGroup)
 	return;
@@ -740,17 +684,27 @@ void thread_list_callback(Widget gw,
     }
 
     for (art = subj->thread ; art ; art = next_in_thread_preorder(art))
-	if (art->subject == subj && art->from && !art->read)
-	    break;
-
-    global.curr_art = art;
-    read_article(art, False, NULL);
+	if (art->subject == subj && art->from)
+	    if (!art->read)
+		break;
+	    else if (!first)
+		first = art;
 
     if (!art) {
-	if (global.bell)
-	    XBell(XtDisplay(gw), 0);
-	set_message("No unread article with that subject!", True);
+	art = global.curr_art;
+	if (!art || art->subject != subj)
+	    art = first;
+	else
+	    while ((art = next_in_thread_preorder(art)))
+		if (art->from && art->subject == subj)
+		    break;
     }
+
+    global.curr_art = art;
+    read_article(art, False, NULL, NULL);
+
+    if (!art && global.bell)
+	XBell(display, 0);
 }
 
 void arttree_sel_callback(Widget gw,
@@ -759,12 +713,11 @@ void arttree_sel_callback(Widget gw,
 {
     ARTICLE	*art = (ARTICLE *)call_data;
 
-    if (global.busy) return;
+    if (global.busy || global.mode != NewsModeThread)
+	return;
 
-    if (global.mode == NewsModeThread) {
-	set_curr_art(art, False);
-	read_article(art, False, NULL);
-    }	
+    set_curr_art(art, False);
+    read_article(art, False, NULL, NULL);
 }
 
 void group_list_callback(Widget gw,
@@ -772,33 +725,36 @@ void group_list_callback(Widget gw,
 			 XtPointer call_data)
 {
     long	row = (long)call_data;
+    long	n;
 
-    if (global.busy) return;
+    if (global.busy)
+	return;
 
     global.curr_group = NULL;
 
-    if (global.mode == NewsModeAllgroups) {
+    switch (global.mode) {
+    case NewsModeDisconnected:
+    case NewsModeGroup:
+    case NewsModeThread:
+    case NewsModeNewgroups:
+	break;
+    case NewsModeAllgroups:
 	popdown_search();
 	global.curr_group = global.groups[row];
-    } else if (global.mode == NewsModeConnected) {
-	GROUP		**loop = global.groups;
-	long		n = global.no_groups;
-
-	while (n > 0 && (*loop)->subscribed) {
-	    if ((*loop)->disp == row) {
-		global.curr_group = *loop;
+	break;
+    case NewsModeConnected:
+    case NewsModeSomegroups:
+	for (n = 0 ; n < global.no_groups ; n++)
+	    if (global.groups[n]->disp == row) {
+		global.curr_group = global.groups[n];
 		break;
 	    }
-	    loop++;
-	    n--;
-	}
-
-	if (!global.curr_group)
-	    return;
-    } else {
-	return;
+	break;
     }
 
+    if (!global.curr_group)
+	return;
+
     read_group(NULL, True, 0);
 }
 
@@ -806,9 +762,10 @@ void group_list_sel_callback(Widget w,
 			     XtPointer client_data,
 			     XtPointer call_data)
 {
-    long	row;
+    long	row, i;
 
-    if (global.busy) return;
+    if (global.busy)
+	return;
 
     switch (global.mode) {
     case NewsModeDisconnected:
@@ -817,20 +774,16 @@ void group_list_sel_callback(Widget w,
     case NewsModeThread:
 	break;
     case NewsModeConnected:
+    case NewsModeSomegroups:
 	row = (long)call_data;
-	if (row >= 0) {
-	    long	i;
-
-	    for (i = 0 ; i < global.no_groups ; i++) {
-		if (!global.groups[i]->subscribed)
-		    break;
-		if (global.groups[i]->disp == row) {
-		    global.curr_group = global.groups[i];
-		    return;
-		}
+	global.curr_group = NULL;
+	if (row < 0)
+	    break;
+	for (i = 0 ; i < global.no_groups ; i++)
+	    if (global.groups[i]->disp == row) {
+		global.curr_group = global.groups[i];
+		return;
 	    }
-	    global.curr_group = NULL;
-	}
 	break;
     case NewsModeAllgroups:
 	row = (long)call_data;
@@ -866,14 +819,17 @@ void group_list_dnd_callback(Widget w,
 	    global.groups[index[1]] = temp;
 	    break;
 	}
-	/* fall through */
+	/*
+	 * FALL THROUGH
+	 */
     case NewsModeDisconnected:
+    case NewsModeConnected:
     case NewsModeGroup:
     case NewsModeThread:
-    case NewsModeConnected:
+    case NewsModeSomegroups:
     case NewsModeNewgroups:
 	if (global.bell)
-	    XBell(XtDisplay(w), 0);
+	    XBell(display, 0);
 	break;
     }
 }
@@ -885,7 +841,7 @@ void delete_window_callback(Widget w,
     if (global.busy)
 	return;
 
-    quit(True);
+    disconnect(True);
 }
 
 void sash_callback(Widget w, XtPointer client_data, XtPointer call_data)
@@ -893,25 +849,25 @@ void sash_callback(Widget w, XtPointer client_data, XtPointer call_data)
     int			y, i1, i2, i3;
     unsigned int	mods;
     Window		w1, w2;
-    Arg			arg[2];
-    Dimension		width, height;
+    Arg			arg;
+    Dimension		height;
 
     /*
      *  Avoid race condition.
      */
 
-    XQueryPointer(XtDisplay(w), XtWindow(w),
+    XQueryPointer(display, XtWindow(w),
 		  &w1, &w2, &i1, &i2, &i3, &y, &mods);
     if (y == 0 || mods == 0)
 	return;
 
-    XtSetArg(arg[0], XtNheight, &height);
-    XtSetArg(arg[1], XtNwidth, &width);
-    XtGetValues(main_widgets.top_layout, arg, 2);
+    XtSetArg(arg, XtNheight, &height);
+    XtGetValues(main_widgets.top_layout, &arg, 1);
     y += height;
     if (y <= 0)
 	y = 1;
     else if (y > 32767)
 	y = 32767;
-    (void)XtMakeResizeRequest(main_widgets.top_layout, width, y, NULL, NULL);
+    XtSetArg(arg, XtNheight, y);
+    XtSetValues(main_widgets.top_layout, &arg, 1);
 }
