@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #include "global.h"
 #include "cache.h"
@@ -17,13 +17,14 @@
 #include "codes.h"
 #include "connect.h"
 #include "decode.h"
-#include "kill.h"
+#include "expand.h"
 #include "mailcap.h"
 #include "parse.h"
 #include "partial.h"
 #include "font.h"
 #include "read.h"
 #include "resource.h"
+#include "save.h"
 #include "server.h"
 #include "tag.h"
 #include "util.h"
@@ -33,8 +34,7 @@
 #include "../Widgets/ArtText.h"
 #include "../Widgets/ArtTree.h"
 #include "../Widgets/Dialogue.h"
-#include "../Widgets/ScrBar.h"
-#include "../Widgets/ScrList.h"
+#include "../Widgets/Scrollable.h"
 
 #define STARTS_AS_BOUNDARY(buffer, boundary, len)          \
     (boundary && buffer[0] == '-' && buffer[1] =='-' &&    \
@@ -62,11 +62,25 @@ static void decode_header(char *header)
     char	*c;
     long	len;
 
-    while ((c = next_enc_word(header, &data))) {
-	if (font && font->fid != data.font->fid && append)
-	    ArtTextAddLine(main_widgets.text, "    ",
-			   data.font, global.header_pixel);
-	font = data.font;
+    for (;;) {
+	for (c = next_enc_word(header, &data) ; c ;
+	     c = next_enc_word(c + 1, &data)) {
+	    MimeFont	*f;
+
+	    (c + 2)[data.ch_len] = '\0';
+	    f = get_font(c + 2);
+	    (c + 2)[data.ch_len] = '?';
+	    if (f && f->header_font && (!f->funcs || f->head_enc_hack)) {
+		if (font && font->fid != f->header_font->fid && append)
+		    ArtTextAddLine(main_widgets.text, "    ",
+				   f->header_font, global.header_pixel);
+		font = f->header_font;
+		break;
+	    }
+	}
+
+	if (!c)
+	    break;
 
 	if (c > header) {
 	    *c = '\0';
@@ -212,40 +226,43 @@ static int insert_headers(char **headers, int n, int full_header)
     return added_some;
 }
 
-static char *get_mime_body(SERVER *server, char *buffer,
-			   char **data_ret, long *len_ret,
-			   MimeEnc enc, char *bound, long len)
+static char *get_mime_body(SERVER	 *server,
+			   char		 *buffer,
+			   char		**datap,
+			   long		 *lenp,
+			   int		  enc,
+			   char		 *bound,
+			   long		  bound_len)
 {
-    char	*data    = NULL;
-    long	alloced  = 0;
+    long	alloced  = 65536;
+    char	*data = XtMalloc(alloced);
     long	data_len = 0;
     long	n;
     int		tmp, soft;
 
 #define REALLOC(data, alloced, data_len, n)        \
     if (n + data_len + 8 >= alloced) {             \
-	alloced = 2 * alloced + n + data_len + 8;  \
-	data = XtRealloc(data, alloced);           \
+        alloced = 2 * alloced + n + data_len + 8;  \
+        data = XtRealloc(data, alloced);           \
     }
 
     switch (enc) {
-    case MimeEncUnknown:
-	if (enc == MimeEncUnknown)
-	    ArtTextAddLine(main_widgets.text,
-			   "[knews: unrecoginzed Content-Transfer-Encoding.]",
-			   ascii_font->body_font, global.alert_pixel);
-	/* Fall through */
+    default:
+	ArtTextAddLine(main_widgets.text,
+		       "[knews: unrecoginzed Content-Transfer-Encoding.]",
+		       ascii_font->body_font, global.alert_pixel);
+	/*
+	 *  Fall through.
+	 */
     case MimeEncNone:
 	while (buffer && !IS_DOT(buffer) &&
-	       !IS_BOUNDARY(buffer, bound, len)) {
-	    int	dot;
-
+	       !IS_BOUNDARY(buffer, bound, bound_len)) {
+	    if (*buffer == '.')
+		buffer++;
 	    n = strlen(buffer);
+	    buffer[n++] = '\n';
 	    REALLOC(data, alloced, data_len, n);
-	    buffer[n++] = '\n';  /* Maybe CRLF? */
-	    dot = (*buffer == '.');
-	    memmove(data + data_len, buffer + dot, n - dot);
-	    data_len += n;
+	    memcpy(data + data_len, buffer, n);
 	    buffer = server_read(server);
 	}
 	break;
@@ -254,40 +271,40 @@ static char *get_mime_body(SERVER *server, char *buffer,
 	    B64Context	bc = {0, };
 
 	    while (buffer && !IS_DOT(buffer) &&
-		   !IS_BOUNDARY(buffer, bound, len)) {
-		n = strlen(buffer);
-		REALLOC(data, alloced, data_len, n);
-		n = decode_base64(&bc, data + data_len, buffer, n);
-		if (n > 0)
-		    data_len += n;
-		buffer = server_read(server);
-	    }
+                   !IS_BOUNDARY(buffer, bound, bound_len)) {
+                n = strlen(buffer);
+                REALLOC(data, alloced, data_len, n);
+                n = decode_base64(&bc, data + data_len, buffer, n);
+                if (n > 0)
+                    data_len += n;
+                buffer = server_read(server);
+            }
 
-	    n = decode_base64(&bc, data + data_len, NULL, 0);
-	    if (n >= 0)
-		data_len += n;
+            n = decode_base64(&bc, data + data_len, NULL, 0);
+            if (n >= 0)
+                data_len += n;
 
-	    tmp = base64_status(&bc);
-	    if (tmp & BASE64_ERROR)
-		ArtTextAddLine(main_widgets.text,
-			       "[knews: base64 decode error.]",
-			       ascii_font->body_font, global.alert_pixel);
-	    if (tmp & BASE64_TRAILING_GARBAGE)
-		ArtTextAddLine(main_widgets.text,
-			       "[knews: trailing garbage in base64 data.]",
-			       ascii_font->body_font, global.alert_pixel);
+            tmp = base64_status(&bc);
+            if (tmp & BASE64_ERROR)
+                ArtTextAddLine(main_widgets.text,
+                               "[knews: base64 decode error.]",
+                               ascii_font->body_font, global.alert_pixel);
+            if (tmp & BASE64_TRAILING_GARBAGE)
+                ArtTextAddLine(main_widgets.text,
+                               "[knews: trailing garbage in base64 data.]",
+                               ascii_font->body_font, global.alert_pixel);
 	}
 	break;
     case MimeEncQP:
 	while (buffer && !IS_DOT(buffer) &&
-	       !IS_BOUNDARY(buffer, bound, len)) {
+	       !IS_BOUNDARY(buffer, bound, bound_len)) {
+	    if (*buffer == '.')
+		buffer++;
 	    n = strlen(buffer);
 	    REALLOC(data, alloced, data_len, n);
-	    n = decode_qp(buffer, buffer, n, &soft, False);
+	    data_len += decode_qp(data + data_len, buffer, n, &soft, False);
 	    if (!soft)
-		buffer[n++] = '\n';
-	    memmove(data + data_len, buffer, n);
-	    data_len += n;
+		data[data_len++] = '\n';
 	    buffer = server_read(server);
 	}
 	break;
@@ -295,39 +312,32 @@ static char *get_mime_body(SERVER *server, char *buffer,
 	{
 	    UueContext	uc = {0, };
 
-	    while (buffer && !IS_DOT(buffer) &&
-		   !IS_BOUNDARY(buffer, bound, len)) {
-		n = decode_uue(&uc, buffer, buffer, strlen(buffer));
-		if (n > 0) {
-		    REALLOC(data, alloced, data_len, n);
-		    memmove(data + data_len, buffer, n);
-		    data_len += n;
-		}
-		buffer = server_read(server);
-	    }
+            while (buffer && !IS_DOT(buffer) &&
+                   !IS_BOUNDARY(buffer, bound, bound_len)) {
+		n = strlen(buffer);
+		REALLOC(data, alloced, data_len, n);
+                n = decode_uue(&uc, data + data_len, buffer, n);
+		if (n > 0)
+                    data_len += n;
+                buffer = server_read(server);
+            }
 
-	    tmp = uue_status(&uc);
-	    if (tmp & UUE_ERROR)
-		ArtTextAddLine(main_widgets.text,
-			       "[knews: uudecode error]",
-			       ascii_font->body_font, global.alert_pixel);
-	    if (tmp & UUE_NO_BEGIN)
-		ArtTextAddLine(main_widgets.text,
-			       "[knews: uudecode error: no begin line]",
-			       ascii_font->body_font, global.alert_pixel);
-	    if (tmp & UUE_NO_END)
-		ArtTextAddLine(main_widgets.text,
-			       "[knews: uudecode error: no end line]",
-			       ascii_font->body_font, global.alert_pixel);
+            tmp = uue_status(&uc);
+            if (tmp & UUE_NO_BEGIN)
+                ArtTextAddLine(main_widgets.text,
+                               "[knews: uudecode error: no begin line]",
+                               ascii_font->body_font, global.alert_pixel);
+            else if (tmp & UUE_NO_END)
+                ArtTextAddLine(main_widgets.text,
+                               "[knews: uudecode error: no end line]",
+                               ascii_font->body_font, global.alert_pixel);
+            else if (tmp & UUE_ERROR)
+                ArtTextAddLine(main_widgets.text,
+                               "[knews: uudecode error]",
+                               ascii_font->body_font, global.alert_pixel);
 	}
 	break;
-    default:
-	while (buffer && !IS_DOT(buffer) &&
-	       !IS_BOUNDARY(buffer, bound, len))
-	    buffer = server_read(server);
-	break;
     }
-
 #undef REALLOC
 
     if (!buffer) {
@@ -335,15 +345,11 @@ static char *get_mime_body(SERVER *server, char *buffer,
 	return NULL;
     }
 
-    data[data_len] = '\0';
-    data = XtRealloc(data, data_len + 1);
-
-    *data_ret = data;
-    *len_ret  = data_len;
+    *datap = data;
+    *lenp  = data_len;
 
     return buffer;
 }
-
 static void add_body_line(char *buffer, regex_t *re,
 			  MimeFont *f, void *dec_data, int is_lf_term)
 {
@@ -409,7 +415,7 @@ static void add_body_line_multi(char *buffer, regex_t *re,
 }
 
 static char *insert_body(SERVER *server, char *buffer, MimeFont *f,
-			 MimeEnc enc, char *bound, int len)
+			 int enc, char *bound, int len)
 {
     regex_t	*re = res_quote_regexp();
     long	n;
@@ -420,11 +426,13 @@ static char *insert_body(SERVER *server, char *buffer, MimeFont *f,
 	dec_data = f->funcs->init();
 
     switch (enc) {
-    case MimeEncUnknown:
+    default:
 	ArtTextAddLine(main_widgets.text,
 		       "[knews: unrecoginzed Content-Transfer-Encoding.]",
 		       ascii_font->body_font, global.alert_pixel);
-	/* Fall through */
+	/*
+	 * Fall through
+	 */
     case MimeEncNone:
 	while (buffer && !IS_DOT(buffer) &&
 	       !IS_BOUNDARY(buffer, bound, len)) {
@@ -506,24 +514,19 @@ static char *insert_body(SERVER *server, char *buffer, MimeFont *f,
 	    }
 
 	    tmp = uue_status(&uc);
-	    if (tmp & UUE_ERROR)
-		ArtTextAddLine(main_widgets.text, "[knews: uudecode error]",
-			       ascii_font->body_font, global.alert_pixel);
 	    if (tmp & UUE_NO_BEGIN)
 		ArtTextAddLine(main_widgets.text,
 			       "[knews: uudecode error: no begin line]",
 			       ascii_font->body_font, global.alert_pixel);
-	    if (tmp & UUE_NO_END)
+	    else if (tmp & UUE_NO_END)
 		ArtTextAddLine(main_widgets.text,
 			       "[knews: uudecode error: no end line]",
 			       ascii_font->body_font, global.alert_pixel);
+	    else if (tmp & UUE_ERROR)
+		ArtTextAddLine(main_widgets.text, "[knews: uudecode error]",
+			       ascii_font->body_font, global.alert_pixel);
 	}
 	break;
-    default:
-	while (buffer && !IS_DOT(buffer) &&
-	       !IS_BOUNDARY(buffer, bound, len))
-	    buffer = server_read(server);
-	break;
     }
 
     if (f->funcs)
@@ -564,7 +567,7 @@ static char *do_multipart(SERVER *server, char *buffer, char *bound,
 		    ArtTextAddSeparator(main_widgets.text, 2, 4);
 		do_line = True;
 		buffer = do_mime(NULL, server, server_read(server),
-				 default_to_rfc822, bound, n);
+				 default_to_rfc822, bound, n, NULL);
 	    } while (buffer && !IS_DOT(buffer) &&
 		     !IS_LAST_BOUNDARY(buffer, bound, n));
 
@@ -584,14 +587,19 @@ static char *do_multipart(SERVER *server, char *buffer, char *bound,
     return buffer;
 }
 
-char *do_mime(ARTICLE *art, SERVER *server, char *buffer,
-	      int default_to_rfc822, char *ext_bound, int ext_len)
+char *do_mime(ARTICLE    *art,
+	      SERVER     *server,
+	      char       *buffer,
+	      int         default_to_rfc822,
+	      char       *ext_bound,
+	      int         ext_len,
+	      char      **mime_hack)
 {
-    MimeEnc	enc = MimeEncNone;
+    int		enc = MimeEncNone;
     char	**headers;
     char	type_buf[MIME_BUF_LEN + 1], subtype_buf[MIME_BUF_LEN + 1];
     MimeArg	args[N_ARGS + 1] = {{0, },};
-    int		no_alloc, n, i, ct_index;
+    int		no_alloc, n, i, ct_index, cd_index;
 
     n = 0;
     no_alloc = 16;
@@ -615,30 +623,41 @@ char *do_mime(ARTICLE *art, SERVER *server, char *buffer,
     }
 
     ct_index = -1;
+    cd_index = -1;
     for (i = 0 ; i < n ; i++)
-	if (case_lstrncmp(headers[i], "content-type:", 13) == 0)
-	    ct_index = i;
-	else if (case_lstrncmp(headers[i],
-			       "content-transfer-encoding:", 26) == 0)
-	    enc = parse_content_enc(headers[i]);
+	if (case_lstrncmp(headers[i], "content-", 8) == 0)
+	    if (case_lstrncmp(headers[i] + 8, "type:", 5) == 0)
+		ct_index = i;
+	    else if (case_lstrncmp(headers[i] + 8,
+				   "transfer-encoding:", 18) == 0)
+		enc = parse_content_enc(headers + i);
+	    else if (case_lstrncmp(headers[i] + 8, "disposition:", 12)== 0)
+		cd_index = i;
+
+    if (mime_hack)
+	enc = parse_content_enc(mime_hack + 2);
 
     if (buffer && buffer[0] == '\0')
 	buffer = server_read(server);
 
-    if (!ext_bound && art) { /* top level */
-	if (!art->from)
+    if (!ext_bound) { /* top level */
+	if (art && !art->from)
 	    realize_fake(art, headers, n);
-	if (!art->xref && !art->read && res_process_xrefs())
+	if (art && !art->xref && !art->read && res_process_xrefs())
 	    fake_xref(art, headers, n);
 	ArtTextClearLines(main_widgets.text);
-	ScrBarSetLengthsAndPos(main_widgets.text_scrbar, 0, 0, 0);
+	ScrollableSuspend(main_widgets.text);
     }
 
-    if (ct_index >= 0) {
-	if (!parse_content_type(headers + ct_index,
-				type_buf, sizeof type_buf,
+    if (mime_hack) {
+	if (parse_content_type(mime_hack, type_buf, sizeof type_buf,
+			       subtype_buf, sizeof subtype_buf,
+			       args, N_ARGS, False))
+	    ct_index = 0;
+    } else if (ct_index >= 0) {
+	if (!parse_content_type(headers + ct_index, type_buf, sizeof type_buf,
 				subtype_buf, sizeof subtype_buf,
-				args, N_ARGS))
+				args, N_ARGS, False))
 	    ct_index = -1;
     } else if (default_to_rfc822) {
 	strcpy(type_buf, "message");
@@ -655,8 +674,7 @@ char *do_mime(ARTICLE *art, SERVER *server, char *buffer,
     } else do { /* one pass loop to continue out of */
 	const MailcapData	*mailcap;
 	int		do_line;
-	char		*charset, *bound, *data;
-	long		len;
+	char		*charset, *bound;
 	MimeFont	*f;
 
 	ascii_lower(type_buf);
@@ -667,9 +685,17 @@ char *do_mime(ARTICLE *art, SERVER *server, char *buffer,
 
 	switch (type_buf[0]) {
 	case 't':
-	    if (strcmp(type_buf, "text") != 0 ||
-		(mailcap && strcmp(subtype_buf, "plain") != 0))
+	    if (strcmp(type_buf, "text") != 0)
 		break;
+	    if (strcmp(subtype_buf, "plain") != 0) {
+		if (mailcap)
+		    break;
+		ArtTextAddLine(main_widgets.text,
+			       "[knews: no mailcap viewer for 'text/",
+			       ascii_font->body_font, global.alert_pixel);
+		ArtTextAppendToLast(main_widgets.text, subtype_buf);
+		ArtTextAppendToLast(main_widgets.text, "'.]");
+	    }
 
 	    charset = get_charset(args);
 	    if (!charset)
@@ -678,11 +704,10 @@ char *do_mime(ARTICLE *art, SERVER *server, char *buffer,
 		f = get_font(charset);
 		if (!f) {
 		    ArtTextAddLine(main_widgets.text,
-				   "[knews: no font for charset ",
-				   ascii_font->body_font,
-				   global.alert_pixel);
+				   "[knews: no font for charset '",
+				   ascii_font->body_font, global.alert_pixel);
 		    ArtTextAppendToLast(main_widgets.text, charset);
-		    ArtTextAppendToLast(main_widgets.text, ".]");
+		    ArtTextAppendToLast(main_widgets.text, "'.]");
 		    if (default_font->funcs)
 			f = ascii_font;
 		    else
@@ -729,7 +754,7 @@ char *do_mime(ARTICLE *art, SERVER *server, char *buffer,
 	    } else if (strcmp(type_buf, "message") == 0) {
 		if (strcmp(subtype_buf, "rfc822") == 0) {
 		    buffer = do_mime(art, server, buffer,
-				     False, ext_bound, ext_len);
+				     False, ext_bound, ext_len, NULL);
 		    continue;
 		} else if (strcmp(subtype_buf, "partial") == 0) {
 		    buffer = insert_body(server, buffer,
@@ -742,7 +767,7 @@ char *do_mime(ARTICLE *art, SERVER *server, char *buffer,
 			   strcmp(subtype_buf, "external-body") == 0) {
 		    ArtTextAddLine(main_widgets.text,
 				   "[knews: no mailcap entry for "
-				   "message/external-body.]",
+				   "Content-Type: message/external-body.]",
 				   ascii_font->body_font, global.alert_pixel);
 		    buffer = insert_body(server, buffer,
 					 default_font, MimeEncNone,
@@ -753,44 +778,31 @@ char *do_mime(ARTICLE *art, SERVER *server, char *buffer,
 	    break;
 	}
 
-	buffer = get_mime_body(server, buffer, &data, &len,
-			       enc, ext_bound, ext_len);
-	if (server != main_server || buffer) {
-	    VIEWABLE	*viewable;
-	    char	*viewer;
+	{
+	    char	*view_cmd  = NULL;
+	    char	*file_name = NULL;
+	    char	*data;
+	    long	data_len;
 
 	    if (mailcap && mailcap->view_command)
-		viewer =
-		    expand_view_command(mailcap->view_command,
-					type_buf, subtype_buf, args,
-					mailcap->needsterminal,
-					mailcap->copiousoutput);
-	    else
-		viewer = NULL;
+		view_cmd = expand_view_command(mailcap->view_command,
+					       type_buf, subtype_buf, args,
+					       mailcap->needsterminal,
+					       mailcap->copiousoutput);
+	    if (cd_index >= 0 &&
+		(file_name = parse_content_disp(headers + cd_index)))
+		file_name = XtNewString(file_name);
 
-	    viewable = (VIEWABLE *)XtMalloc(sizeof(VIEWABLE));
-	    viewable->data      = data;
-	    viewable->len       = len;
-	    viewable->viewer    = viewer;
-	    viewable->ref_count = 1;
-
-	    i = ct_index;
-	    do {
-		ArtTextAddLine(main_widgets.text, headers[i],
-			       ascii_font->header_font, global.header_pixel);
-	    } while (headers[++i] && IS_SPACE(headers[i][0]));
-	    for (i = 0 ; i < n && headers[i] ; i++)
-		if (case_lstrncmp(headers[i],
-				  "content-disposition:", 20) == 0)
-		    break;
-	    if (i < n)
+	    if (!mime_hack) {
+		i = ct_index;
 		do {
 		    ArtTextAddLine(main_widgets.text, headers[i],
 				   ascii_font->header_font,
 				   global.header_pixel);
 		} while (headers[++i] && IS_SPACE(headers[i][0]));
-	    ArtTextAddLine(main_widgets.text, "",
-			   ascii_font->body_font, global.pixel);
+		ArtTextAddLine(main_widgets.text, "",
+			       ascii_font->body_font, global.pixel);
+	    }
 
 	    if (mailcap && mailcap->description) {
 		ArtTextAddLine(main_widgets.text,
@@ -802,26 +814,17 @@ char *do_mime(ARTICLE *art, SERVER *server, char *buffer,
 		ArtTextAppendToLast(main_widgets.text, "]");
 	    }
 
-	    if (!viewer)
-		ArtTextAddClickable(main_widgets.text,
-				    "[knews: no mailcap entry.]  "
-				    "Save or pipe.",
-				    ascii_font->header_font,
-				    global.clickable_pixel,
-				    click_callback, (XtPointer)viewable);
-	    else {
-		ArtTextAddClickable(main_widgets.text, "View with:  ",
-				    ascii_font->header_font,
-				    global.clickable_pixel,
-				    click_callback, (XtPointer)viewable);
-		ArtTextAppendToLast(main_widgets.text, viewer);
-	    }
-
-	    ArtTextAddLine(main_widgets.text, "",
-			   ascii_font->body_font, global.pixel);
+	    buffer = get_mime_body(server, buffer, &data, &data_len,
+				   enc, ext_bound, ext_len);
+	    if (buffer)
+		do_viewer(type_buf, subtype_buf, view_cmd,
+			  file_name, data, data_len);
 	}
     } while (0);
 
+    if (!ext_bound) /* toplevel */
+	ScrollableResume(main_widgets.text);
+
     for (i = 0 ; i < N_ARGS ; i++) {
 	XtFree(args[i].name);
 	XtFree(args[i].value);
@@ -834,13 +837,15 @@ char *do_mime(ARTICLE *art, SERVER *server, char *buffer,
     return buffer;    
 }
 
-static char *get_article(ARTICLE *art, int full_header, long *sel_data)
+static char *get_article(ARTICLE  *art,
+			 int       full_header,
+			 long     *sel_data,
+			 char    **mime_hack)
 {
     SERVER	*server = main_server;
     FILE	*cf = NULL;
     char	command[256];
     char	*buffer;
-    long	first, shown, size;
 
     if (art->no > 0) {
 	server = cache_get_server(art->no, True);
@@ -884,11 +889,12 @@ static char *get_article(ARTICLE *art, int full_header, long *sel_data)
     buffer = server_read(server);
 
     if (!full_header)
-	buffer = do_mime(art, server, buffer, False, NULL, 0);
+	buffer = do_mime(art, server, buffer, False, NULL, 0, mime_hack);
     else {
 	long	i;
 
 	ArtTextClearLines(main_widgets.text);
+	ScrollableSuspend(main_widgets.text);
 
 	i = -1;
 	while (buffer && buffer[0] != '\0' && !IS_DOT(buffer)) {
@@ -935,12 +941,11 @@ static char *get_article(ARTICLE *art, int full_header, long *sel_data)
 	}
     }
 
-    ArtTextGetFirstShownSize(main_widgets.text, &first, &shown, &size);
-    ScrBarSetLengthsAndPos(main_widgets.text_scrbar, size, shown, first);
+    ScrollableResume(main_widgets.text);
 
     if (cf) {
 	server_set_bs(server, NULL);
-	if (fclose(cf) != EOF && buffer)
+	if (fclose(cf) == 0 && buffer)
 	    cache_fetch_done(art->no);
 	else
 	    cache_fetch_failed(art->no);
@@ -962,7 +967,7 @@ static char *get_article(ARTICLE *art, int full_header, long *sel_data)
 	    ArtTreeNodeSetPixmap(main_widgets.arttree,
 				 (ART_TREE_NODE *)art, None);
 	}
-	if (art->hot > 0) {
+	if (art->pixmap != None) {
 	    global.n_hot--;
 	    update_subj_hot_value(art->subject);
 	}
@@ -975,13 +980,17 @@ static char *get_article(ARTICLE *art, int full_header, long *sel_data)
     return CODE_TO_STR(NNTP_OK_ARTICLE);
 }
 
-int read_article(ARTICLE *art, int full_header, long *sel_data)
+int read_article(ARTICLE  *art,
+		 int       full_header,
+		 long     *sel_data,
+		 char    **mime_hack)
 {
     char	*reply;
 
     if (art) {
-	set_busy(True, True);
-	reply = get_article(art, full_header || res_full_header(), sel_data);
+	set_busy(True);
+	reply = get_article(art, full_header || res_full_header(),
+			    sel_data, mime_hack);
 	if (!reply) {
 	    reconnect_server(True);
 	    unset_busy();
@@ -998,15 +1007,63 @@ int read_article(ARTICLE *art, int full_header, long *sel_data)
 
 	    if (strlen(reply) > 200)
 		reply[200] ='\0';
-	    sprintf(message,
-		    "Error!  Message from server is: %s", reply);
+	    sprintf(message, "Error!  Message from server is: %s", reply);
 	    set_message(message, True);
 	}
     } else {
 	ArtTextClearLines(main_widgets.text);
-	ScrBarSetLengthsAndPos(main_widgets.text_scrbar, 0, 0, 0);
 	set_standard_message();
     }
 
     return True;
 }
+
+/*********************************************************************/
+
+void action_read_article(Widget    w,
+			 XEvent   *event,
+			 String   *params,
+			 Cardinal *no_params)
+{
+    if (global.busy || (global.mode != NewsModeGroup &&
+			global.mode != NewsModeThread))
+	return;
+
+    if (global.curr_art)
+	read_article(global.curr_art, *no_params != 0, NULL, NULL);
+}
+
+void action_mime_hack(Widget    w,
+		      XEvent   *event,
+		      String   *params,
+		      Cardinal *no_params)
+{
+    char	*mime_hack[4];
+    char	*ct, *cte;
+
+    if (global.busy || (global.mode != NewsModeGroup &&
+			global.mode != NewsModeThread))
+	return;
+
+    if (!global.curr_art) {
+	set_message("No selected article!", True);
+	return;
+    } else if (*no_params != 2) {
+	set_message("Wrong number of parameters to mime-hack()!", True);
+	return;
+    }
+
+    ct  = XtMalloc(16 + strlen(params[0]));
+    cte = XtMalloc(32 + strlen(params[1]));
+    sprintf(ct,  "Content-Type: %s",             params[0]);
+    sprintf(cte, "Content-Transfer-Ecoding: %s", params[1]);
+    mime_hack[0] = ct;
+    mime_hack[1] = NULL;
+    mime_hack[2] = cte;
+    mime_hack[3] = NULL;
+
+    read_article(global.curr_art, False, NULL, mime_hack);
+
+    XtFree(ct);
+    XtFree(cte);
+}
