@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,10 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
-extern int	 read_article(ARTICLE*, int, long*);
-extern char	*do_mime(ARTICLE*, struct SERVER*, char *, int, char*, int);
+extern int	 read_article(ARTICLE*, int, long*, char**);
+extern char	*do_mime(ARTICLE*, struct SERVER*,
+			 char *, int, char*, int, char**);
+extern void	 action_read_article(Widget, XEvent*, String*, Cardinal*);
+extern void	 action_mime_hack(Widget, XEvent*, String*, Cardinal*);
