@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,9 +9,11 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #include "global.h"
+#include <sys/stat.h>
+#include <utime.h>
 #include <X11/Xresource.h>
 #include "file.h"
 #include "resource.h"
@@ -31,6 +33,13 @@ typedef enum {
 
 #define DEFAULT_DB_SIZE   4096
 
+static const char	*bad_regex_msg =
+"knews: regcomp() couldn't compile the default quote regexp:\n"
+"          \"" DEFAULT_QUOTE_REGEXP "\"\n"
+"       Chances are your system's regex implementaton sucks\n"
+"       You might want to recompile knews with HAVE_POSIX_REGEXPS\n"
+"       set to 0.  See the file configure.h for details.\n";
+
 static XrmDatabase	db = 0;
 
 static char	*pw_name = NULL;
@@ -47,6 +56,8 @@ struct per_server{
 	char	*newsrc_file;
 	char	*old_newsrc_file;
 	char	*kill_file;
+	char	*remote_newsrc_file;
+	char	*remote_kill_file;
 	char	*auth_info_user;
 	char	*auth_info_pass;
 	char	*rescan_timeout;
@@ -61,10 +72,12 @@ struct per_server{
 	char	*cache_dir;
 	char	*descriptions_file;
 	char	*save_thread_info;
+	char	*group_name_columns;
     } str;
     struct {
 	char	**thread_ahead_groups;
 	long	rescan_timeout;
+	int	group_name_columns;
 	char	retrieve_descriptions;
 	char	check_for_new_groups;
 	char	read_active_file;
@@ -82,12 +95,15 @@ struct per_group {
 	char	*header_format;
 	char	*quote_regexp;
 	char	*quote_string;
+	char	*quote_quote_string;
 	char	*attribution;
 	char	*full_name;
 	char	*signature_file;
 	char	*organization;
 	char	*reply_to;
 	char	*extra_headers;
+	char	*followup_headers;
+	char	*posted_and_mailed;
 	char	*uu_dir;
 	char	*uu_program;
 	char	*distribution;
@@ -101,12 +117,14 @@ struct per_group {
 	char	*assemble_partials;
 	char	*cache_ahead_size;
 	char	*cache_trail_size;
+	char	*subject_columns;
     } str;
     struct {
 	char	**header_format;
 	regex_t	*quote_regexp;
 	int	cache_ahead_size;
 	int	cache_trail_size;
+	int	subject_columns;
 	char	full_header;
 	char	process_xrefs;
 	char	show_number_lines;
@@ -121,15 +139,15 @@ static struct per_group per_group, per_group_default = {{0, }, };
 typedef union {
     char	*str;
     XrmQuark	q;
-} res_name_t;
+} RES_NAME;
 
 typedef struct {
-    res_name_t	name;
-    res_name_t	class;
+    RES_NAME	name;
+    RES_NAME	class;
     const int	offset;
-} res_desc_t;
+} RES_DESC;
 
-static res_desc_t per_server_desc[] = {
+static RES_DESC per_server_desc[] = {
 #define offset(field) XtOffsetOf(struct per_server, str.field)
     {{"newsrcFile"},		{"NewsrcFile"},
      offset(newsrc_file)},
@@ -137,6 +155,10 @@ static res_desc_t per_server_desc[] = {
      offset(old_newsrc_file)},
     {{"killFile"},		{"KillFile"},
      offset(kill_file)},
+    {{"remoteNewsrcFile"},	{"RemoteNewsrcFile"},
+     offset(remote_newsrc_file)},
+    {{"remoteKillFile"},	{"RemoteKillFile"},
+     offset(remote_kill_file)},
     {{"authInfoUser"},		{"AuthInfoUser"},
      offset(auth_info_user)},
     {{"authInfoPass"},		{"AuthInfoPass"},
@@ -165,10 +187,12 @@ static res_desc_t per_server_desc[] = {
      offset(descriptions_file)},
     {{"saveThreadInfo"},	{"SaveThreadInfo"},
      offset(save_thread_info)},
+    {{"groupNameColumns"},	{"GroupNameColumns"},
+     offset(group_name_columns)},
 #undef offset
 };
 
-static res_desc_t per_group_desc[] = {
+static RES_DESC per_group_desc[] = {
 #define offset(field)	XtOffsetOf(struct per_group, str.field)
     {{"fullName"},		{"FullName"},
      offset(full_name)},
@@ -184,12 +208,18 @@ static res_desc_t per_group_desc[] = {
      offset(quote_regexp)},
     {{"quoteString"},		{"QuoteString"},
      offset(quote_string)},
+    {{"quoteQuoteString"},	{"QuoteString"},
+     offset(quote_quote_string)},
     {{"attribution"},		{"Attribution"},
      offset(attribution)},
     {{"signatureFile"},		{"SignatureFile"},
      offset(signature_file)},
-    {{"extraHeaders"},		{"ExraHeaders"},
+    {{"extraHeaders"},		{"ExtraHeaders"},
      offset(extra_headers)},
+    {{"followupHeaders"},	{"FollowupHeaders"},
+     offset(followup_headers)},
+    {{"postedAndMailed"},	{"PostedAndMailed"},
+     offset(posted_and_mailed)},
     {{"uuDir"},			{"UuDir"},
      offset(uu_dir)},
     {{"uuProgram"},		{"UuProgram"},
@@ -214,6 +244,8 @@ static res_desc_t per_group_desc[] = {
      offset(cache_ahead_size)},
     {{"cacheTrailSize"},	{"CacheSize"},
      offset(cache_trail_size)},
+    {{"subjectColumns"},	{"SubjectColumns"},
+     offset(subject_columns)},
 #undef offset
 };
 
@@ -314,7 +346,7 @@ static int str_to_bool(char *str, int def)
 	if (len == values[i].len && case_lstrcmp(str, values[i].str) == 0)
 	    return values[i].val;
 
-    fprintf(stderr, "Warning: failed to convert \"%s\" to a boolean.\n", str);
+    fprintf(stderr, "knews: failed to convert \"%s\" to a boolean.\n", str);
 
     return def;
 }
@@ -339,7 +371,7 @@ static long str_to_long(char *str, long def)
     if (sscanf(str, "%ld", &res) == 1)
 	return res;
 
-    fprintf(stderr, "Warning: failed to convert \"%s\" to an integer.\n", str);
+    fprintf(stderr, "knews: failed to convert \"%s\" to an integer.\n", str);
 
     return def;
 }
@@ -399,7 +431,7 @@ static void get_default_database(char *str, long len)
     int	is_config_server;
 
     is_config_server =
-	global.config_nntp_server &&
+	global.config_nntp_server && global.nntp_server &&
 	strcmp(global.config_nntp_server, global.nntp_server) == 0;
 
     strcpy(str,
@@ -449,6 +481,7 @@ static void get_default_database(char *str, long len)
     str += print_res(str, "descriptionsFile", NULL,
 		     per_server_default.str.descriptions_file);
     str += print_res(str, "saveThreadInfo", NULL, "False");
+    str += print_res(str, "groupNameColumns", NULL, "42");
 
     strcpy(str,
 	   "\n"
@@ -464,7 +497,8 @@ static void get_default_database(char *str, long len)
 		     "Subject:Newsgroups:Followup-To:Reply-To:\\\n"
 		     "!		Content-Description:Date:Organization:From:");
     str += print_res(str, "*quoteRegexp", NULL, "^[ 	]*[:>|]");
-    str += print_res(str, "*quoteString", NULL, ">");
+    str += print_res(str, "*quoteString", NULL, "> ");
+    str += print_res(str, "*quotequoteString", NULL, ">");
     str += print_bool(str, "*showNumberLines", global.show_number_lines,
 		      per_group_default.conv.show_number_lines);
     str += print_res(str, "*attribution", NULL, 
@@ -476,6 +510,9 @@ static void get_default_database(char *str, long len)
     str += print_res(str, "*replyTo", NULL, "$REPLYTO");
     str += print_res(str, "*distribution", NULL, "$DEFNEWSDIS");
     str += print_res(str, "*extraHeaders", NULL, NULL);
+    str += print_res(str, "*followupHeaders", NULL, NULL);
+    str += print_res(str, "*postedAndMailed", NULL,
+		     per_group_default.str.posted_and_mailed);
     str += print_res(str, "*uuDir", NULL, "~/News");
     str += print_res(str, "*uuProgram", NULL, NULL);
     str += print_bool(str, "*expireKills", NULL,
@@ -485,6 +522,7 @@ static void get_default_database(char *str, long len)
     str += print_res(str, "*defaultCharset", NULL, "us-ascii");
     str += print_res(str, "*cacheAheadSize", NULL, "0");
     str += print_res(str, "*cacheTrailSize", NULL, "0");
+    str += print_res(str, "*groupNameColumns", NULL, "56");
 
     strcpy(str,
 	   "\n"
@@ -502,37 +540,41 @@ static XrmDatabase load_database(char *path)
 {
     FILE	*fp = NULL;
     XrmDatabase	db = 0;
+    struct stat	stat_buf;
+    int		fd;
 
-    if (path) {
-	int	fd = open(path, O_RDONLY);
-
-	if (fd < 0) {
+    if (path)
+	if (stat(path, &stat_buf) < 0 ||
+	    (fd = open(path, O_RDONLY)) < 0)
 	    if (errno != ENOENT)
 		perror(path);
 	    else {
 		fprintf(stderr, "Knews: creating config file %s\n", path);
-		fd = open_mkdir(path, O_WRONLY|O_TRUNC|O_EXCL|O_CREAT,
-				0644, True);
+		fd = open_mkdir(path, O_WRONLY|O_TRUNC|O_EXCL|O_CREAT, True);
 		if (fd >= 0 && !(fp = fdopen(fd, "w"))) {
 		    perror(path);
 		    close(fd);
 		}
-		popup_filenotice(fp ? "Created config file" :
-				 "Failed to create config file",
-				 path, fp == NULL);
+		popup_title_notice(fp ? "Created config file" :
+				   "Failed to create config file",
+				   path, fp == NULL);
 	    }
-	} else {
+	else {
 	    char	*buffer;
 
+	    global.last_time = stat_buf.st_atime;
 	    buffer = snarf_file(fd, NULL);
 	    if (buffer) {
 		db = XrmGetStringDatabase(buffer);
 		free(buffer);
 	    }
 
+	    if (global.bogus_file_system)
+		if (utime(path, NULL) < 0)
+		    perror("utime");
+
 	    close(fd);
 	}
-    }
 
     if (!db) {
 	char	default_db[4096];
@@ -540,16 +582,15 @@ static XrmDatabase load_database(char *path)
 	get_default_database(default_db, sizeof default_db);
 	if (fp) {
 	    fputs(default_db, fp);
-	    if (fclose(fp) == EOF)
+	    if (fclose(fp) != 0)
 		perror(path);
 	}
 	db = XrmGetStringDatabase(default_db);
 	if (!db)
 	    db = XrmGetStringDatabase("");
 	if (!db)
-	    fprintf(stderr,
-		    "Knews: Failed to create empty config database! "
-		    "This ain't happning!\n");
+	    fputs("Knews: Failed to create empty config database! "
+		  "This ain't happning!\n", stderr);
     }
 
     return db;
@@ -575,8 +616,8 @@ static XrmQuark *quarkify(char *str, int cap)
 	c = strchr(str, '.');
 	if (c)
 	    *c = '\0';
-	if (cap && isupper((unsigned char)ch))
-	    *str = tolower((unsigned char)ch);
+	if (cap && IS_UPPER(ch))
+	    *str = TO_LOWER(ch);
 
 	res[n++] = XrmStringToQuark(str);
 
@@ -590,97 +631,11 @@ static XrmQuark *quarkify(char *str, int cap)
     return res;
 }
 
-/*************************************************************************/
-
-#if XtSpecificationRelease > 4
-
-typedef struct {
-    int		first;
-    XrmQuark	*res;
-} ENUM_PROC_DATA;
-
-static Bool db_check_proc(XrmDatabase	*db,
-			  XrmBinding	*bindings,
-			  XrmQuark	*quarks,
-			  XrmQuark	*type,
-			  XrmValue	*value,
-			  XPointer	client_data)
-{
-    ENUM_PROC_DATA	*data = (ENUM_PROC_DATA *)client_data;
-    XrmQuark		*res = data->res;
-    XrmQuark		last;
-    int			i = 0;
-
-    if (quarks[0] == 0)
-	return 0;
-
-    while (quarks[i + 1] != 0)
-	i++;
-    last = quarks[i];
-
-    while (*res != 0)
-	if (*res++ == last) {
-	    if (data->first) {
-		data->first = False;
-		fprintf(stderr,
-			"Warning: Ignoring obsolete resource settings:\n");
-	    }
-	    if (*bindings == XrmBindTightly && *quarks != 0)
-		fprintf(stderr, "%s", XrmQuarkToString(*quarks++));
-	    while (*quarks != 0)
-		fprintf(stderr, "%c%s",
-			*bindings++ == XrmBindTightly ? '.' : '*',
-			XrmQuarkToString(*quarks++));
-	    fprintf(stderr, ": %s\n", (char *)value->addr);
-	    break;
-	}
-
-    return 0;
-}
-
-static void check_compatibility(void)
-{
-    XrmQuark		name[2], class[2];
-    XrmQuark		res[16];
-    ENUM_PROC_DATA	data;
-    int			n = 0;
-
-    name[0] = XrmPermStringToQuark("knews");
-    name[1] = 0;
-    class[0] = XrmPermStringToQuark("Knews");
-    class[1] = 0;
-    res[n++] = XrmPermStringToQuark("newsrcFile");
-    res[n++] = XrmPermStringToQuark("oldNewsrcFile");
-    res[n++] = XrmPermStringToQuark("killFile");
-    res[n++] = XrmPermStringToQuark("attribution");
-    res[n++] = XrmPermStringToQuark("signatureFile");
-    res[n++] = XrmPermStringToQuark("threadEmAll");
-    res[n++] = 0;
-
-    data.first = True;
-    data.res = res;
-
-    XrmEnumerateDatabase(XtScreenDatabase(XtScreen(main_widgets.shell)),
-			 name, class, XrmEnumAllLevels,
-			 db_check_proc, (XPointer)&data);
-}
-
-#else /* pre R5 */
-
-static void check_compatibility(void)
-{
-    return;
-}
-
-#endif
-
 void res_initialize(void)
 {
     char	*tmp;
     int		i;
 
-    check_compatibility();
-
     for (i = 0 ; i < XtNumber(per_server_desc) ; i++) {
 	per_server_desc[i].name.q =
 	    XrmPermStringToQuark(per_server_desc[i].name.str);
@@ -695,16 +650,9 @@ void res_initialize(void)
 	    XrmPermStringToQuark(per_group_desc[i].class.str);
     }
 
-    if (regcomp(&default_quote_re,
-		DEFAULT_QUOTE_REGEXP, REGEXP_COMPILE_FLAGS) != 0)
-	fprintf(stderr,
-		"Warning:  regcomp() couldn't compile "
-		"\"" DEFAULT_QUOTE_REGEXP "\"\n"
-		"          Chances are your system's regex "
-                "implementaton sucks.\n"
-		"          You might want to recompile knews "
-		"with NEED_POSIX_REGEXPS.\n"
-		"          See the file configure.h for details.\n");
+    if (regcomp(&default_quote_re, DEFAULT_QUOTE_REGEXP,
+		REGEXP_COMPILE_FLAGS & ~REG_NOSUB) != 0)
+	fputs(bad_regex_msg, stderr);
 
     per_server_default.str.newsrc_file =
 	global.newsrc_templ ? global.newsrc_templ : "~/.newsrc-%s";
@@ -730,11 +678,14 @@ void res_initialize(void)
     per_server_default.conv.fill_newsrc_file       = False;
     per_server_default.conv.try_list_active        = True;
     per_server_default.conv.save_thread_info	   = False;
+    per_server_default.conv.group_name_columns     = 42;
 
-    per_group_default.str.quote_string   = ">";
-    per_group_default.str.attribution    = DEFAULT_ATTRIBUTION;
-    per_group_default.str.signature_file = "~/.signature";
-    per_group_default.str.uu_dir         = "~/News";
+    per_group_default.str.quote_string        = "> ";
+    per_group_default.str.quote_quote_string  = ">";
+    per_group_default.str.attribution         = DEFAULT_ATTRIBUTION;
+    per_group_default.str.signature_file      = "~/.signature";
+    per_group_default.str.uu_dir              = "~/News";
+    per_group_default.str.posted_and_mailed   = "[Posted and mailed]";
     per_group_default.str.show_number_lines =
 	global.show_number_lines;
     per_group_default.str.keep_thread_info =
@@ -748,6 +699,7 @@ void res_initialize(void)
     per_group_default.conv.expire_kills      = True;
     per_group_default.conv.assemble_partials = True;
     per_group_default.conv.quote_regexp      = &default_quote_re;
+    per_group_default.conv.subject_columns   = 56;
 
     tmp = getenv("NAME");
     if (tmp)
@@ -795,6 +747,7 @@ int res_load(char *path)
     if (db)
 	XrmDestroyDatabase(db);
 
+    global.last_time = 0;
     db = load_database(path);
     if (!db)
 	return -1;
@@ -818,6 +771,9 @@ int res_load(char *path)
     per_server.conv.rescan_timeout =
 	str_to_long(per_server.str.rescan_timeout,
 		    per_server.conv.rescan_timeout);
+    per_server.conv.group_name_columns =
+	str_to_long(per_server.str.group_name_columns,
+		    per_server.conv.group_name_columns);
 
     per_server.conv.retrieve_descriptions =
 	str_to_bool(per_server.str.retrieve_descriptions,
@@ -841,6 +797,8 @@ int res_load(char *path)
 	str_to_bool(per_server.str.save_thread_info,
 		    per_server.conv.save_thread_info);
 
+    res_enter_group("none");
+
     return 0;
 }
 
@@ -890,7 +848,7 @@ void res_enter_group(char *group)
 
     if (per_group.str.quote_regexp)
 	if (regcomp(&quote_re, per_group.str.quote_regexp,
-		    REGEXP_COMPILE_FLAGS) == 0)
+		    REGEXP_COMPILE_FLAGS & ~REG_NOSUB) == 0)
 	    per_group.conv.quote_regexp = &quote_re;
 	else
 	    fprintf(stderr, "Warning: failed to compile regexp \"%s\".\n",
@@ -915,6 +873,9 @@ void res_enter_group(char *group)
     per_group.conv.cache_trail_size =
 	str_to_long(per_group.str.cache_trail_size,
 		    per_group.conv.cache_trail_size);
+    per_group.conv.subject_columns =
+	str_to_long(per_group.str.subject_columns,
+		    per_group.conv.subject_columns);
 }
 
 #define PER_SERVER_STR_FUNC(field) \
@@ -926,6 +887,8 @@ char *res_##field(void)            \
 PER_SERVER_STR_FUNC(newsrc_file)
 PER_SERVER_STR_FUNC(old_newsrc_file)
 PER_SERVER_STR_FUNC(kill_file)
+PER_SERVER_STR_FUNC(remote_newsrc_file)
+PER_SERVER_STR_FUNC(remote_kill_file)
 PER_SERVER_STR_FUNC(auth_info_user)
 PER_SERVER_STR_FUNC(auth_info_pass)
 PER_SERVER_STR_FUNC(posting_agent)
@@ -947,6 +910,7 @@ PER_SERVER_FUNC(int,    ask_how_many)
 PER_SERVER_FUNC(int,    fill_newsrc_file)
 PER_SERVER_FUNC(int,    try_list_active)
 PER_SERVER_FUNC(int,	save_thread_info)
+PER_SERVER_FUNC(int,	group_name_columns)
 
 #define PER_GROUP_STR_FUNC(field) \
 char *res_##field(void)           \
@@ -955,12 +919,15 @@ char *res_##field(void)           \
 }
 
 PER_GROUP_STR_FUNC(quote_string)
+PER_GROUP_STR_FUNC(quote_quote_string)
 PER_GROUP_STR_FUNC(attribution)
 PER_GROUP_STR_FUNC(full_name)
 PER_GROUP_STR_FUNC(signature_file)
 PER_GROUP_STR_FUNC(organization)
 PER_GROUP_STR_FUNC(reply_to)
 PER_GROUP_STR_FUNC(extra_headers)
+PER_GROUP_STR_FUNC(followup_headers)
+PER_GROUP_STR_FUNC(posted_and_mailed)
 PER_GROUP_STR_FUNC(uu_dir)
 PER_GROUP_STR_FUNC(uu_program)
 PER_GROUP_STR_FUNC(distribution)
@@ -982,6 +949,7 @@ PER_GROUP_FUNC(regex_t*, quote_regexp)
 PER_GROUP_FUNC(int,      assemble_partials)
 PER_GROUP_FUNC(int,      cache_ahead_size)
 PER_GROUP_FUNC(int,      cache_trail_size)
+PER_GROUP_FUNC(int,      subject_columns) 
 
 int res_keep_thread_info(int subscribed)
 {
@@ -1008,3 +976,8 @@ void res_set_ask_how_many(int ask)
 {
     per_server.conv.ask_how_many = ask;
 }
+
+void res_set_default_charset(char *charset)
+{
+    per_group.str.default_charset = charset;
+}
