@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #include "global.h"
 #include <X11/Shell.h>
@@ -31,6 +31,7 @@
 #include "../Widgets/Message.h"
 #include "../Widgets/Notice.h"
 #include "../Widgets/Sash.h"
+#include "../Widgets/Scrollable.h"
 #include "../Widgets/ScrBar.h"
 #include "../Widgets/ScrList.h"
 #include "../Widgets/TextField.h"
@@ -148,10 +149,7 @@ static void set_scope_toggle(SearchScope scope, int set)
 	return;
     }
 
-    if (set)
-	ToggleSet(w);
-    else
-	ToggleUnset(w);
+    ToggleSet(w, set);
 }
 
 static void do_subj_toggle(int set)
@@ -204,26 +202,24 @@ static void set_search_mode(SearchMode mode)
 		      mode == SearchModeXpat && n_xpats > 0);
 }
 
-static int valid_header(char *header)
+static int valid_header(char *c)
 {
-    for (;;) {
-	unsigned char	c = *(unsigned char *)header++;
-
-	if (c == '\0')
-	    return True;
-
-	if (!isalpha(c) && !isdigit(c) && c != '-')
+    while (*c != '\0')
+	if (!isalpha((unsigned char)*c) &&
+	    !isdigit((unsigned char)*c) &&
+	    *c != '-' && *c != '/')
 	    return False;
-    }
+	else
+	    c++;
 
-    /* not reached */
+    return True;
 }
 
 /*************************************************************************/
 
 static void do_group_search(regex_t *re)
 {
-    long	i, n;
+    long	i;
 
     i = ScrListGetFirstSelected(main_widgets.group_list);
     if (i < 0)
@@ -242,11 +238,7 @@ static void do_group_search(regex_t *re)
     else {
 	global.curr_group = global.groups[i];
 	ScrListSetSelected(main_widgets.group_list, i, True);
-	ScrListSetFirst(main_widgets.group_list, i);
-	ScrListGetFirstShownSize(main_widgets.group_list,
-				 NULL, &n, NULL);
-	ScrBarSetLengthsAndPos(main_widgets.top_vbar,
-			       global.no_groups, n, i);
+	ScrListMakeVisible(main_widgets.group_list, i);
     }
 }
 
@@ -301,7 +293,7 @@ static void do_from_search(regex_t *re)
 	set_curr_art(art, True);
 	if (global.mode == NewsModeThread)
 	    setNewsModeThread();
-	read_article(art, False, NULL);
+	read_article(art, False, NULL, NULL);
     }    
 }
 
@@ -340,7 +332,7 @@ static void do_art_search(regex_t *re)
     strcpy(message, "Searching...   ");
     mp = message + strlen(message);
 
-    set_busy(True, True);
+    set_busy(True);
     stopped = False;
     XtSetSensitive(search_widgets.stop_knapp, True);
 
@@ -414,7 +406,7 @@ static void do_art_search(regex_t *re)
     set_curr_art(art, True);
     if (global.mode == NewsModeThread)
 	setNewsModeThread();
-    read_article(art, True, sel_data);
+    read_article(art, True, sel_data, NULL);
 }
 
 static void do_xpat_search(char *header, char *wildcard)
@@ -446,7 +438,7 @@ static void do_xpat_search(char *header, char *wildcard)
 	    global.curr_group->last_art,
 	    wildcard);
 
-    set_busy(True, True);
+    set_busy(True);
     set_message("Server contacted, waiting for response, "
 		"this may take some time...", False);
     buffer = server_comm(main_server, command, True);
@@ -512,7 +504,7 @@ static void do_xpat_search(char *header, char *wildcard)
 	set_curr_art(art, True);
 	if (global.mode == NewsModeThread)
 	    setNewsModeThread();
-	read_article(art, False, NULL);
+	read_article(art, False, NULL, NULL);
 	set_xpat_message(art == NULL);
 	KnappSetSensitive(search_widgets.submit_knapp, False);
 	KnappSetSensitive(search_widgets.first_knapp, True);
@@ -530,28 +522,25 @@ static void submit_knapp_callback(Widget w,
 	return;
 
     if (search_mode == SearchModeXpat) {
-	char	*header = NULL;
-	char	*wildcard = NULL;
-	Arg	arg;
+	char	*header, *wildcard;
 
 	if (global.mode != NewsModeGroup &&
 	    global.mode != NewsModeThread)
 	    return;
 	
-	XtSetArg(arg, XtNbuffer, &header);
-	XtGetValues(search_widgets.header_field, &arg, 1);
-	XtSetArg(arg, XtNbuffer, &wildcard);
-	XtGetValues(search_widgets.wildcard_field, &arg, 1);
+	header   = TextFieldGetBuffer(search_widgets.header_field);
+	wildcard = TextFieldGetBuffer(search_widgets.wildcard_field);
 
-	if (header && wildcard) {
+	if (header && wildcard)
 	    if (valid_header(header))
 		do_xpat_search(header, wildcard);
 	    else
 		set_message("Invalid header name!", True);
-	} else {
-	    if (global.bell)
-		XBell(XtDisplay(w), 0);
-	}
+	else if (global.bell)
+		XBell(display, 0);
+
+	XtFree(header);
+	XtFree(wildcard);
     }
 }
 
@@ -559,21 +548,18 @@ static void search_knapp_callback(Widget w,
 				  XtPointer client_data,
 				  XtPointer call_data)
 {
-    Arg		arg;
-    char	*buffer = NULL;
+    char	*buffer;
     regex_t	re;
     int		code;
 
     if (global.busy || global.mode == SearchModeXpat)
 	return;
 
-    XtSetArg(arg, XtNbuffer, &buffer);
-    XtGetValues(search_widgets.regexp_field, &arg, 1);
-    if (!buffer)
-	buffer ="";
-
+    buffer = TextFieldGetBuffer(search_widgets.regexp_field);
     code = regcomp(&re, buffer,
 		   (global.icase_regexps ? REG_ICASE : 0) | REG_EXTENDED);
+    XtFree(buffer);
+
     if (code != 0) {
 	popup_regexpnotice(code, &re);
 	return;
@@ -633,7 +619,7 @@ static void first_knapp_callback(Widget w,
 	set_curr_art(art, True);
 	if (global.mode == NewsModeThread)
 	    setNewsModeThread();
-	read_article(art, False, NULL);
+	read_article(art, False, NULL, NULL);
 	set_xpat_message(art == NULL);
 	KnappSetSensitive(search_widgets.next_knapp, n_xpats > 1);
     }
@@ -650,7 +636,7 @@ static void next_knapp_callback(Widget w,
 	set_curr_art(art, True);
 	if (global.mode == NewsModeThread)
 	    setNewsModeThread();
-	read_article(art, False, NULL);
+	read_article(art, False, NULL, NULL);
 	set_xpat_message(art == NULL);
 	KnappSetSensitive(search_widgets.next_knapp, curr_xpat + 1 < n_xpats);
     }
@@ -667,9 +653,9 @@ static void from_toggle_callback(Widget w,
 
     *set = !*set;
     if (*set) {
-	ToggleUnset(search_widgets.subject_toggle);
-	ToggleUnset(search_widgets.head_toggle);
-	ToggleUnset(search_widgets.body_toggle);
+	ToggleSet(search_widgets.subject_toggle, False);
+	ToggleSet(search_widgets.head_toggle, False);
+	ToggleSet(search_widgets.body_toggle, False);
 	do_subj_toggle(False);
     }
 }
@@ -685,9 +671,9 @@ static void subject_toggle_callback(Widget w,
 
     *set = !*set;
     if (*set) {
-	ToggleUnset(search_widgets.from_toggle);
-	ToggleUnset(search_widgets.head_toggle);
-	ToggleUnset(search_widgets.body_toggle);
+	ToggleSet(search_widgets.from_toggle, False);
+	ToggleSet(search_widgets.head_toggle, False);
+	ToggleSet(search_widgets.body_toggle, False);
     }
 
     do_subj_toggle(*set);
@@ -704,8 +690,8 @@ static void head_toggle_callback(Widget w,
 
     *set = !*set;
     if (*set) {
-	ToggleUnset(search_widgets.from_toggle);
-	ToggleUnset(search_widgets.subject_toggle);
+	ToggleSet(search_widgets.from_toggle, False);
+	ToggleSet(search_widgets.subject_toggle, False);
 	do_subj_toggle(False);
     }
 }
@@ -721,8 +707,8 @@ static void body_toggle_callback(Widget w,
 
     *set = !*set;
     if (*set) {
-	ToggleUnset(search_widgets.from_toggle);
-	ToggleUnset(search_widgets.subject_toggle);
+	ToggleSet(search_widgets.from_toggle, False);
+	ToggleSet(search_widgets.subject_toggle, False);
 	do_subj_toggle(False);
     }
 }
@@ -784,13 +770,38 @@ static void unread_toggle_callback(Widget w,
     only_unread = *set;
 }
 
+static void tab_callback(Widget w, XtPointer client_data, XtPointer call_data)
+{
+    if (global.busy || (global.mode != NewsModeGroup &&
+			global.mode != NewsModeThread))
+	return;
+
+    if (ToggleGet(search_widgets.from_toggle)) {
+	ToggleSet(search_widgets.from_toggle, False);
+	ToggleSet(search_widgets.subject_toggle, True);
+	do_subj_toggle(True);
+    } else if (ToggleGet(search_widgets.subject_toggle)) {
+	ToggleSet(search_widgets.subject_toggle, False);
+	ToggleSet(search_widgets.head_toggle, True);
+	do_subj_toggle(False);
+    } else if (ToggleGet(search_widgets.head_toggle)) {
+	if (ToggleGet(search_widgets.body_toggle))
+	    ToggleSet(search_widgets.head_toggle, False);
+	else
+	    ToggleSet(search_widgets.body_toggle, True);
+    } else {
+	ToggleSet(search_widgets.body_toggle, False);
+	ToggleSet(search_widgets.from_toggle, True);
+    }
+}
+
 /*************************************************************************/
 			   
 static void create_search_widgets(void)
 {
     Arg		args[8];
     Widget	layout;
-    Widget	temp;
+    Widget	w;
 
     XtSetArg(args[0], XtNallowShellResize, True);
     XtSetArg(args[1], XtNinput, True);
@@ -805,8 +816,8 @@ static void create_search_widgets(void)
 	XtVaCreateManagedWidget("searchlayout", layoutWidgetClass,
 				search_widgets.shell,
 				XtVaTypedArg, XtNlayout, XtRString,
-#include "layouts/search.l"
-				(int)sizeof(String), NULL);
+#include "layouts/search.h"
+				(int)sizeof(String), (void *)0);
 
     XtCreateManagedWidget("regexptitle", messageWidgetClass, layout, NULL, 0);
     XtCreateManagedWidget("xpattitle", messageWidgetClass, layout, NULL, 0);
@@ -821,18 +832,21 @@ static void create_search_widgets(void)
 			  layout, args, 1);
 
     XtSetArg(args[0], XtNfocusRoot, search_widgets.shell);
+    XtSetArg(args[1], XtNsingleLine, True);
 
     search_widgets.regexp_field =
 	XtCreateManagedWidget("regexpfield", textFieldWidgetClass,
-			      layout, args, 1);
+			      layout, args, 2);
     XtAddCallback(search_widgets.regexp_field, XtNcallback,
 		  search_knapp_callback, NULL);
+    XtAddCallback(search_widgets.regexp_field, XtNtabCallback,
+		  tab_callback, NULL);
     XtAddCallback(search_widgets.regexp_field, XtNfocusCallback,
 		  focus_callback, NULL);
 
     search_widgets.wildcard_field =
 	XtCreateManagedWidget("wildcardfield", textFieldWidgetClass,
-			      layout, args, 1);
+			      layout, args, 2);
     XtAddCallback(search_widgets.wildcard_field, XtNcallback,
 		  xpat_field_callback, NULL);
     XtAddCallback(search_widgets.wildcard_field, XtNtabCallback,
@@ -842,7 +856,7 @@ static void create_search_widgets(void)
 
     search_widgets.header_field =
 	XtCreateManagedWidget("headerfield", textFieldWidgetClass,
-			      layout, args, 1);
+			      layout, args, 2);
     XtAddCallback(search_widgets.header_field, XtNcallback,
 		  xpat_field_callback, NULL);
     XtAddCallback(search_widgets.header_field, XtNtabCallback,
@@ -875,8 +889,8 @@ static void create_search_widgets(void)
     XtAddCallback(search_widgets.first_knapp, XtNcallback,
 		  first_knapp_callback, NULL);
 
-    temp = XtCreateManagedWidget("close", knappWidgetClass, layout, NULL, 0);
-    XtAddCallback(temp, XtNcallback, close_knapp_callback, NULL);
+    w = XtCreateManagedWidget("close", knappWidgetClass, layout, NULL, 0);
+    XtAddCallback(w, XtNcallback, close_knapp_callback, NULL);
 
     XtSetArg(args[0], XtNsensitive, False);
     search_widgets.stop_knapp =
@@ -884,66 +898,72 @@ static void create_search_widgets(void)
     XtAddCallback(search_widgets.stop_knapp, XtNcallback,
 		  stop_knapp_callback, NULL);
 
-    XtSetArg(args[0], XtNset, True);
-    search_widgets.from_toggle =
-	XtCreateManagedWidget("fromtoggle", toggleWidgetClass,
-			      layout, args, 1);
-    XtAddCallback(search_widgets.from_toggle, XtNcallback,
-		  from_toggle_callback, NULL);
-
-    XtSetArg(args[0], XtNset, True);
     search_widgets.unread_toggle =
 	XtCreateManagedWidget("unreadtoggle", toggleWidgetClass,
-			      layout, args, 1);
+			      layout, NULL, 0);
     XtAddCallback(search_widgets.unread_toggle, XtNcallback,
 		  unread_toggle_callback, NULL);
 
-    XtSetArg(args[0], XtNset, False);
+    /***/
+
+    search_widgets.from_toggle =
+	XtCreateManagedWidget("fromtoggle", toggleWidgetClass,
+			      layout, NULL, 0);
+    XtAddCallback(search_widgets.from_toggle, XtNcallback,
+		  from_toggle_callback, NULL);
+
     search_widgets.subject_toggle =
 	XtCreateManagedWidget("subjecttoggle", toggleWidgetClass,
-			      layout, args, 1);
+			      layout, NULL, 0);
     XtAddCallback(search_widgets.subject_toggle, XtNcallback,
 		  subject_toggle_callback, NULL);
 
-    XtSetArg(args[0], XtNset, False);
     search_widgets.head_toggle =
 	XtCreateManagedWidget("headtoggle", toggleWidgetClass,
-			      layout, args, 1);
+			      layout, NULL, 0);
     XtAddCallback(search_widgets.head_toggle, XtNcallback,
 		  head_toggle_callback, NULL);
 
-    XtSetArg(args[0], XtNset, False);
     search_widgets.body_toggle =
 	XtCreateManagedWidget("bodytoggle", toggleWidgetClass,
-			      layout, args, 1);
+			      layout, NULL, 0);
     XtAddCallback(search_widgets.body_toggle, XtNcallback,
 		  body_toggle_callback, NULL);
 
-    XtSetArg(args[0], XtNset, True);
+    if (ToggleGet(search_widgets.head_toggle) ||
+	ToggleGet(search_widgets.body_toggle)) {
+	ToggleSet(search_widgets.from_toggle, False);
+	ToggleSet(search_widgets.subject_toggle, False);
+    } else {
+	ToggleSet(search_widgets.head_toggle, False);
+	ToggleSet(search_widgets.body_toggle, False);
+	ToggleSet(search_widgets.subject_toggle,
+		  !ToggleGet(search_widgets.from_toggle));
+    }
+
+    /***/
+
     search_widgets.all_scope =
 	XtCreateManagedWidget("allscope", toggleWidgetClass,
-			      layout, args, 1);
+			      layout, NULL, 0);
     XtAddCallback(search_widgets.all_scope, XtNcallback,
 		  scope_callback, (XtPointer)SearchScopeAll);
 
-    XtSetArg(args[0], XtNset, False);
     search_widgets.thread_scope =
 	XtCreateManagedWidget("threadscope", toggleWidgetClass,
-			      layout, args, 1);
+			      layout, NULL, 0);
     XtAddCallback(search_widgets.thread_scope, XtNcallback,
 		  scope_callback, (XtPointer)SearchScopeThread);
 
-    XtSetArg(args[0], XtNset, False);
     search_widgets.subthread_scope =
 	XtCreateManagedWidget("subthreadscope", toggleWidgetClass,
-			      layout, args, 1);
+			      layout, NULL, 0);
     XtAddCallback(search_widgets.subthread_scope, XtNcallback,
 		  scope_callback, (XtPointer)SearchScopeSubthread);
 
-    XtSetArg(args[0], XtNset, False);
     search_widgets.tagged_scope =
 	XtCreateManagedWidget("taggedscope", toggleWidgetClass,
-			      layout, args, 1);
+			      layout, NULL, 0);
     XtAddCallback(search_widgets.tagged_scope, XtNcallback,
 		  scope_callback, (XtPointer)SearchScopeTagged);
 
@@ -996,8 +1016,7 @@ void set_busy_search(int busy)
     if (!search_widgets.shell)
 	return;
 
-    XDefineCursor(XtDisplay(search_widgets.shell),
-		  XtWindow(search_widgets.shell),
+    XDefineCursor(display, XtWindow(search_widgets.shell),
 		  busy ? global.busy_cursor : global.cursor);
     KnappSetActive(search_widgets.search_knapp, !busy);
     KnappSetActive(search_widgets.submit_knapp, !busy);
@@ -1035,7 +1054,8 @@ static void find_group_callback(Widget w,
 
     if (global.busy || !report ||
 	(global.mode != NewsModeConnected &&
-	 global.mode != NewsModeAllgroups))
+	 global.mode != NewsModeAllgroups &&
+	 global.mode != NewsModeSomegroups))
 	return;
 
     switch (report->reply) {
@@ -1045,12 +1065,12 @@ static void find_group_callback(Widget w,
 	    XtPopdown(w);
 	else if (!report->buffer || strlen(report->buffer) > 500) {
 	    if (global.bell)
-		XBell(XtDisplay(w), 0);
+		XBell(display, 0);
 	} else {
 	    char	command[512];
 	    char	*reply;
 
-	    set_busy(True, True);
+	    set_busy(True);
 	    set_message("Asking server about group...", False);
 	    sprintf(command, "GROUP %s\r\n", report->buffer);
 	    reply = server_comm(main_server, command, True);
@@ -1071,7 +1091,7 @@ static void find_group_callback(Widget w,
 		}
 
 		if (global.mode == NewsModeAllgroups)
-		    setNewsModeAllgroups(); /* if we're aborted later */
+		    setNewsModeAllgroups(NULL); /* if we're aborted later */
 		read_group(reply, True, 0);
 	    } else {
 		if (strlen(reply) > 80)
@@ -1081,14 +1101,26 @@ static void find_group_callback(Widget w,
 	    }
 	}
 	break;
-    case DialogueReplyMiddle: /* clear */
-	XtSetArg(arg, XtNbuffer, "");
-	XtSetValues(w, &arg, 1);
+    case DialogueReplyMiddle: /* regexp */
+	{
+	    regex_t	re;
+	    int		code;
+
+	    code = regcomp(&re, report->buffer, REGEXP_COMPILE_FLAGS);
+	    if (code != 0) {
+		popup_regexpnotice(code, &re);
+		return;
+	    }
+
+	    XtPopdown(w);
+	    setNewsModeAllgroups(&re);
+	    regfree(&re);
+	}
 	break;
     case DialogueReplyTab: /* complete */
 	if (report->buffer && report->buffer[0] != '\0') {
 	    long	len = strlen(report->buffer);
-	    long	first, last, n;
+	    long	first, last, n, sub, first_sub;
 	    char	*c;
 
 	    for (first = 0 ; first < global.no_groups ; first++)
@@ -1110,31 +1142,35 @@ static void find_group_callback(Widget w,
 	    if (first == global.no_groups) {
 		c = NULL;
 		n = 0;
+		first = -1;
 	    } else {
 		c = global.groups[first]->name;
 		n = getnmatching(c, global.groups[last-1]->name);
 	    }
 
-	    for (first = 0 ; first < global.no_groups ; first++)
-		if (!global.groups[first]->subscribed)
+	    first_sub = -1;
+	    for (sub = 0 ; sub < global.no_groups ; sub++)
+		if (!global.groups[sub]->subscribed)
 		    break;
-		else if (strncmp(global.groups[first]->name,
+		else if (strncmp(global.groups[sub]->name,
 				 report->buffer, len) == 0) {
 		    if (c) {
 			long	temp;
 
-			temp = getnmatching(c, global.groups[first]->name);
+			temp = getnmatching(c, global.groups[sub]->name);
 			if (temp < n)
 			    n = temp;
 		    } else {
-			c = global.groups[first]->name;
+			c = global.groups[sub]->name;
 			n = strlen(c);
 		    }
+		    if (first_sub < 0)
+			first_sub = sub;
 		}
 
 	    if (!c || n > 500) {
 		if (global.bell)
-		    XBell(XtDisplay(w), 0);
+		    XBell(display, 0);
 	    } else {
 		char	buffer[512];
 
@@ -1142,6 +1178,11 @@ static void find_group_callback(Widget w,
 		buffer[n] = '\0';
 		XtSetArg(arg, XtNbuffer, buffer);
 		XtSetValues(w, &arg, 1);
+
+		if (first < 0)
+		    first = first_sub;
+		if (global.mode == NewsModeAllgroups)
+		    ScrollableSetVPos(main_widgets.group_list, first);
 	    }
 	}
 	break;
@@ -1156,15 +1197,16 @@ void popup_find_group(void)
 {
     if (global.busy ||
 	(global.mode != NewsModeConnected &&
-	 global.mode != NewsModeAllgroups))
+	 global.mode != NewsModeAllgroups &&
+	 global.mode != NewsModeSomegroups))
 	return;
 
     if (find_group_widget)
 	popup_under_pointer(find_group_widget, XtGrabNone);
     else
 	find_group_widget =
-	    popup_dialogue("findgroup", "Find group by name",
-			   "Goto group", "Clear", "Cancel",
+	    popup_dialogue("findgroup", "Find group(s) by name/regexp",
+			   "Goto group", "Regexp", "Cancel",
 			   find_group_callback, NULL, XtGrabNone);
 }
 
