@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,10 +9,11 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 
-/* The general threading algorithm, and in particular the functions
+/*
+ * The general threading algorithm, and in particular the functions
  * link_child(), unlink_child() and thread_article(), are modified
  * from the source to trn.
  */
@@ -24,7 +25,7 @@
 #include "codes.h"
 #include "connect.h"
 #include "font.h"
-#include "kill.h"
+#include "k_file.h"
 #include "parse.h"
 #include "read.h"
 #include "resource.h"
@@ -40,9 +41,12 @@
 
 #define HASH_SIZE  237
 
-#ifndef HEAD_DEBUG
-#  define HEAD_DEBUG 0
-#endif
+#undef  HEAD_DEBUG
+#define HEAD_DEBUG   0
+#undef  ART_BYTES
+#define ART_BYTES    0
+#undef  DATE_ERRORS
+#define DATE_ERRORS  0
 
 struct THREAD_CONTEXT {
     ARTICLE		*articles;
@@ -55,18 +59,19 @@ struct THREAD_CONTEXT {
     char		*refs;
     long		total;
     long		n_done;
+    char		*charset;
 };
 
-THREAD_CONTEXT	*main_thr;
+THREAD_CONTEXT	*main_thr = NULL;
 
 /*************************************************************************/
 
-static long str_hash(unsigned char *str, long len)
+static long str_hash(const char *str, long len)
 {
-    long	hash = 0;
+    unsigned long	hash = 0;
 
     while (len-- > 0)
-	hash += *str++;
+	hash += *str++ & 0xdfu;
 
     return hash % HASH_SIZE;
 }
@@ -88,15 +93,15 @@ static void art_hash_table_clear(ARTICLE **art_ht) {
     }
 }
 
-static ARTICLE **art_hash_find(ARTICLE **art_ht, char *msgid, long len)
+static ARTICLE **art_hash_find(ARTICLE **art_ht, const char *msgid, long len)
 {
     ARTICLE	*loop, *prev = NULL;
     ARTICLE	**hash_ptr;
 
-    hash_ptr = &art_ht[str_hash((unsigned char *)msgid, len)];
+    hash_ptr = &art_ht[str_hash(msgid, len)];
     for (loop = *hash_ptr ; loop ; prev = loop, loop = loop->hash_next)
-	if (loop->hash_len == len &&
-	    strncmp(msgid, loop->msgid, len) == 0) break;
+	if (len == loop->hash_len && memcmp(msgid, loop->msgid, len) == 0)
+	    break;
 
     return prev ? &prev->hash_next : hash_ptr;
 }
@@ -115,15 +120,18 @@ static void subj_hash_table_clear(SUBJECT **subj_ht) {
     }
 }
 
-static SUBJECT **subj_hash_find(SUBJECT **subj_ht, char *subject, long len)
+static SUBJECT **subj_hash_find(SUBJECT **subj_ht,
+				const char *subject,
+				long len)
 {
     SUBJECT	*loop, *prev = NULL;
     SUBJECT	**hash_ptr;
 
-    hash_ptr = &subj_ht[str_hash((unsigned char *)subject, len)];
+    hash_ptr = &subj_ht[str_hash(subject, len)];
     for (loop = *hash_ptr ; loop ; prev = loop, loop = loop->hash_next)
 	if (loop->hash_len == len &&
-	    strncmp(subject, loop->subject, len) == 0) break;
+	    strncmp(subject, loop->subject, len) == 0)
+	    break;
 
     return prev ? &prev->hash_next : hash_ptr;
 }
@@ -140,6 +148,7 @@ void clear_thread_context(THREAD_CONTEXT *context)
     context->refs = NULL;
     context->total = 1;
     context->n_done = 0;
+    context->charset = NULL;
 }
 
 static void nullify_thread_context(THREAD_CONTEXT *context)
@@ -154,6 +163,7 @@ static void nullify_thread_context(THREAD_CONTEXT *context)
     context->refs = NULL;
     context->total = 1;
     context->n_done = 0;
+    context->charset = NULL;
 
     for (i = 0 ; i < HASH_SIZE ; i++) {
 	context->art_ht[i] = NULL;
@@ -186,7 +196,7 @@ void set_subjects(THREAD_CONTEXT *context, SUBJECT *subjects)
     context->subjects = subjects;
 }
 
-ARTICLE *find_article(char *msgid, long msgid_len)
+ARTICLE *find_article(const char *msgid, long msgid_len)
 {
     return *art_hash_find(main_thr->art_ht, msgid, msgid_len);
 }
@@ -203,9 +213,11 @@ char *thread_from_file(SERVER *server, long first_art)
 {
     char	*buffer;
 
-    for (buffer = server_read(server) ;
-	 buffer && !IS_DOT(buffer) ;
-	 buffer = server_read(server)) {
+    main_thr->charset = res_default_charset();
+    if (!main_thr->charset)
+	main_thr->charset = "iso-8859-1";
+
+    while ((buffer = server_read(server)) && !IS_DOT(buffer)) {
 	ARTICLE	*art;
 
 	if (atol(buffer) < first_art)
@@ -228,9 +240,44 @@ char *thread_from_file(SERVER *server, long first_art)
 
 /*************************************************************************/
 
+static char *get_msgid(char *beg, char *end)
+{
+    char	*mid;
+
+    while (*beg == ' ')
+	beg++;
+    if (beg >= end || *beg++ != '<')
+	return NULL;
+    while (end > beg && *end == ' ')
+	*end-- = '\0';
+    if (*end != '>')
+	return NULL;
+    *end = '\0';
+
+    mid = strchr(beg, '@');
+    if (mid)
+	while (++mid < end)
+	    if (IS_UPPER(*mid))
+		*mid = LOWER(*mid);
+
+    return beg;
+}
+
+/* replaces ascii 0-32 & 127 and iso-8859-x 128-159 & 255 with ' ' */
+static void munge_nonprintable(char *str)
+{
+    unsigned char	*c = (unsigned char *)str;
+
+    while (*c != '\0') {
+	if ((*c & 0x60) == 0 || *c == '\b')
+	    *c = ' ';
+	c++;
+    }
+}
+
 static void overview_error(long art_no, char *string)
 {
-    fprintf(stderr, "Bad overview record");
+    fputs("Bad overview record", stderr);
     if (art_no)
 	fprintf(stderr, ", article no %ld", art_no);
     fprintf(stderr, "; %s\n", string);
@@ -349,11 +396,15 @@ static void merge_threads(THREAD_CONTEXT *context, SUBJECT *s1, SUBJECT *s2)
     }
 }
 
-static SUBJECT *get_subject(THREAD_CONTEXT *context, char *str)
+static SUBJECT *get_subject(THREAD_CONTEXT *context, char *subj)
 {
-    SUBJECT	    **sp;
+    SUBJECT	**sp;
 
-    if (!*(sp = subj_hash_find(context->subj_ht, str, strlen(str)))) {
+    decode_rfc1522(subj, context->charset);
+    munge_nonprintable(subj);
+    subj = eat_re(subj);
+
+    if (!*(sp = subj_hash_find(context->subj_ht, subj, strlen(subj)))) {
 	(*sp) = (SUBJECT *)XtMalloc(sizeof(SUBJECT));
 	(*sp)->hash_next = NULL;
 	(*sp)->next = NULL;
@@ -361,10 +412,10 @@ static SUBJECT *get_subject(THREAD_CONTEXT *context, char *str)
 	(*sp)->thread = NULL;
 	(*sp)->no_unread = 0;
 	(*sp)->disp = 0;
-	(*sp)->hot = 0;
+	(*sp)->pixmap = None;
 	(*sp)->has_tagged = False;
-	(*sp)->subject = XtNewString(str);
-	(*sp)->hash_len = strlen(str);
+	(*sp)->subject = XtNewString(subj);
+	(*sp)->hash_len = strlen(subj);
 	if (context->subjects) {
 	    if (!context->last_subj || context->last_subj->next) {
 		/* subjects have been reordered by merge_threads */
@@ -385,18 +436,6 @@ static SUBJECT *get_subject(THREAD_CONTEXT *context, char *str)
     return *sp;
 }
 
-/* replaces ascii 0-32 & 127 and iso-8859-x 128-159 & 255 with ' ' */
-static void munge_nonprintable(char *str)
-{
-    unsigned char	*c = (unsigned char *)str;
-
-    while (*c != '\0') {
-	if ((*c & 0x60) == 0 || *c == '\b')
-	    *c = ' ';
-	c++;
-    }
-}
-
 static ARTICLE *parse_xover_line(THREAD_CONTEXT *context, char *buffer)
 {
     ARTICLE	*art;
@@ -406,11 +445,11 @@ static ARTICLE *parse_xover_line(THREAD_CONTEXT *context, char *buffer)
     char	*subject, *from, *msgid;
     time_t	date;
     long	msgid_len;
-#if 0
+#if ART_BYTES
     long	bytes;
 #endif
 
-    if (!(c1 = strchr(buffer, '\t')) || buffer[0] < '0' || buffer[0] > '9') {
+    if (!(c1 = strchr(buffer, '\t')) || !IS_DIGIT(buffer[0])) {
 	overview_error(0, "no article number.");
 	return NULL;
     }
@@ -422,14 +461,14 @@ static ARTICLE *parse_xover_line(THREAD_CONTEXT *context, char *buffer)
 	return NULL;
     }
     *c2++ = '\0';
-    munge_nonprintable(c1);
-    subject = eat_re(c1);
+    subject = c1;
 
     if (!(c1 = strchr(c2, '\t'))) {
 	overview_error(art_no, "no From: field.");
 	return NULL;
     }
     *c1++ = '\0';
+    decode_rfc1522(c2, context->charset);
     munge_nonprintable(c2);
     from = c2;
 
@@ -439,6 +478,10 @@ static ARTICLE *parse_xover_line(THREAD_CONTEXT *context, char *buffer)
     }
     *c2++ = '\0';
     date = parsedate(c1);
+#if DATE_ERRORS
+    if (date == PARSEDATE_ERROR)
+	fprintf(stderr, "Bad date: %s\n", c1);
+#endif
 
     if (!(c1 = strchr(c2, '\t'))) {
 	overview_error(art_no, "no Message-Id: field.");
@@ -465,21 +508,13 @@ static ARTICLE *parse_xover_line(THREAD_CONTEXT *context, char *buffer)
 	return NULL;
     }
     *c1++ = '\0';
-#if 0
-    if (*c2 < '0' || *c2 > '9')
-	bytes = 0;
-    else
-	bytes = atol(c2);
-    /* Bytes, not optional */
+#if ART_BYTES
+    bytes = atol(c2);
 #endif
 
     if ( (c2 = strchr(c1, '\t')) )
 	*(c2++) = '\0';
-    if (*c1 < '0' || *c1 > '9')
-	lines = 0;
-    else
-	lines = atol(c1);
-    /* Lines, not optional */
+    lines = atol(c1);
 
     if ( (art = *(fake_p = art_hash_find(context->art_ht,
 					 msgid, msgid_len))) ) {
@@ -501,7 +536,7 @@ static ARTICLE *parse_xover_line(THREAD_CONTEXT *context, char *buffer)
 	art->hash_next = NULL;
 	art->next = NULL;
 	art->xref = NULL;
-	art->hot = 0;
+	art->pixmap = None;
 	art->read = FALSE;
 	art->killed = FALSE;
 	art->msgid = memcpy(XtMalloc(msgid_len + 1), msgid, msgid_len + 1);
@@ -514,7 +549,7 @@ static ARTICLE *parse_xover_line(THREAD_CONTEXT *context, char *buffer)
     art->from = XtNewString(from);
     art->date = date;
     art->lines = lines;
-#if 0
+#if ART_BYTES
     art->bytes = bytes;
 #endif
 
@@ -596,9 +631,9 @@ static void thread_article(THREAD_CONTEXT *context, ARTICLE *art)
 		a->xref = NULL;
 		a->subject = NULL;
 		a->no = 0;
-		a->hot = 0;
+		a->pixmap = None;
 		a->lines = 0;
-#if 0
+#if ART_BYTES
 		a->bytes = 0;
 #endif
 		a->read = FALSE;
@@ -685,25 +720,27 @@ static void mark_read_articles(THREAD_CONTEXT *context, GROUP *group)
     group->no_unread = n;
 }
 
-void fix_author(ARTICLE *art, char *from, int show_number_lines)
+void fix_author(ARTICLE *art, char *from, int show_no_lines)
 {
-    char	author[128];
+    const char	*author;
     char	buffer[160];
     char	*c;
-    int		n = strlen(from);
+    long	len = 0;
 
-    if (n > sizeof(author) - 1)
-	n = sizeof(author) - 1;
-    memcpy(author, from, n);
-    author[n] = '\0';
-    c = parse_author(author);
-
-    if (show_number_lines) {
-	sprintf(buffer, "%4ld  %s", art->lines, c);
+    if (!show_no_lines)
 	c = buffer;
+    else {
+	sprintf(buffer, "%4hu  ", art->lines);
+	c = buffer + strlen(buffer);
     }
 
-    art->tree_data.label = XtNewString(c);
+    author = parse_author(from, &len);
+    if (len > sizeof buffer - 16)
+	len = sizeof buffer - 16;
+    memcpy(c, author, len);
+    c[len] = '\0';
+
+    art->tree_data.label = XtNewString(buffer);
 }
 
 static void fix_authors(THREAD_CONTEXT *context)
@@ -712,7 +749,8 @@ static void fix_authors(THREAD_CONTEXT *context)
     int		show_number_lines = res_show_number_lines();
 
     for (art = context->articles ; art ; art = art->next)
-	fix_author(art, art->from, show_number_lines);
+	if (art->from)
+	    fix_author(art, art->from, show_number_lines);
 }
 
 static char *xover_articles(void)
@@ -725,9 +763,7 @@ static char *xover_articles(void)
     j = m = main_thr->total / 32;
     i = main_thr->n_done * 100;
 
-    for (buffer = server_read(main_server) ;
-	 buffer && !IS_DOT(buffer) ;
-	 i += 100, buffer = server_read(main_server)) {
+    while ((buffer = server_read(main_server)) && !IS_DOT(buffer)) {
 	art = parse_xover_line(main_thr, buffer);
 	if (art) {
 	    if (main_thr->last_art)
@@ -745,6 +781,7 @@ static char *xover_articles(void)
 	    set_message(command, False);
 	    j = m;
 	}
+	i += 100;
     }
 
     if (!buffer)
@@ -811,14 +848,16 @@ static ARTICLE *gather_headers(long art_no, THREAD_CONTEXT *context,
     }
 
     art->subject = get_subject(context, subject);
+    decode_rfc1522(from, context->charset);
+    munge_nonprintable(from);
     art->from = XtNewString(from);
     art->xref = xref ? XtNewString(xref) : NULL;
     art->date = date;
     art->lines = lines;
-#if 0
+#if ART_BYTES
     art->bytes = 0;
 #endif
-    art->hot = 0;
+    art->pixmap = None;
     art->read = False;
     art->killed = False;
     art->no = art_no;
@@ -863,7 +902,7 @@ ARTICLE *parse_head(long art_no, THREAD_CONTEXT *context, char *buffer)
 	    tmp = toupper(tmp);
 
 	switch (*buffer) {
-#if 0
+#if ART_BYTES
 	case 'B':
 	    if (case_lstrncmp(buffer, "bytes:", 6) == 0) {
 		buffer += 6;
@@ -924,7 +963,6 @@ ARTICLE *parse_head(long art_no, THREAD_CONTEXT *context, char *buffer)
 		buffer += 8;
 		while (IS_SPACE(*buffer))
 		    buffer++;
-		buffer = eat_re(buffer);
 		subject = buffer;
 		skip = False;
 	    }
@@ -951,7 +989,6 @@ ARTICLE *parse_head(long art_no, THREAD_CONTEXT *context, char *buffer)
     if (!from || !subject || !msgid)
 	return NULL;
 
-    munge_nonprintable(subject);
     munge_nonprintable(from);
     context->refs = refs;
 
@@ -1103,6 +1140,10 @@ static char *thread_articles(long asked_first)
     long	first = global.curr_group->first_art;
     long	last  = global.curr_group->last_art;
 
+    main_thr->charset = res_default_charset();
+    if (!main_thr->charset)
+	main_thr->charset = "iso-8859-1";
+
     if (asked_first < 0 || asked_first > global.curr_group->last_art)
 	asked_first = global.curr_group->last_art;
 
@@ -1161,7 +1202,8 @@ static void popup_ask_how_many(void);
 
 void read_group(char *reply, int may_ask, long first)
 {
-    int	threaded;
+    char	message[256];
+    int		threaded;
 
     if (!global.curr_group ||
 	(threaded = thread_ahead_check(global.curr_group)) < 0)
@@ -1170,7 +1212,7 @@ void read_group(char *reply, int may_ask, long first)
     res_enter_group(global.curr_group->name);
     font_enter_group();
 
-    set_busy(True, True);
+    set_busy(True);
 
     if (may_ask) {
 	set_message("Server contacted, waiting for response...", False);
@@ -1191,41 +1233,35 @@ void read_group(char *reply, int may_ask, long first)
 	clear_thread_context(main_thr);
 	reconnect_server(True);
 	unset_busy();
+    } else if (atoi(reply) == NNTP_OK_GROUP) {
+	change_interruptible(False);
+	set_message("Killing...", False);
+	mark_read_articles(main_thr, global.curr_group);
+	fix_authors(main_thr);
+	kill_articles(global.curr_group);
+	set_message("Sorting...", False);
+	sort_threads();
+	unset_busy();
+
+	sprintf(message,
+		"Group %s with %ld unread articles, "
+		"%ld killed, %ld hot.",
+		global.curr_group->name,
+		global.curr_group->no_unread,
+		global.n_killed, global.n_hot);
+	set_message(message, False);
+	setNewsModeGroup(False);
+
+	cache_enter_group();
     } else {
-	char	message[256];
-	int	status = atoi(reply);
+	unset_busy();
 
-	if (status == NNTP_OK_GROUP) {
-	    long	n_killed;
-
-	    change_interruptible(False);
-	    set_message("Killing...", False);
-	    mark_read_articles(main_thr, global.curr_group);
-	    fix_authors(main_thr);
-	    n_killed = kill_articles();
-	    set_message("Sorting...", False);
-	    sort_threads();
-	    unset_busy();
-
-	    sprintf(message,
-		    "Group %s with %ld unread articles, "
-		    "%ld killed, %ld hot.",
-		    global.curr_group->name,
-		    global.curr_group->no_unread,
-		    n_killed, global.n_hot);
-	    set_message(message, False);
-	    setNewsModeGroup(False);
-
-	    cache_enter_group();
-	} else {
-	    unset_busy();
-
-	    if (strlen(reply) > 200)
-		reply[200] = '\0';
-	    sprintf(message,
-		    "Error: message from server is %s", reply);
-	    set_message(message, True);
-	}
+	if (strlen(reply) > 200)
+	    reply[200] = '\0';
+	sprintf(message,
+		"Error: message from server is %s", reply);
+	set_message(message, True);
+	clear_thread_context(main_thr);
     }
 }
 
@@ -1240,7 +1276,7 @@ static void ask_dialogue_callback(Widget w,
 
     XtPopdown(w);
     XtDestroyWidget(w);
-    XFlush(XtDisplay(w));
+    XFlush(display);
 
     switch (report->reply) {
     case DialogueReplyRight:   /* Cancel */
@@ -1255,7 +1291,7 @@ static void ask_dialogue_callback(Widget w,
     case DialogueReplyEnter:
     case DialogueReplyTab:
 	if (sscanf(report->buffer, "%ld", &first) != 1) {
-	    XBell(XtDisplay(w), 0);
+	    XBell(display, 0);
 	    return;
 	}
 	set_message("Reading group...", False);
