@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #ifndef THREAD_H
 #define THREAD_H
@@ -25,7 +25,7 @@ extern SUBJECT		*get_subjects(THREAD_CONTEXT*);
 extern void		 set_subjects(THREAD_CONTEXT*, SUBJECT*);
 extern char		*get_refs(THREAD_CONTEXT*);
 
-extern ARTICLE	*find_article(char*, long);
+extern ARTICLE	*find_article(const char*, long);
 extern void	 read_group(char*, int, long);
 extern ARTICLE	*parse_head(long, THREAD_CONTEXT*, char*);
 extern void	 fix_author(ARTICLE*, char*, int);
