@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,21 +9,20 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #include "global.h"
 #include <X11/Shell.h>
 #include "ahead.h"
-#include "kill.h"
 #include "misc.h"
 #include "newsrc.h"
-#include "pixmaps.h"
+#include "p_menu.h"
 #include "procs.h"
-#include "psetup.h"
+#include "resource.h"
+#include "save.h"
 #include "tag.h"
 #include "thread.h"
 #include "util.h"
-#include "viewer.h"
 #include "widgets.h"
 #include "xutil.h"
 #include "../Widgets/ArtText.h"
@@ -34,96 +33,170 @@
 #include "../Widgets/MenuKnapp.h"
 #include "../Widgets/Message.h"
 #include "../Widgets/Sash.h"
+#include "../Widgets/Scrollable.h"
 #include "../Widgets/ScrBar.h"
 #include "../Widgets/ScrList.h"
 #include "../Widgets/Util.h"
 
+#include "sysdeps.h"
+
 #if HAVE_XMU && XtSpecificationRelease > 4
 #include <X11/Xmu/Editres.h>
 #endif
 
+static Widget	top_vbar, top_hbar, message_line;
+static Widget	text_hbar, text_vbar;
+
 void set_message(char *message, int beep)
 {
-    MessageSetAndRedraw(main_widgets.message_line,
-			message, beep && global.bell);
+    MessageSetAndRedraw(message_line, message, beep && global.bell);
 }
 
 #define DESCRIPTION(desc)	((desc) ? (desc) : "")
 #define ARTTREE_STEPSIZE	16
-#define LINE_LEN		128
+#define LINE_LEN		256
+
+static int	group_name_len	= 1;
+static int	subject_len	= 1;
 
 struct main_widgets main_widgets;
 
-static void print_group_info_subscr(char *buffer, GROUP *group)
+static void print_group_info_subscr(char *buffer, int len, GROUP *group)
 {
-    sprintf(buffer, "%c %-42.42s %6ld %-64.64s",
-	    group->ahead_flag ? thread_ahead_char(group) : ' ',
-	    group->name, group->no_unread,
-	    DESCRIPTION(group->description));
-}
+    int	n;
 
-static void print_group_info_all(char *buffer, GROUP *group)
-{
-    long	num;
+    *buffer++ = group->ahead_flag ? thread_ahead_char(group) : ' ';
+    *buffer++ = ' ';
+    len -= 2;
 
-    if (group->last_art == 0)
-	num = 0;
-    else {
-	num = group->last_art - group->first_art + 1;
-	if (num < 0)
-	    num = 0;
+    n = strlen(group->name);
+    if (n >= group_name_len)
+	n = group_name_len;
+    memcpy(buffer, group->name, n);
+    if (n < group_name_len)
+	memset(buffer + n, ' ', group_name_len - n);
+    buffer += group_name_len;
+    len -= group_name_len;
+
+    sprintf(buffer, "%6ld", group->no_unread);
+    n = strlen(buffer);
+    buffer += n;
+    *buffer++ = ' ';
+    len -= n + 1;
+
+    if (group->description) {
+	n = strlen(group->description);
+	if (n > len - 2)
+	    n = len - 2;
+	memcpy(buffer, group->description, n);
+	buffer += n;
     }
 
-    sprintf(buffer, "%3s %c %-42.42s %6ld %-64.64s",
-	    group->subscribed ? "[s]" : "   ",
-	    group->ahead_flag ? thread_ahead_char(group) : ' ',
-	    group->name,
-	    num < 0 ? 0 : num,
-	    DESCRIPTION(group->description));
+    *buffer = '\0';
 }
 
-static void print_subj_info(char *buffer, SUBJECT *subj)
+static void print_group_info_all(char *buffer, int len, GROUP *group)
 {
-    Boolean	indent = False;
+    long	num;
+    int		n;
+
+    num = group->last_art - group->first_art + 1;
+    if (num < 0)
+	num = 0;
+
+    memcpy(buffer, group->subscribed ? "[s] " : "    ", 4);
+    buffer += 4;
+    *buffer++ = group->ahead_flag ? thread_ahead_char(group) : ' ';
+    *buffer++ = ' ';
+    len -= 6;
+
+    n = strlen(group->name);
+    if (n >= group_name_len)
+	n = group_name_len;
+    memcpy(buffer, group->name, n);
+    if (n < group_name_len)
+	memset(buffer + n, ' ', group_name_len - n);
+    buffer += group_name_len;
+    len -= group_name_len;
+
+    sprintf(buffer, "%6ld", num);
+    n = strlen(buffer);
+    buffer += n;
+    *buffer++ = ' ';
+    len -= n + 1;
+
+    if (group->description) {
+	n = strlen(group->description);
+	if (n > len - 2)
+	    n = len - 2;
+	memcpy(buffer, group->description, n);
+	buffer += n;
+    }
+
+    *buffer = '\0';
+}
+
+static void print_subj_info(char *buffer, int len, SUBJECT *subj)
+{
+    int		indent = False;
     SUBJECT	*loop = subj->prev;
     ARTICLE	*first = first_unread_article_with_subject(subj);
-    char	*author;
+    int		n, i;
 
     if (subj->no_unread == 0)
-	sprintf(buffer, "     ");
+	strcpy(buffer, "   ");
     else
-	sprintf(buffer, "%3ld  ", subj->no_unread);
-    if (subj->has_tagged)
-	buffer[3] = '*';
+	sprintf(buffer, "%3ld", subj->no_unread);
+    n = strlen(buffer);
+    buffer += n;
+    len -= n;
+    *buffer++ = subj->has_tagged ? '*' : ' ';
+    *buffer++ = ' ';
 
-    while (loop && loop->thread == subj->thread) {
+    for (loop = subj->prev ;
+	 loop && loop->thread == subj->thread ;
+	 loop = loop->prev)
 	if (loop->disp >= 0) {
 	    indent = True;
 	    break;
 	}
-	loop = loop->prev;
-    }
 
-    if (first && first->tree_data.label)
-	author = first->tree_data.label;
-    else
-	author = "";
+    n = subject_len;
+    if (indent) {
+	*buffer++ = ' ';
+	*buffer++ = ' ';
+	len -= 2;
+	n -= 2;
+    }
 
     if (first && A_PARENT(first) && A_PARENT(first)->subject == subj) {
-	if (indent)
-	    sprintf(buffer + 5, "  Re: %-50.50s %-64.64s",
-		    subj->subject, author);
-	else
-	    sprintf(buffer + 5, "Re: %-52.52s %-64.64s",
-		    subj->subject, author);
-    } else {
-	if (indent)
-	    sprintf(buffer + 5, "  %-54.54s %-64.64s",
-		    subj->subject, author);
-	else
-	    sprintf(buffer + 5, "%-56.56s %-64.64s",
-		    subj->subject, author);
+	memcpy(buffer, "Re: ", 4);
+	buffer += 4;
+	len -= 4;
+	n -= 4;
     }
+
+    i = strlen(subj->subject);
+    if (i >= n)
+	i = n;
+    memcpy(buffer, subj->subject, i);
+    if (i < n)
+	memset(buffer + i, ' ', n - i);
+    buffer += n;
+    *buffer++ = ' ';
+    len -= subject_len + 1;
+
+    if (first && first->tree_data.label) {
+	char	*author = first->tree_data.label;
+
+	n = strlen(author);
+	if (n > len - 2)
+	    n = len - 2;
+	memcpy(buffer, author, n);
+	buffer += n;
+    }
+
+    *buffer = '\0';
 }
 
 static void knapp_set_mode(NewsMode mode)
@@ -208,6 +281,7 @@ static void knapp_set_mode(NewsMode mode)
 	XtSetArg(arg, XtNmenuName, "miscshell1");
 	break;
     case NewsModeAllgroups:
+    case NewsModeSomegroups:
 	XtSetMappedWhenManaged(main_widgets.knapp[2], True);
 	XtSetMappedWhenManaged(main_widgets.knapp[6], True);
 	XtSetMappedWhenManaged(main_widgets.knapp[7], True);
@@ -251,17 +325,17 @@ static void knapp_set_mode(NewsMode mode)
 void setNewsModeDisconnected(void)
 {
     global.mode = NewsModeDisconnected;
+    ArtTextClearLines(main_widgets.text);
+    ScrListClearLines(main_widgets.thread_list);
+    ScrListClearLines(main_widgets.group_list);
     XtUnmanageChild(main_widgets.arttree);
     XtUnmanageChild(main_widgets.thread_list);
     XtUnmanageChild(main_widgets.group_list);
     knapp_set_mode(NewsModeDisconnected);
-    ArtTextClearLines(main_widgets.text);
-    XtSetSensitive(main_widgets.top_vbar, False);
-    XtSetSensitive(main_widgets.top_hbar, False);
-    XtSetSensitive(main_widgets.text_scrbar, False);
-    ScrBarSetLengthsAndPos(main_widgets.top_vbar, 0, 0, 0);
-    ScrBarSetLengthsAndPos(main_widgets.top_hbar, 0, 0, 0);
-    ScrBarSetLengthsAndPos(main_widgets.text_scrbar, 0, 0, 0);
+    XtSetSensitive(top_vbar, False);
+    XtSetSensitive(top_hbar, False);
+    ScrBarSetLengthsAndPos(top_hbar, 0, 0, 0);
+    ScrBarSetLengthsAndPos(top_vbar, 0, 0, 0);
 }
 
 void update_group_entry(GROUP *group)
@@ -271,14 +345,15 @@ void update_group_entry(GROUP *group)
     switch (global.mode) {
     case NewsModeConnected:
 	if (group->disp >= 0) {
-	    print_group_info_subscr(buffer, group);
+	    print_group_info_subscr(buffer, sizeof buffer, group);
 	    ScrListSetLine(main_widgets.group_list, group->disp,
 			   buffer, None);
 	}
 	break;
     case NewsModeAllgroups:
+    case NewsModeSomegroups:
 	if (group->disp >= 0) {
-	    print_group_info_all(buffer, group);
+	    print_group_info_all(buffer, sizeof buffer,  group);
 	    ScrListSetLine(main_widgets.group_list, group->disp,
 			   buffer, None);
 	}
@@ -294,9 +369,12 @@ void update_group_entry(GROUP *group)
 void setNewsModeConnected(void)
 {
     Arg		args[4];
-    long	i, n, sel = -1;
-    long	shown, first;
-    Boolean	flag = False;
+    long	n, sel = -1;
+    int		flag = False;
+
+    group_name_len = res_group_name_columns();
+    if (group_name_len <= 0 || group_name_len > 128)
+	group_name_len = 128;
 
     global.mode = NewsModeConnected;
 
@@ -305,14 +383,13 @@ void setNewsModeConnected(void)
     ArtTextClearLines(main_widgets.text);
     ScrListClearLines(main_widgets.thread_list);
     ScrListClearLines(main_widgets.group_list);
-    XtSetSensitive(main_widgets.top_vbar, True);
-    XtSetSensitive(main_widgets.top_hbar, False);
-    XtSetSensitive(main_widgets.text_scrbar, False);
-    ScrBarSetLengthsAndPos(main_widgets.text_scrbar, 0, 0, 0);
-    ScrBarSetLengthsAndPos(main_widgets.top_hbar, 0, 0, 0);
+    XtSetSensitive(top_hbar, True);
+    XtSetSensitive(top_vbar, True);
     XtSetArg(args[0], XtNstepSize, 1);
-    XtSetValues(main_widgets.top_vbar, args, 1);
-    ScrBarSetLengthsAndPos(main_widgets.top_vbar, 0, 0, 0);
+    XtSetArg(args[1], XtNsensitive, True);
+    XtSetValues(top_vbar, args, 2);
+    ScrBarSetLengthsAndPos(top_hbar, 0, 0, 0);
+    ScrBarSetLengthsAndPos(top_vbar, 0, 0, 0);
     knapp_set_mode(NewsModeConnected);
     update_misc_menu(NewsModeConnected);
     XtSetArg(args[0], XtNatMostOne, True);
@@ -321,6 +398,8 @@ void setNewsModeConnected(void)
     XtSetValues(main_widgets.group_list, args, 3);
     XtManageChild(main_widgets.group_list);
 
+    ScrollableSuspend(main_widgets.group_list);
+
     for (n = 0 ; n < global.no_groups ; n++) {
 	GROUP	*group = global.groups[n];
 	char	buffer[LINE_LEN];
@@ -331,19 +410,21 @@ void setNewsModeConnected(void)
 	calc_no_unread(group);
 	if (group == global.curr_group)
 	    flag = True;
-	if (group->no_unread > 0) {
-	    print_group_info_subscr(buffer, group);
+	if (group->no_unread <= 0)
+	    group->disp = -1;
+	else {
+	    print_group_info_subscr(buffer, sizeof buffer, group);
 	    group->disp = ScrListAddLine(main_widgets.group_list,
 					 buffer, None);
 	    if (flag) {
 		sel = group->disp;
 		flag = False;
 	    }
-	} else {
-	    group->disp = -1;
 	}
     }
 
+    ScrollableResume(main_widgets.group_list);
+
     while (n < global.no_groups)
 	global.groups[n++]->disp = -1;
 
@@ -361,20 +442,11 @@ void setNewsModeConnected(void)
 	}
     }
 
-    if (sel >= 0)
+    if (sel >= 0) {
 	ScrListSetSelected(main_widgets.group_list, sel, True);
-    ScrListGetFirstShownSize(main_widgets.group_list, NULL, &shown, &i);
-    first = 0;
-    if (sel >= shown) {
-	first = sel;
-	if (first + shown >= i) {
-	    first = i - shown;
-	    if (first < 0)
-		first = 0;
-	}
-	ScrListSetFirst(main_widgets.group_list, first);
+	ScrListMakeVisible(main_widgets.group_list, sel);
     }
-    ScrBarSetLengthsAndPos(main_widgets.top_vbar, i, shown, first);
+    Remanage(main_widgets.top_manager);
 
     set_standard_message();
 }
@@ -385,33 +457,38 @@ void update_subj_entry(SUBJECT *subj)
     char	buffer[LINE_LEN];
 
     if (row >= 0) {
-	print_subj_info(buffer, subj);
-	ScrListSetLine(main_widgets.thread_list, row, buffer,
-		       global.hot_pixmaps[subj->hot]);
+	print_subj_info(buffer, sizeof buffer, subj);
+	ScrListSetLine(main_widgets.thread_list, row, buffer, subj->pixmap);
     }
 }
 
 void setNewsModeGroup(int show_all)
 {
-    Arg		args[8];
     SUBJECT	*loop;
-    long	n, shown;
+    long	n;
+    Arg		args[2];
 
-    XtSetSensitive(main_widgets.top_vbar, True);
-    XtSetSensitive(main_widgets.top_hbar, False);
-    XtSetSensitive(main_widgets.text_scrbar, True);
+    XtSetSensitive(top_hbar, True);
+    XtSetSensitive(top_vbar, True);
+    XtSetArg(args[0], XtNstepSize, 1);
+    XtSetValues(top_vbar, args, 1);
 
     if (global.mode != NewsModeThread) {
+	subject_len = res_subject_columns();
+	if (subject_len <= 0 || subject_len > 128)
+	    subject_len = 128;
+
 	global.mode = NewsModeGroup;
-	XtUnmanageChild(main_widgets.arttree);
-	XtUnmanageChild(main_widgets.group_list);
 	ScrListClearLines(main_widgets.group_list);
 	ScrListClearLines(main_widgets.thread_list);
+	XtUnmanageChild(main_widgets.arttree);
+	XtUnmanageChild(main_widgets.group_list);
 	XtManageChild(main_widgets.thread_list);
 	ArtTreeSetTree(main_widgets.arttree, NULL);
 	knapp_set_mode(NewsModeGroup);
 	update_misc_menu(NewsModeGroup);
-	ScrBarSetLengthsAndPos(main_widgets.top_hbar, 0, 0, 0);
+
+	ScrollableSuspend(main_widgets.thread_list);
 
 	for (n = 0, loop = get_subjects(main_thr) ; loop ; loop = loop->next) {
 	    char	buffer[LINE_LEN];
@@ -421,136 +498,124 @@ void setNewsModeGroup(int show_all)
 		continue;
 	    }
 
-	    print_subj_info(buffer, loop);
+	    print_subj_info(buffer, sizeof buffer, loop);
 	    loop->disp =
-		ScrListAddLine(main_widgets.thread_list, buffer,
-			       global.hot_pixmaps[loop->hot]);
+		ScrListAddLine(main_widgets.thread_list, buffer, loop->pixmap);
 	    if (n == 0)
 		global.curr_subj = loop;
 
 	    n++;
 	}
 
-	ScrListGetFirstShownSize(main_widgets.thread_list, NULL, &shown, &n);
-	XtSetArg(args[0], XtNstepSize, 1);
-	XtSetValues(main_widgets.top_vbar, args, 1);
-	ScrBarSetLengthsAndPos(main_widgets.top_vbar, n, shown, 0);
-
+	ScrollableResume(main_widgets.thread_list);
+	Remanage(main_widgets.top_manager);
 	/* message already set by get_articles */
     } else {
-	long	first;
-
 	global.mode = NewsModeGroup;
 	XtUnmanageChild(main_widgets.arttree);
 	ArtTreeSetTree(main_widgets.arttree, NULL);
 	XtManageChild(main_widgets.thread_list);
+	ScrollableResume(main_widgets.thread_list);
 	set_curr_subj(global.curr_subj);
-	ScrListGetFirstShownSize(main_widgets.thread_list, &first, &shown, &n);
-	ScrBarSetLengthsAndPos(main_widgets.top_hbar, 0, 0, 0);
-	ScrBarSetLengthsAndPos(main_widgets.top_vbar, n, shown, first);
-	XtSetArg(args[0], XtNstepSize, 1);
-	XtSetValues(main_widgets.top_vbar, args, 1);
 	knapp_set_mode(NewsModeGroup);
+
 	set_standard_message();
     }
 }
 
 void setNewsModeThread(void)
 {
-    Arg		args[8];
-    Dimension	s_w, s_h, w, h;
+    Arg		args[4];
 
     global.mode = NewsModeThread;
     XtUnmanageChild(main_widgets.group_list);
     XtUnmanageChild(main_widgets.thread_list);
     ArtTreeSetTree(main_widgets.arttree, NULL);
+    ScrollableSuspend(main_widgets.arttree);
     XtSetArg(args[0], XtNtree, global.curr_subj->thread);
-    XtSetArg(args[1], XtNsuspend, True);
-    XtSetArg(args[2], XtNx, 0);
-    XtSetArg(args[3], XtNy, 0);
-    XtSetValues(main_widgets.arttree, args, 4);
+    XtSetArg(args[1], XtNx, 0);
+    XtSetArg(args[2], XtNy, 0);
+    XtSetValues(main_widgets.arttree, args, 3);
     set_tree_stuff(global.curr_subj->thread);
     mark_tagged_articles(global.curr_subj->thread);
-    XtSetArg(args[0], XtNwidth, &w);
-    XtSetArg(args[1], XtNheight, &h);
-    XtGetValues(main_widgets.arttree, args, 2);
-    XtSetArg(args[0], XtNwidth, &s_w);
-    XtSetArg(args[1], XtNheight, &s_h);
-    XtGetValues(main_widgets.top_manager, args, 2);
-    XtSetSensitive(main_widgets.top_vbar, True);
-    XtSetSensitive(main_widgets.top_hbar, True);
-    XtSetSensitive(main_widgets.text_scrbar, True);
-    ScrBarSetLengthsAndPos(main_widgets.top_hbar, w, s_w, 0);
-    ScrBarSetLengthsAndPos(main_widgets.top_vbar, h, s_h, 0);
+
     XtSetArg(args[0], XtNstepSize, ARTTREE_STEPSIZE);
-    XtSetValues(main_widgets.top_hbar, args, 1);
-    XtSetValues(main_widgets.top_vbar, args, 1);
+    XtSetArg(args[1], XtNsensitive, True);
+    XtSetValues(top_hbar, args, 2);
+    XtSetValues(top_vbar, args, 2);
+
     if (global.curr_art) {
 	ArtTreeNodeMakeVisible(main_widgets.arttree,
 			       (ART_TREE_NODE *)global.curr_art);
 	ArtTreeNodeSetSelected(main_widgets.arttree,
 			       (ART_TREE_NODE *)global.curr_art, True);
     }
+
     XtManageChild(main_widgets.arttree);
-    XtSetArg(args[0], XtNsuspend, False);
-    XtSetValues(main_widgets.arttree, args, 1);
+    ScrollableResume(main_widgets.arttree);
     knapp_set_mode(NewsModeThread);
     set_standard_message();
 }
 
-void setNewsModeAllgroups(void)
+void setNewsModeAllgroups(regex_t *re)
 {
     Arg		args[8];
-    long	n = global.no_groups;
-    long	shown, first = -1;
+    long	n, first = -1;
 
-    global.mode = NewsModeAllgroups;
+    group_name_len = res_group_name_columns();
+    if (group_name_len <= 0 || group_name_len > 128)
+	group_name_len = 128;
 
-    XtSetArg(args[0], XtNnAlloc, global.no_groups + 4);
+    if (re)
+	global.mode = NewsModeSomegroups;
+    else
+	global.mode = NewsModeAllgroups;
+
+    XtSetArg(args[0], XtNnAlloc, global.no_groups + 8);
     XtSetValues(main_widgets.group_list, args, 1);
 
-    XtSetSensitive(main_widgets.top_vbar, True);
-    XtSetSensitive(main_widgets.top_hbar, False);
-    XtSetSensitive(main_widgets.text_scrbar, False);
+    XtSetSensitive(top_vbar, True);
+    XtSetSensitive(top_hbar, True);
 
-    XtUnmanageChild(main_widgets.arttree);
-    XtUnmanageChild(main_widgets.thread_list);
     ScrListClearLines(main_widgets.thread_list);
     ArtTextClearLines(main_widgets.text);
-    ScrBarSetLengthsAndPos(main_widgets.text_scrbar, 0, 0, 0);
-    knapp_set_mode(NewsModeAllgroups);
-    update_misc_menu(NewsModeAllgroups);
+    XtUnmanageChild(main_widgets.arttree);
+    XtUnmanageChild(main_widgets.thread_list);
+    knapp_set_mode(global.mode);
+    update_misc_menu(global.mode);
     ScrListClearLines(main_widgets.group_list);
     XtSetArg(args[0], XtNatMostOne, False);
     XtSetArg(args[1], XtNatLeastOne, False);
-    XtSetArg(args[2], XtNallowDnd, True);
+    XtSetArg(args[2], XtNallowDnd, re == NULL);
     XtSetValues(main_widgets.group_list, args, 3);
     XtManageChild(main_widgets.group_list);
     XtSetArg(args[0], XtNstepSize, 1);
-    XtSetValues(main_widgets.top_vbar, args, 1);
-    ScrBarSetLengthsAndPos(main_widgets.top_hbar, 0, 0, 0);
+    XtSetValues(top_vbar, args, 1);
+    ScrBarSetLengthsAndPos(top_hbar, 0, 0, 0);
+
+    ScrollableSuspend(main_widgets.group_list);
 
     for (n = 0 ; n < global.no_groups ; n++) {
 	GROUP	*group = global.groups[n];
 	char	buffer[LINE_LEN];
 
-	if (group == global.curr_group)
-	    first = n;
-	print_group_info_all(buffer, group);
-	group->disp =
-	    ScrListAddLine(main_widgets.group_list, buffer, None);
+	if (re && regexec(re, group->name, 0, NULL, 0) != 0)
+	    group->disp = -1;
+	else {
+	    print_group_info_all(buffer, sizeof buffer, group);
+	    group->disp =
+		ScrListAddLine(main_widgets.group_list, buffer, None);
+	    if (group == global.curr_group)
+		first = group->disp;
+	}
     }
 
-    ScrListGetFirstShownSize(main_widgets.group_list, NULL, &shown, &n);
-    if (first < 0)
-	first = 0;
-    else {
+    ScrollableResume(main_widgets.group_list);
+    if (first >= 0) {
 	ScrListSetSelected(main_widgets.group_list, first, True);
-	if (first > shown && first > n - shown)
-	    first = n - shown;
-	ScrListSetFirst(main_widgets.group_list, first);
+	ScrListMakeVisible(main_widgets.group_list, first);
     }
-    ScrBarSetLengthsAndPos(main_widgets.top_vbar, n, shown, first);
+    Remanage(main_widgets.top_manager);
 
     set_standard_message();
 }
@@ -559,7 +624,10 @@ void setNewsModeNewgroups(void)
 {
     Arg		args[8];
     long	n;
-    long	shown;
+
+    group_name_len = res_group_name_columns();
+    if (group_name_len <= 0 || group_name_len > 128)
+	group_name_len = 128;
 
     global.mode = NewsModeNewgroups;
 
@@ -568,19 +636,17 @@ void setNewsModeNewgroups(void)
 	calc_no_unread(global.groups[n]);
     }
 
-    XtSetArg(args[0], XtNnAlloc, global.no_new_groups + 4);
+    XtSetArg(args[0], XtNnAlloc, global.no_new_groups + 8);
     XtSetValues(main_widgets.group_list, args, 1);
 
-    XtSetSensitive(main_widgets.top_vbar, True);
-    XtSetSensitive(main_widgets.top_hbar, False);
-    XtSetSensitive(main_widgets.text_scrbar, False);
+    XtSetSensitive(top_vbar, True);
+    XtSetSensitive(top_hbar, True);
 
-    XtUnmanageChild(main_widgets.arttree);
-    XtUnmanageChild(main_widgets.thread_list);
     ScrListClearLines(main_widgets.thread_list);
     ScrListClearLines(main_widgets.group_list);
     ArtTextClearLines(main_widgets.text);
-    ScrBarSetLengthsAndPos(main_widgets.text_scrbar, 0, 0, 0);
+    XtUnmanageChild(main_widgets.arttree);
+    XtUnmanageChild(main_widgets.thread_list);
     knapp_set_mode(NewsModeNewgroups);
     update_misc_menu(NewsModeNewgroups);
     XtSetArg(args[0], XtNatMostOne, False);
@@ -589,20 +655,21 @@ void setNewsModeNewgroups(void)
     XtSetValues(main_widgets.group_list, args, 3);
     XtManageChild(main_widgets.group_list);
     XtSetArg(args[0], XtNstepSize, 1);
-    XtSetValues(main_widgets.top_vbar, args, 1);
-    ScrBarSetLengthsAndPos(main_widgets.top_hbar, 0, 0, 0);
+    XtSetValues(top_vbar, args, 1);
+    ScrBarSetLengthsAndPos(top_hbar, 0, 0, 0);
+
+    ScrollableSuspend(main_widgets.group_list);
 
     for (n = 0 ; n < global.no_new_groups ; n++) {
 	GROUP	*group = global.new_groups[n];
 	char	buffer[LINE_LEN];
 
-	print_group_info_all(buffer, group);
+	print_group_info_all(buffer, sizeof buffer, group);
 	group->disp = ScrListAddLine(main_widgets.group_list, buffer, None);
     }
 
-    ScrListGetFirstShownSize(main_widgets.group_list, NULL, &shown, NULL);
-    ScrBarSetLengthsAndPos(main_widgets.top_vbar,
-			   global.no_new_groups, shown, 0);
+    ScrollableResume(main_widgets.group_list);
+    Remanage(main_widgets.top_manager);
 
     set_message("New groups found.", False);
 }
@@ -615,7 +682,7 @@ void create_main_widgets(void)
 	"knapp8",	"abort",	"save",		"search",
     };
     Widget	main_layout, knapp_layout, text_layout;
-    Widget	sash;
+    Widget	temp, text_manager;
     Arg		args[10];
     int		one_win = !global.separate_windows;
     int		i;
@@ -626,8 +693,8 @@ void create_main_widgets(void)
 	    XtVaCreateManagedWidget("singlelayout", layoutWidgetClass,
 				    main_widgets.shell,
 				    XtVaTypedArg, XtNlayout, XtRString,
-#include "layouts/single.l"
-				    (int)sizeof(String), NULL);
+#include "layouts/single.h"
+				    (int)sizeof(String), (void *)0);
     } else {
 	XtSetArg(args[0], XtNinput, True);
 	XtSetArg(args[1], XtNcolormap, global.cmap);
@@ -640,8 +707,8 @@ void create_main_widgets(void)
 	    XtVaCreateManagedWidget("doublelayout", layoutWidgetClass,
 				    main_widgets.shell,
 				    XtVaTypedArg, XtNlayout, XtRString,
-#include "layouts/double.l"
-				    (int)sizeof(String), NULL);
+#include "layouts/double.h"
+				    (int)sizeof(String), (void *)0);
     }
 
     text_layout =
@@ -649,53 +716,58 @@ void create_main_widgets(void)
 				main_widgets.second_shell ?
 				main_widgets.second_shell : main_layout,
 				XtVaTypedArg, XtNlayout, XtRString,
-#include "layouts/text.l"
-				(int)sizeof(String), NULL);
+#include "layouts/text.h"
+				(int)sizeof(String), (void *)0);
 
     main_widgets.top_layout =
 	XtVaCreateManagedWidget("toplayout", layoutWidgetClass, main_layout,
 				XtVaTypedArg, XtNlayout, XtRString,
-#include "layouts/top.l"
-				(int)sizeof(String), NULL);
+#include "layouts/top.h"
+				(int)sizeof(String), (void *)0);
 
     XtSetArg(args[0], XtNcenter, False);
-    main_widgets.message_line =
+    message_line =
 	XtCreateManagedWidget("message", messageWidgetClass,
 			      main_layout, args, 1);
     knapp_layout =
 	XtVaCreateManagedWidget("knapplayout", layoutWidgetClass, main_layout,
 				XtVaTypedArg, XtNlayout, XtRString,
-#include "layouts/knapp.l"
-				(int)sizeof(String), NULL);
+#include "layouts/knapp.h"
+				(int)sizeof(String), (void *)0);
 
     main_widgets.top_manager =
 	XtCreateManagedWidget("topmanager", managerWidgetClass,
 			      main_widgets.top_layout, NULL, 0);
 
-    sash = XtCreateManagedWidget("sash1", sashWidgetClass,
+    temp = XtCreateManagedWidget("sash1", sashWidgetClass,
 				 main_layout, NULL, 0);
-    XtAddCallback(sash, XtNcallback, sash_callback, NULL);
-    sash = XtCreateManagedWidget("sash2", sashWidgetClass,
+    XtAddCallback(temp, XtNcallback, sash_callback, NULL);
+    temp = XtCreateManagedWidget("sash2", sashWidgetClass,
 				 main_layout, NULL, 0);
-    XtAddCallback(sash, XtNcallback, sash_callback, NULL);
+    XtAddCallback(temp, XtNcallback, sash_callback, NULL);
     if (one_win) {
-	sash = XtCreateManagedWidget("sash3", sashWidgetClass,
+	temp = XtCreateManagedWidget("sash3", sashWidgetClass,
 				     main_layout, NULL, 0);
-	XtAddCallback(sash, XtNcallback, sash_callback, NULL);
+	XtAddCallback(temp, XtNcallback, sash_callback, NULL);
     }
 
-    XtSetArg(args[0], XtNcoverHoriz, True);
-    XtSetArg(args[1], XtNcoverVert, True);
-    XtSetArg(args[2], XtNcontainHoriz, False);
-    XtSetArg(args[3], XtNcontainVert, False);
-    XtSetArg(args[4], XtNpixmapWidth, HOT_PIXMAP_SIZE);
-    XtSetArg(args[5], XtNpixmapHeight, HOT_PIXMAP_SIZE);
-    XtSetArg(args[6], XtNdepthOne, False);
+    top_vbar = XtCreateManagedWidget("topvbar", scrBarWidgetClass,
+				     main_widgets.top_layout, NULL, 0);
+    XtSetArg(args[0], XtNvertical, False);
+    top_hbar = XtCreateManagedWidget("tophbar", scrBarWidgetClass,
+				     main_widgets.top_layout, args, 1);
+
+    XtSetArg(args[0], XtNcontainHoriz, False);
+    XtSetArg(args[1], XtNcontainVert, False);
+    XtSetArg(args[2], XtNpixmapWidth, HOT_PIXMAP_SIZE);
+    XtSetArg(args[3], XtNpixmapHeight, HOT_PIXMAP_SIZE);
+    XtSetArg(args[4], XtNdepthOne, False);
+    XtSetArg(args[5], XtNhBar, top_hbar);
+    XtSetArg(args[6], XtNvBar, top_vbar);
     main_widgets.arttree =
 	XtCreateWidget("arttree", artTreeWidgetClass,
 		       main_widgets.top_manager, args, 7);
-    args[2].value = (XtArgVal)True;
-    args[3].value = (XtArgVal)True;
+    XtSetArg(args[1], XtNcontainVert, True);
     XtSetArg(args[7], XtNusePixmaps, False);
     main_widgets.group_list =
 	XtCreateManagedWidget("grouplist", scrListWidgetClass,
@@ -705,23 +777,23 @@ void create_main_widgets(void)
     XtSetArg(args[9], XtNatMostOne, True);
     main_widgets.thread_list =
 	XtCreateWidget("threadlist", scrListWidgetClass,
-		       main_widgets.top_manager, args, 10);
+		       main_widgets.top_manager, args, 9);
 
-    main_widgets.top_vbar =
-	XtCreateManagedWidget("topvbar", scrBarWidgetClass,
-			      main_widgets.top_layout, NULL, 0);
+    text_vbar = XtCreateManagedWidget("textvbar", scrBarWidgetClass,
+				      text_layout, NULL, 0);
     XtSetArg(args[0], XtNvertical, False);
-    XtSetArg(args[1], XtNstepSize, ARTTREE_STEPSIZE);
-    main_widgets.top_hbar =
-	XtCreateManagedWidget("tophbar", scrBarWidgetClass,
-			      main_widgets.top_layout, args, 1);
-
+    text_hbar = XtCreateManagedWidget("texthbar", scrBarWidgetClass,
+				      text_layout, args, 1);
+    text_manager =
+	XtCreateManagedWidget("textmanager", managerWidgetClass,
+			      text_layout, NULL, 0);
+    XtSetArg(args[0], XtNhBar, text_hbar);
+    XtSetArg(args[1], XtNvBar, text_vbar);
+    XtSetArg(args[2], XtNcontainVert, True);
+    XtSetArg(args[3], XtNcontainHoriz, False);
     main_widgets.text =
 	XtCreateManagedWidget("text", artTextWidgetClass,
-			      text_layout, NULL, 0);
-    main_widgets.text_scrbar =
-	XtCreateManagedWidget("textscrbar", scrBarWidgetClass,
-			      text_layout, NULL, 0);
+			      text_manager, args, 4);
 
     XtSetArg(args[0], XtNresizable, False);
     for (i = 0 ; i < 12 ; i++) {
@@ -773,27 +845,15 @@ void create_main_widgets(void)
     XtAddCallback(main_widgets.knapp[8], XtNcallback,
 		  knapp8_callback, NULL);
     XtAddCallback(main_widgets.knapp[9], XtNcallback,
-		  knapp9_callback, NULL);
+		  abort_callback, NULL);
     XtAddCallback(main_widgets.knapp[10], XtNcallback,
 		  knapp10_callback, NULL);
     XtAddCallback(main_widgets.knapp[11], XtNcallback,
 		  knapp11_callback, NULL);
-    XtAddCallback(main_widgets.top_vbar, XtNscrollCallback,
-		  top_vbar_callback, NULL);
-    XtAddCallback(main_widgets.top_hbar, XtNscrollCallback,
-		  top_hbar_callback, NULL);
-    XtAddCallback(main_widgets.text_scrbar, XtNscrollCallback,
-		  text_scrbar_callback, NULL);
-    XtAddCallback(main_widgets.top_manager, XtNresizeCallback,
-		  top_manager_callback, NULL);
-    XtAddCallback(main_widgets.text, XtNresizeCallback,
-		  text_resize_callback, NULL);
     XtAddCallback(main_widgets.text, XtNurlCallback,
 		  text_url_callback, NULL);
     XtAddCallback(main_widgets.arttree, XtNselectCallback,
 		  arttree_sel_callback, NULL);
-    XtAddCallback(main_widgets.arttree, XtNscrollCallback,
-		  arttree_scroll_callback, NULL);
     XtAddCallback(main_widgets.arttree, XtNouterCallback,
 		  arttree_tag_callback, NULL);
     XtAddCallback(main_widgets.thread_list, XtNselectCallback,
@@ -817,24 +877,22 @@ void create_main_widgets(void)
 #endif
 }
 
-void purge_hot(short hot)
+void purge_hot(Pixmap pixmap)
 {
-    Pixmap	pixmap = global.hot_pixmaps[hot];
     SUBJECT	*subj;
     ARTICLE	*art;
 
-    if (global.mode != NewsModeGroup &&
-	global.mode != NewsModeThread)
+    if (global.mode != NewsModeGroup && global.mode != NewsModeThread)
 	return;
 
     ScrListPurgePixmap(main_widgets.thread_list, pixmap);
 
     for (art = get_articles(main_thr) ; art ; art = art->next)
-	if (art->hot == hot)
-	    art->hot = 0;
+	if (art->pixmap == pixmap)
+	    art->pixmap = 0;
 
     for (subj = get_subjects(main_thr) ; subj ; subj = subj->next)
-	if (subj->hot == hot)
+	if (subj->pixmap == pixmap)
 	    update_subj_hot_value(subj);
 
     if (global.mode == NewsModeThread && global.curr_subj) {
