@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,22 +9,18 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 struct main_widgets {
     Widget	shell;
     Widget	second_shell;
-    Widget	message_line;
     Widget	top_manager;
     Widget	top_layout;
     Widget	arttree;
     Widget	group_list;
     Widget	thread_list;
-    Widget	top_hbar;
-    Widget	top_vbar;
     Widget	knapp[12];
     Widget	text;
-    Widget	text_scrbar;
 };
 
 extern struct main_widgets main_widgets;
@@ -34,9 +30,9 @@ extern void setNewsModeDisconnected(void);
 extern void setNewsModeConnected(void);
 extern void setNewsModeGroup(int);
 extern void setNewsModeThread(void);
-extern void setNewsModeAllgroups(void);
+extern void setNewsModeAllgroups(regex_t*);
 extern void setNewsModeNewgroups(void);
 extern void update_subj_entry(SUBJECT*);
 extern void update_group_entry(GROUP*);
-extern void purge_hot(short);
+extern void purge_hot(Pixmap);
 extern void set_message(char*, int);
