@@ -1,4 +1,4 @@
-/* This software is Copyright 1995 by Karl-Johan Johnsson
+/* This software is Copyright 1995, 1996 by Karl-Johan Johnsson
  *
  * Permission is hereby granted to copy, reproduce, redistribute or otherwise
  * use this software as long as: there is no monetary profit gained
@@ -9,7 +9,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. ANY USE OF THIS
- * SOFTWARE IS AT THE USER'S OWN RISK.
+ * SOFTWARE IS AT THE USERS OWN RISK.
  */
 #include "global.h"
 #include <X11/Shell.h>
@@ -18,11 +18,12 @@
 #include "cache.h"
 #include "codes.h"
 #include "connect.h"
+#include "expand.h"
 #include "file.h"
-#include "kedit.h"
-#include "kill.h"
+#include "k_file.h"
 #include "newsrc.h"
 #include "partial.h"
+#include "p_setup.h"
 #include "resource.h"
 #include "save.h"
 #include "search.h"
@@ -35,10 +36,13 @@
 #include "../Widgets/ArtTree.h"
 #include "../Widgets/Dialogue.h"
 #include "../Widgets/Knapp.h"
+#include "../Widgets/Menu.h"
+#include "../Widgets/MenuShell.h"
 #include "../Widgets/Notice.h"
-#include "../Widgets/ScrBar.h"
+#include "../Widgets/Scrollable.h"
 #include "../Widgets/ScrList.h"
 #include "../Widgets/Shadow.h"
+#include "../Widgets/StringG.h"
 #include "../Widgets/Util.h"
 
 #if HAVE_XPM
@@ -70,22 +74,8 @@ void set_curr_subj(SUBJECT *subj)
 {
     global.curr_subj = subj;
     if (subj && subj->disp >= 0) {
-	long	first, shown, size;
-	long	new = subj->disp;
-
-	ScrListSetSelected(main_widgets.thread_list, new, True);
-	ScrListGetFirstShownSize(main_widgets.thread_list,
-				 &first, &shown, &size);
-	if (new < first || first + shown <= new) {
-	    if (new + shown >= size) {
-		new = size - shown;
-		if (new < 0)
-		    new = 0;
-	    }
-	    ScrListSetFirst(main_widgets.thread_list, new);
-	    if (global.mode == NewsModeGroup)
-		ScrBarSetSliderPosition(main_widgets.top_vbar, new);
-	}
+	ScrListMakeVisible(main_widgets.thread_list, subj->disp);
+	ScrListSetSelected(main_widgets.thread_list, subj->disp, True);
     }
 }
 
@@ -95,14 +85,13 @@ void set_tree_stuff(ARTICLE *thr)
 	if (thr->from && !thr->read) {
 	    ArtTreeNodeSetInner(main_widgets.arttree,
 				(ART_TREE_NODE *)thr, True);
-	    if (thr->hot > 0)
+	    if (!thr->read && thr->pixmap != None)
 		ArtTreeNodeSetPixmap(main_widgets.arttree,
-				     (ART_TREE_NODE *)thr,
-				     global.hot_pixmaps[thr->hot]);
+				     (ART_TREE_NODE *)thr, thr->pixmap);
 	}
 	if (A_PARENT(thr) && A_PARENT(thr)->subject != thr->subject) {
 	    ArtTreeNodeSetDashed(main_widgets.arttree,
-				 (ART_TREE_NODE*)thr, True);
+				 (ART_TREE_NODE *)thr, True);
 	}
 
 	if (A_CHILD1(thr))
@@ -118,22 +107,18 @@ void set_curr_group(void)
     switch (global.mode) {
     case NewsModeConnected:
     case NewsModeAllgroups:
+    case NewsModeSomegroups:
     case NewsModeNewgroups:
 	n = ScrListGetFirstSelected(main_widgets.group_list);
 	if (!global.curr_group || global.curr_group->disp != n) {
 	    global.curr_group = NULL;
-
 	    if (n < 0)
 		break;
-
-	    for (i = 0 ; i < global.no_groups ; i++) {
-		if (!global.groups[i]->subscribed)
-		    break;
-		else if (global.groups[i]->disp == n) {
+	    for (i = 0 ; i < global.no_groups ; i++)
+		if (global.groups[i]->disp == n) {
 		    global.curr_group = global.groups[i];
 		    break;
 		}
-	    }
 	}
 	break;
     case NewsModeGroup:
@@ -143,270 +128,6 @@ void set_curr_group(void)
     }
 }
 
-int global_cleanup(int notify, int exit_ignore_errors)
-{
-    int		ret = 0;
-    long	i;
-
-    if (notify)
-	set_message("Error: Connection to server broken!", True);
-
-    if (global.mode == NewsModeGroup || global.mode == NewsModeThread) {
-	free_read_arts_list(global.curr_group);
-	global.curr_group->read_arts = create_read_arts_list();
-    }
-
-    if (global.mode != NewsModeDisconnected) {
-	if (!update_newsrc())
-	    ret |= FAILED_NEWSRC;
-	if (!update_kill_file())
-	    ret |= FAILED_KILL;
-	if (ret != 0 && !exit_ignore_errors)
-	    return ret;
-	if (!global.busy || global.quit_nicely)
-	    server_write_raw(main_server, "\r\nQUIT\r\n", 8);
-    }
-
-    server_close(main_server);
-
-    if (exit_ignore_errors)
-	exit(0);
-
-    clear_thread_context(main_thr);
-    bg_shutdown();
-    thread_ahead_shutdown();
-    cache_leave_group();
-    popdown_find_group();
-    popdown_kill_editor();
-    popdown_save();
-    popdown_search();
-    partial_clear_cache();
-    clear_tagged_articles();
-    clear_history();
-
-    free_kill_file();
-    for (i = 0 ; i < global.no_groups ; i++) {
-	XtFree(global.groups[i]->name);
-	XtFree(global.groups[i]->description);
-	free_read_arts_list(global.groups[i]);
-	XtFree((char *)global.groups[i]);
-    }
-    XtFree((char *)global.groups);
-    global.groups = NULL;
-    global.no_groups = 0;
-    global.max_groups = 0;
-    global.curr_group = NULL;
-    XtFree((char *)global.new_groups);
-    global.new_groups = NULL;
-    global.no_new_groups = 0;
-    
-    global.curr_art = NULL;
-    global.curr_subj = NULL;
-
-    setNewsModeDisconnected();
-
-    return ret;
-}
-
-void change_interruptible(int interruptible)
-{
-    XtSetSensitive(main_widgets.knapp[9], interruptible);
-    ShadowRedrawWidget(main_widgets.knapp[9]);
-}
-
-void set_busy(int interruptible, int quit_nicely)
-{
-    Display	*disp = XtDisplay(main_widgets.shell);
-
-    if (global.busy) {
-	fprintf(stderr, "warning: set_busy when already busy!\n");
-	return;
-    }
-
-    global.busy = True;
-    global.quit_nicely = quit_nicely;
-    global.aborted = False;
-
-    XDefineCursor(disp, XtWindow(main_widgets.shell), global.busy_cursor);
-    if (main_widgets.second_shell)
-	XDefineCursor(disp, XtWindow(main_widgets.second_shell),
-		      global.busy_cursor);
-    set_busy_kill(True);
-    set_busy_save(True);
-    set_busy_search(True);
-
-    if (interruptible) {
-	XtSetSensitive(main_widgets.knapp[9], True);
-	ShadowRedrawWidget(main_widgets.knapp[9]);
-    }
-
-    KnappSetActive(main_widgets.knapp[0], False);
-    KnappSetActive(main_widgets.knapp[1],
-		   global.mode == NewsModeGroup ||
-		   global.mode == NewsModeThread);
-    KnappSetActive(main_widgets.knapp[2], False);
-    KnappSetActive(main_widgets.knapp[3], False);
-    KnappSetActive(main_widgets.knapp[4], False);
-    KnappSetActive(main_widgets.knapp[6], False);
-    KnappSetActive(main_widgets.knapp[7], False);
-    KnappSetActive(main_widgets.knapp[8], False);
-    KnappSetActive(main_widgets.knapp[11], global.mode != NewsModeAllgroups);
-    ArtTreeSetActive(main_widgets.arttree, False);
-    ScrListSetActive(main_widgets.group_list, False);
-    ScrListSetActive(main_widgets.thread_list, False);
-
-    XFlush(XtDisplay(main_widgets.shell));
-}
-
-void unset_busy(void)
-{
-    Display	*disp = XtDisplay(main_widgets.shell);
-
-    if (!global.busy) {
-	fprintf(stderr, "warning: unset_busy when not busy\n");
-	return;
-    }
-
-    global.busy = False;
-    global.quit_nicely = False;
-    global.aborted = False;
-
-    XDefineCursor(disp, XtWindow(main_widgets.shell), global.cursor);
-    if (main_widgets.second_shell)
-	XDefineCursor(disp, XtWindow(main_widgets.second_shell),
-		      global.cursor);
-    set_busy_kill(False);
-    set_busy_save(False);
-    set_busy_search(False);
-
-    XtSetSensitive(main_widgets.knapp[9], False);
-    ShadowRedrawWidget(main_widgets.knapp[9]);
-
-    KnappSetActive(main_widgets.knapp[0], True);
-    KnappSetActive(main_widgets.knapp[1], True);
-    KnappSetActive(main_widgets.knapp[2], True);
-    KnappSetActive(main_widgets.knapp[3], True);
-    KnappSetActive(main_widgets.knapp[4], True);
-    KnappSetActive(main_widgets.knapp[6], True);
-    KnappSetActive(main_widgets.knapp[7], True);
-    KnappSetActive(main_widgets.knapp[8], True);
-    KnappSetActive(main_widgets.knapp[11], True);
-    ArtTreeSetActive(main_widgets.arttree, True);
-    ScrListSetActive(main_widgets.group_list, True);
-    ScrListSetActive(main_widgets.thread_list, True);
-
-    XFlush(disp);
-}
-
-void set_standard_message(void)
-{
-    char	message[512];
-    long	n;
-
-    switch (global.mode) {
-    case NewsModeDisconnected:
-	set_message("Knews " KNEWS_VERSION ".   "
-		    "Copyright 1996 Karl-Johan Johnsson.", False);
-	break;
-    case NewsModeConnected:
-	if (global.groups && global.groups[0] &&
-	    global.groups[0]->subscribed)
-	    set_message("Subscribed groups "
-			"with unread articles displayed.",
-			False);
-	else
-	    set_message("No subscribed groups.", False);
-	break;
-    case NewsModeGroup:
-	ScrListGetFirstShownSize(main_widgets.thread_list, NULL, NULL, &n);
-	sprintf(message,
-		"%s with %ld unread articles, %ld subjects.  %ld hot.",
-		global.curr_group->name, global.curr_group->no_unread,
-		n, global.n_hot);
-	set_message(message, False);
-	break;
-    case NewsModeThread:
-	if (!global.curr_subj)
-	    return;
-	sprintf(message, "%ld unread articles with Subject: ",
-		global.curr_subj->no_unread);
-	strncat(message, global.curr_subj->subject, 128);
-	set_message(message, False);
-	break;
-    case NewsModeAllgroups:
-	set_message("All groups displayed.", False);
-	break;
-    case NewsModeNewgroups:
-	set_message("New groups displayed.", False);
-	break;
-    }
-}
-
-void stderr_popup(char *stderr_buf, long timeout)
-{
-    if (timeout < 0)
-	timeout = global.stderr_timeout;
-
-    if (stderr_buf && stderr_buf[0] != '\0' && timeout >= 0)
-	popup_notice("stderr", stderr_buf, "Ok", NULL, NULL,
-		     timeout, NULL, NULL, XtGrabNone);
-}
-
-/*****************************************************************************/
-
-static void quit_verify_callback(Widget w,
-				 XtPointer client_data,
-				 XtPointer call_data)
-{
-    NoticeReply	reply = (NoticeReply)call_data;
-
-    XtPopdown(w);
-    XtDestroyWidget(w);
-
-    switch (reply) {
-    case NoticeReplyTimeout:
-    case NoticeReplyRight:
-    case NoticeReplyClose:  /* abort */
-	break;
-    case NoticeReplyLeft:   /* quit (anyway) */
-	quit(False);
-	break;
-    case NoticeReplyMiddle: /* retry */
-	quit(True);
-	break;
-    }
-}
-
-void quit(int ask)
-{
-    char	*msg;
-    int		status;
-
-    if (ask && global.confirm_quit) {
-	popup_notice("quitverify", "Really quit?", "Quit", NULL, "Abort", 0,
-		     quit_verify_callback, NULL, XtGrabExclusive);
-	return;
-    }
-
-    if (global.mode == NewsModeDisconnected)
-	exit(0);
-
-    status = global_cleanup(False, False);
-    if (status == 0)
-	exit(0);
-
-    if (status == FAILED_NEWSRC)
-	msg = "Failed to update newsrc file!\n\nReally quit?";
-    else if (status == FAILED_KILL)
-	msg = "Failed to update kill file!\n\nReally quit?";
-    else
-	msg = "Failed to update newsrc and kill files!\n\nReally quit?";
-
-    popup_notice("quitverify", msg, "Quit", "Retry", "Abort", 0,
-		 quit_verify_callback, NULL, XtGrabExclusive);
-    XBell(XtDisplay(main_widgets.shell), 0);
-}
-
 void realize_fake(ARTICLE *art, char **headers, int n)
 {
     if (art->from || art->tree_data.label)
@@ -432,8 +153,312 @@ void realize_fake(ARTICLE *art, char **headers, int n)
 
 /*****************************************************************************/
 
+void change_interruptible(int interruptible)
+{
+    XtSetSensitive(main_widgets.knapp[9], interruptible);
+    ShadowRedrawWidget(main_widgets.knapp[9]);
+}
+
+void set_busy(int interruptible)
+{
+    if (global.busy) {
+	fputs("knews: internal error: set_busy when already busy!\n", stderr);
+	return;
+    }
+
+    global.busy = True;
+
+    XDefineCursor(display, XtWindow(main_widgets.shell), global.busy_cursor);
+    if (main_widgets.second_shell)
+	XDefineCursor(display, XtWindow(main_widgets.second_shell),
+		      global.busy_cursor);
+    set_busy_save(True);
+    set_busy_search(True);
+
+    if (interruptible)
+	KnappSetSensitive(main_widgets.knapp[9], True);
+
+    KnappSetActive(main_widgets.knapp[0], False);
+    KnappSetActive(main_widgets.knapp[1],
+		   global.mode == NewsModeGroup ||
+		   global.mode == NewsModeThread);
+    KnappSetActive(main_widgets.knapp[2], False);
+    KnappSetActive(main_widgets.knapp[3], False);
+    KnappSetActive(main_widgets.knapp[4], False);
+    KnappSetActive(main_widgets.knapp[6], False);
+    KnappSetActive(main_widgets.knapp[7], False);
+    KnappSetActive(main_widgets.knapp[8], False);
+    KnappSetActive(main_widgets.knapp[11], global.mode != NewsModeAllgroups);
+    ArtTreeSetActive(main_widgets.arttree, False);
+    ScrListSetActive(main_widgets.group_list, False);
+    ScrListSetActive(main_widgets.thread_list, False);
+
+    XFlush(display);
+}
+
+void unset_busy(void)
+{
+    if (!global.busy) {
+	fputs("knews: internal error: unset_busy when not busy\n", stderr);
+	return;
+    }
+
+    global.busy = False;
+
+    XDefineCursor(display, XtWindow(main_widgets.shell), global.cursor);
+    if (main_widgets.second_shell)
+	XDefineCursor(display, XtWindow(main_widgets.second_shell),
+		      global.cursor);
+    set_busy_save(False);
+    set_busy_search(False);
+
+    KnappSetSensitive(main_widgets.knapp[9], False);
+
+    KnappSetActive(main_widgets.knapp[0], True);
+    KnappSetActive(main_widgets.knapp[1], True);
+    KnappSetActive(main_widgets.knapp[2], True);
+    KnappSetActive(main_widgets.knapp[3], True);
+    KnappSetActive(main_widgets.knapp[4], True);
+    KnappSetActive(main_widgets.knapp[6], True);
+    KnappSetActive(main_widgets.knapp[7], True);
+    KnappSetActive(main_widgets.knapp[8], True);
+    KnappSetActive(main_widgets.knapp[11], True);
+    ArtTreeSetActive(main_widgets.arttree, True);
+    ScrListSetActive(main_widgets.group_list, True);
+    ScrListSetActive(main_widgets.thread_list, True);
+
+    XFlush(display);
+}
+
+void set_standard_message(void)
+{
+    char	message[512];
+    long	n;
+
+    switch (global.mode) {
+    case NewsModeDisconnected:
+	set_message("Knews " KNEWS_VERSION ".   "
+		    "Copyright 1995, 1996 Karl-Johan Johnsson.", False);
+	break;
+    case NewsModeConnected:
+	if (global.groups && global.groups[0] &&
+	    global.groups[0]->subscribed)
+	    set_message("Subscribed groups "
+			"with unread articles displayed.",
+			False);
+	else
+	    set_message("No subscribed groups.", False);
+	break;
+    case NewsModeGroup:
+	n = ScrollableGetVSize(main_widgets.thread_list);
+	sprintf(message,
+		"%s with %ld unread articles, %ld subjects.  %ld hot.",
+		global.curr_group->name, global.curr_group->no_unread,
+		n, global.n_hot);
+	set_message(message, False);
+	break;
+    case NewsModeThread:
+	if (!global.curr_subj)
+	    return;
+	sprintf(message, "%ld unread articles with Subject: ",
+		global.curr_subj->no_unread);
+	strncat(message, global.curr_subj->subject, 128);
+	set_message(message, False);
+	break;
+    case NewsModeAllgroups:
+	set_message("All groups displayed.", False);
+	break;
+    case NewsModeSomegroups:
+	set_message("Groups matching regexp displayed.", False);
+	break;
+    case NewsModeNewgroups:
+	set_message("New groups displayed.", False);
+	break;
+    }
+}
+
+/*****************************************************************************/
+
+char *do_update(void)
+{
+    int	newsrc_ok = update_newsrc();
+    int	kill_ok   = update_kill_files();
+
+    if (newsrc_ok)
+	if (kill_ok)
+	    return NULL;
+	else
+	    return "Failed to update kill files!   Newsrc file updated.";
+    else
+	if (kill_ok)
+	    return "Failed to update newsrc!   Kill files updated.";
+	else
+	    return "Failed to update newsrc and kill files!";
+}
+
+static void update_failed_callback(Widget w,
+				  XtPointer client_data,
+				  XtPointer call_data)
+{
+    NoticeReply	reply = (NoticeReply)call_data;
+    int		allow_abort = (int)client_data;
+
+    XtPopdown(w);
+    XtDestroyWidget(w);
+
+    switch (reply) {
+    case NoticeReplyLeft:   /* Retry */
+	global_cleanup(True, allow_abort);
+	break;
+    case NoticeReplyMiddle: /* Disconnect */
+	global_cleanup(False, allow_abort);
+	break;
+    case NoticeReplyTimeout:
+    case NoticeReplyRight:
+    case NoticeReplyClose:  /* abort */
+	if (!allow_abort)
+	    exit(1);
+	break;
+    }
+}
+
+int global_cleanup(int try_to_update, int allow_abort)
+{
+    QuitFunc	qf = main_server ? server_get_quit_func(main_server) : NULL;
+    long	i;
+    char	*err_msg;
+
+    if (global.mode == NewsModeGroup || global.mode == NewsModeThread) {
+	free_read_arts_list(global.curr_group);
+	global.curr_group->read_arts = create_read_arts_list();
+    }
+
+    if (global.mode == NewsModeDisconnected)
+	return True;
+
+    if (try_to_update && (err_msg = do_update())) {
+	char	message[128];
+
+	set_message(err_msg, True);
+	sprintf(message, "%s\n\nReally disconnect?", err_msg);
+	popup_notice("verify", message, "Retry", "Disconnect",
+		     allow_abort ? "Abort" : NULL, 0,
+		     update_failed_callback, (XtPointer)allow_abort,
+		     XtGrabExclusive);
+	return False;
+    }
+
+    if (qf)
+	qf(main_server);
+
+    server_close(main_server);
+    XtFree(global.serv_addr);
+    global.serv_addr = NULL;
+
+    clear_thread_context(main_thr);
+    bg_shutdown();
+    thread_ahead_shutdown();
+    cache_leave_group();
+    popdown_find_group();
+    kill_cleanup();
+    popdown_save();
+    popdown_search();
+    partial_clear_cache();
+    clear_tagged_articles();
+    clear_history();
+
+    for (i = 0 ; i < global.no_groups ; i++) {
+	XtFree(global.groups[i]->name);
+	XtFree(global.groups[i]->description);
+	free_read_arts_list(global.groups[i]);
+	XtFree((char *)global.groups[i]);
+    }
+    XtFree((char *)global.groups);
+    global.groups = NULL;
+    global.no_groups = 0;
+    global.max_groups = 0;
+    global.curr_group = NULL;
+    XtFree((char *)global.new_groups);
+    global.new_groups = NULL;
+    global.no_new_groups = 0;
+
+    global.curr_art = NULL;
+    global.curr_subj = NULL;
+
+    setNewsModeDisconnected();
+
+    return True;
+}
+
+static void disconnect_verify_callback(Widget w,
+				       XtPointer client_data,
+				       XtPointer call_data)
+{
+    NoticeReply	reply = (NoticeReply)call_data;
+    long	do_exit = (long)client_data;
+
+    XtPopdown(w);
+    XtDestroyWidget(w);
+
+    if (reply == NoticeReplyLeft && global_cleanup(True, True) && do_exit)
+	exit(0);
+
+    set_standard_message();
+}
+
+static void have_posts_callback(Widget w,
+				XtPointer client_data,
+				XtPointer call_data)
+{
+    NoticeReply	reply = (NoticeReply)call_data;
+    long	do_exit = (long)client_data;
+
+    XtPopdown(w);
+    XtDestroyWidget(w);
+
+    if (reply == NoticeReplyLeft)
+	if (do_exit)
+	    exit(0);
+	else if (global_cleanup(True, True))
+	    set_standard_message();
+}
+
+void disconnect(int do_exit)
+{
+    if (outstanding_posts()) {
+	char	message[128];
+
+	sprintf(message, "You have not posted all articles!\n\n%s anyway?",
+		do_exit ? "Quit" : "Disconnect");
+	popup_notice("verify", message,
+		     do_exit ? "Quit" : "Disconnect", NULL, "Abort", 0,
+		     have_posts_callback, (XtPointer)do_exit,
+		     XtGrabExclusive);
+	return;
+    }
+
+    if (global.confirm_quit) {
+	popup_notice("disconnectverify",
+		     do_exit ? "Really quit?" : "Really disconnect?",
+		     do_exit ? "Quit" : "Disconnect", NULL, "Abort", 0,
+		     disconnect_verify_callback, (XtPointer)do_exit,
+		     XtGrabExclusive);
+	return;
+    }
+
+    if (global.mode == NewsModeDisconnected)
+	exit(0);
+    if (!global_cleanup(True, True))
+	return;
+    if (do_exit)
+	exit(0);
+    set_standard_message();
+}
+
+/*****************************************************************************/
+
 static XtIntervalId	rescan_timeout_id = 0;
-static Boolean		rescan_due = False;
+static int		rescan_due = False;
 
 static void rescan_timeout_callback(XtPointer client_data, XtIntervalId *id)
 {
@@ -449,7 +474,7 @@ static void rescan_timeout_callback(XtPointer client_data, XtIntervalId *id)
 	return; /* no op */
     }
 
-    set_busy(True, True);
+    set_busy(True);
     set_message("Automatic rescan in progress...", False);
     buffer = rescan();
     if (!buffer) {
@@ -588,8 +613,7 @@ Boolean cvt_string_to_pixmap(Display *disp,
 
 #ifdef XpmColorError
     if (tmp == XpmColorError)
-	fprintf(stderr, "XpmReadFileToPixmap warning %s: color error\n",
-		path);
+	fprintf(stderr, "XpmReadFileToPixmap warning %s: color error\n", path);
 #endif
 
     XtFree(path);
@@ -614,58 +638,70 @@ static void notice_callback(Widget w,
     XtDestroyWidget(w);
 }
 
-Widget popup_notice(char *name, char *message,
-		    char *left, char *middle, char *right, long timeout,
-		    XtCallbackProc callback, XtPointer client_data,
-		    XtGrabKind grab)
+Widget popup_notice(char		*name,
+		    char		*message,
+		    char		*left,
+		    char		*middle,
+		    char		*right,
+		    long		 timeout,
+		    XtCallbackProc	 callback,
+		    XtPointer		 client_data,
+		    XtGrabKind		 grab)
 {
     Widget	result;
     Arg		args[10];
+    int		n;
 
-    XtSetArg(args[0], XtNmessage, message);
-    XtSetArg(args[1], XtNleftLabel, left);
-    XtSetArg(args[2], XtNmiddleLabel, middle);
-    XtSetArg(args[3], XtNrightLabel, right);
-    XtSetArg(args[4], XtNtimeout, timeout);
-    XtSetArg(args[6], XtNcolormap, global.cmap);
-    XtSetArg(args[5], XtNvisual, global.visual);
-    XtSetArg(args[7], XtNdepth, global.depth);
-    XtSetArg(args[8], XtNtransientFor, main_widgets.shell);
+    n = 0;
+    XtSetArg(args[n], XtNmessage, message);                 n++;
+    XtSetArg(args[n], XtNleftLabel, left);                  n++;
+    XtSetArg(args[n], XtNmiddleLabel, middle);              n++;
+    XtSetArg(args[n], XtNrightLabel, right);                n++;
+    XtSetArg(args[n], XtNtimeout, timeout);                 n++;
+    XtSetArg(args[n], XtNcolormap, global.cmap);            n++;
+    XtSetArg(args[n], XtNvisual, global.visual);            n++;
+    XtSetArg(args[n], XtNdepth, global.depth);              n++;
+    XtSetArg(args[n], XtNtransientFor, main_widgets.shell); n++;
 
     result =
 	XtCreatePopupShell(name, noticeWidgetClass,
-			   main_widgets.shell, args, 9);
+			   main_widgets.shell, args, n);
     XtRealizeWidget(result);
-    if (callback)
-	XtAddCallback(result, XtNcallback, callback, client_data);
-    else
-	XtAddCallback(result, XtNcallback, notice_callback, NULL);
+    if (!callback)
+	callback = notice_callback;
+    XtAddCallback(result, XtNcallback, callback, client_data);
 
     popup_under_pointer(result, grab);
 
     return result;
 }
 
-Widget popup_dialogue(char *name, char *message,
-		      char *left, char *middle, char *right,
-		      XtCallbackProc callback, XtPointer client_data,
-		      XtGrabKind grab)
+Widget popup_dialogue(char		*name,
+		      char		*message,
+		      char		*left,
+		      char		*middle,
+		      char		*right,
+		      XtCallbackProc	 callback,
+		      XtPointer		 client_data,
+		      XtGrabKind	 grab)
 {
     Widget	result;
     Arg		args[10];
+    int		n;
 
-    XtSetArg(args[0], XtNmessage, message);
-    XtSetArg(args[1], XtNleftLabel, left);
-    XtSetArg(args[2], XtNmiddleLabel, middle);
-    XtSetArg(args[3], XtNrightLabel, right);
-    XtSetArg(args[4], XtNcolormap, global.cmap);
-    XtSetArg(args[5], XtNvisual, global.visual);
-    XtSetArg(args[6], XtNdepth, global.depth);
-    XtSetArg(args[7], XtNtransientFor, main_widgets.shell);
+    n = 0;
+    XtSetArg(args[n], XtNmessage, message);                 n++;
+    XtSetArg(args[n], XtNleftLabel, left);                  n++;
+    XtSetArg(args[n], XtNmiddleLabel, middle);              n++;
+    XtSetArg(args[n], XtNrightLabel, right);                n++;
+    XtSetArg(args[n], XtNcolormap, global.cmap);            n++;
+    XtSetArg(args[n], XtNvisual, global.visual);            n++;
+    XtSetArg(args[n], XtNdepth, global.depth);              n++;
+    XtSetArg(args[n], XtNtransientFor, main_widgets.shell); n++;
 
     result =
 	XtCreatePopupShell(name, dialogueWidgetClass,
-			   main_widgets.shell, args, 8);
+			   main_widgets.shell, args, n);
     XtRealizeWidget(result);
     if (callback)
 	XtAddCallback(result, XtNcallback, callback, client_data);
@@ -678,7 +714,7 @@ Widget popup_dialogue(char *name, char *message,
 void popup_colornotice(int bad_color)
 {
     if (bad_color && global.bell)
-	XBell(XtDisplay(main_widgets.shell), 0);
+	XBell(display, 0);
     popup_notice("colornotice",
 		 bad_color ? "Error: No such color." :
 		 "Warning: Cannot allocate colormap entry.",
@@ -701,15 +737,60 @@ void popup_regexpnotice(int errcode, const regex_t *re)
     
 }
 
-void popup_filenotice(char *msg, char *path, int beep)
+void popup_title_notice(char *title, char *body, int beep)
 {
     char	buffer[1024];
 
-    strcpy(buffer, msg);
-    strcat(buffer, ":\n\n");
-    strncat(buffer, path, 500);
-    popup_notice("filenotice", buffer, "OK", NULL, NULL, 10000,
+    if (!title)
+	buffer[0] = '\0';
+    else {
+	strcpy(buffer, title);
+	strcat(buffer, ":\n\n");
+    }
+    strncat(buffer, body, 500);
+    popup_notice("notice", buffer, "OK", NULL, NULL, 10000,
 		 NULL, NULL, XtGrabNone);
     if (beep && global.bell)
-	XBell(XtDisplay(main_widgets.shell), 0);
+	XBell(display, 0);
 }
+
+void stderr_popup(char *stderr_buf, long timeout)
+{
+    if (timeout < 0)
+	timeout = global.stderr_timeout;
+
+    if (stderr_buf && stderr_buf[0] != '\0' && timeout >= 0)
+	popup_notice("stderr", stderr_buf, "Ok", NULL, NULL,
+		     timeout, NULL, NULL, XtGrabNone);
+}
+
+/*************************************************************************/
+
+Widget create_simple_menu(Widget parent, char *prefix, int size,
+			  XtCallbackProc callback, void *client_data)
+{
+    char	name[128];
+    Widget	shell, menu, temp;
+    Arg		args[4];
+    int		i;
+
+    if (size <= 0)
+	return NULL;
+
+    XtSetArg(args[0], XtNcolormap, global.cmap);
+    XtSetArg(args[1], XtNvisual, global.visual);
+    XtSetArg(args[2], XtNdepth, global.depth);
+    sprintf(name, "%sshell", prefix);
+    shell = XtCreatePopupShell(name, menuShellWidgetClass, parent, args, 3);
+    sprintf(name, "%smenu", prefix);
+    menu = XtCreateManagedWidget(name, menuWidgetClass, shell, NULL, 0);
+
+    for (i = 0 ; i < size ; i++) {
+	sprintf(name, "%s%d", prefix, i);
+	temp = MenuCreateGadget(name, stringGadgetClass, menu, NULL, 0);
+	XtAddCallback(temp, XtNcallback, callback, client_data);
+    }
+
+    return shell;
+}
+
